/*----------- Function -------------------------------------------------

   AddObSpectra
   
------------- Description ----------------------------------------------

   This is a wrapper function to handle all of the on-board spectra
   analysis and binning which has been added to spihist.   All memory
   that is dynamically allocated at this level and below should be
   cleaned up prior to returning from here.
   
   NOTE:  This function makes use of the SPIGain class defined in the
          spi_toolslib library, and therefore requires C++ compilation.
   
------------------------------------------------------------------------
   
   Input Parameters:
   	COEFFDOL	char*		:DOL of calibration table for
					 energy/channel conversion.
	PBIN		binningPtr	:Contains the energy bin boundaries.
	POINTINGNUM	int	  	:Index of pointing.
        PDITHER	        pointingInfoPtr :Pointing information.
        GTLIST		gtIntPtr **	:List containing all gti structs.
	
   Output Parameters:
   	PDET		detinfoPtr	:Contains histogram with ob spectra
					 newly added.
   Return value:
        ISDC_OK if no errors, OR if error prevents processing of ANY ob
		spectra, in which case calling function may still proceed.
	-1 if error occurs after ob spectra binning has begun.  Calling
	    function will most likely want to treat this as a fatal error.
	    
------------- Authors/Modification History: ----------------------------

                        (3.1.0; Jun 03) ;Using different DAL calls to
                                         get OB times and select SWG in
                                         attempt to increase speed.
		        (3.0.4; Feb 02) :Turn off warning when counts are
					 outside of ebounds.
	Craig Gordon 	(3.0.0; Dec 02)

------------------------------------------------------------------------
*/ 

#include "spihist.h"
#include "spi_toolslib.h"

int AddObSpectra(dal_element *og, char *coeffDOL, binningPtr pBin, int pointingNum,
			detinfoPtr pDet, outputParsPtr pOutput, 
			pointingInfoPtr pDither, gtIntPtr **gtList)
{
   
   int i, j, k, nFound=0, proceed=0, status = ISDC_OK;
   int swgUsed = 0;
   long nSwg = 0;
   long nSpectra=0;
   long *addCounts=0;
   long total1=0, total2=0;
   unsigned short *pha=0, tmpPha;
   unsigned char rangeFlag = 0;
   obSpectrumPtr *pSpectra=0;
   obSpectrumPtr currSpec=0;
   dal_element *swg=0;
   dal_element *swgIdx=0;
   dal_element *calib=0;
   char msg[DAL_MAX_STRING];
   char selectStr[12];
   char program[]="AddObSpectra";
   OBTime *obtStart=0;
   OBTime *obtEnd=0;
   OBTime obMean;
   SPIGain gain;
   
   extern const int gl_boundaries[2][6];
   extern const short gl_emer;
   extern const short gl_oper;
   
   strcpy(msg, "Processing OB spectra....\n");
   DispMsg(Log_1, msg, status, program, pOutput->chatter);
   
   status = DALobjectFindElement(og, "GNRL-SCWG-GRP-IDX", &swgIdx, status);
   if (status != ISDC_OK)
   {
      strcpy(msg, "Error -- SWG Index is missing.  No OB spectra processed");
      DispMsg(Error_2, msg, status, program, pOutput->chatter);
      return ISDC_OK;
   }
   status = DALtableGetNumRows(swgIdx, &nSwg, status);
   if (status != ISDC_OK)
   {
      strcpy(msg, "Error -- No SWGs found in index table.  No OB spectra processed");
      DispMsg(Error_2, msg, status, program, pOutput->chatter);
      return ISDC_OK;
   }
   
   status = DAL3GENopenISDCGroup(coeffDOL, &calib, status);
   if (status != ISDC_OK)
   {
      strcpy(msg, "Unable to open gain calibration file.  No OB spectra processed");
      DispMsg(Error_2, msg, status, program, pOutput->chatter);
      return ISDC_OK;
   }
   status = gain.set(calib);
   if (status != ISDC_OK)
   {
      strcpy(msg, "Error returned from SPIGain::set.  No OB spectra processed.");
      DispMsg(Error_2, msg, status, program, pOutput->chatter);
      DALobjectClose(calib, DAL_SAVE, ISDC_OK);
      return ISDC_OK;
   }
   
   DALallocateDataBuffer((void **)&obtStart, nSwg*sizeof(OBTime), status);
   DALallocateDataBuffer((void **)&obtEnd, nSwg*sizeof(OBTime), status);
   status = DAL3GENtableGetOBT(swgIdx, "OBTSTART", NULL, NULL, obtStart, status);
   status = DAL3GENtableGetOBT(swgIdx, "OBTEND", NULL, NULL, obtEnd, status); 
   if (status != ISDC_OK)
   {
      strcpy(msg, "Error while attempting to read OBT values from SWG index. \
                   \nNo OB spectra processed.");
      DispMsg(Error_2, msg, status, program, pOutput->chatter);
      DALobjectClose(calib, DAL_SAVE, ISDC_OK);
      DALfreeDataBuffer(obtStart, ISDC_OK);
      DALfreeDataBuffer(obtEnd, ISDC_OK);
      return ISDC_OK;
   }  
   
   i=0;
   while (i < nSwg) 
   {
      proceed = 0;
      obMean = (obtStart[i] + obtEnd[i])/2;
      if (obMean >= pDither->obStart[pointingNum] &&
      		obMean <= pDither->obStop[pointingNum])
      {
         status = DALelementGetChild(swgIdx, i+1, &swg, NULL, NULL, status);
         if (status != ISDC_OK)
         {
            strcpy(msg, "Error getting SWG from SWG index table. No OB spectra processed.");
            DispMsg(Error_2, msg, status, program, pOutput->chatter);
            status = ISDC_OK;
            break;
         }
	 status = GetObSpectra(swg, pDet, &pSpectra, &nSpectra, pOutput);
	 if (status == 1)
	 {
	    status = ISDC_OK;
	 }
	 else if (status != ISDC_OK)
	 {
            strcpy(msg, "Error in function GetOBSpectra.  No OB spectra processed");
	    DispMsg(Error_2, msg, status, program, pOutput->chatter);
	    status = ISDC_OK;
	    break;
	 }
	 else
	 {
	    ++swgUsed;
	 }
      }
      proceed = 1;
      ++i;
   }
   DALfreeDataBuffer(obtStart,ISDC_OK);
   DALfreeDataBuffer(obtEnd,ISDC_OK);
    
   if (proceed && nSpectra>0)
   {
      sprintf(msg, "Pointing contains %d rows of OB spectra in %d Scwg(s).",
      		nSpectra, swgUsed);
      DispMsg(Log_1, msg, status, program, pOutput->chatter);
      DALallocateDataBuffer((void**)&pha, pBin->nEnergy*sizeof(unsigned short),
      			status);
      DALallocateDataBuffer((void**)&addCounts, (pBin->nEnergy-1)*
      			sizeof(long), status);
      /* Loop through dets i = 0-18 */
      for (i=0; proceed && i< gl_boundaries[0][0]; ++i)
      {
         /* Check if either det i or its corresponding det 85-103 requires
	    an ob spectra. */
	 if (pDet->obSpectraFlag[i] || pDet->obSpectraFlag[i+SPISTRUCT_NDETS])
	 {
	    /* Look for any ob spectra with det_id = detector i */
	    for (j=0; proceed && j<nSpectra; ++j)
	    {
	       currSpec = pSpectra[j];
	       if ((int)(currSpec->det_id) == i)
	       { /* Process the found ob spectrum */
	          obMean = (currSpec->obt_start + currSpec->obt_end)/2;
		  rangeFlag = 0;
		  for (k=0; k<pBin->nEnergy; ++k)
		  {		     
	             status = gain.eng2pha(obMean, currSpec->det_id, 
		     		rangeFlag, pBin->energy[k], &tmpPha);
		     if (status == SPI_TOOLSLIB_ERROR_OUT_OF_RANGE)
		     {
		        /* Change rangeFlag to high range and try again */
			rangeFlag = 1;
			status = gain.eng2pha(obMean, currSpec->det_id, 
		     		rangeFlag, pBin->energy[k], &tmpPha);		        
		     }
		     if (status != ISDC_OK)
		     {
        		strcpy(msg, "Error during eng->pha gain correction.\n");
			strcat(msg, "OB spectra processing will stop.");
			DispMsg(Error_2, msg, status, program, pOutput->chatter);
			proceed = 0;
			status = -1;
			break;
		     }
		     pha[k] = rangeFlag ? tmpPha+SPIHIST_NCHANS : tmpPha;
		  } /* End energy bins loop */
		  if (proceed)
		  {
		     status = Rebin(currSpec->counts, currSpec->e_range, pha, 
		     			pBin->nEnergy-1, addCounts);
		  /*   if (status == -1)
		     {
		        sprintf(msg,"Warning - Some counts are outside of ebounds array in OB spectrum %d",j);
			DispMsg(Alert_1, msg, status, program, pOutput->chatter);		
		     }
		  */   if (status != 1) /* Rebin returns 1 when there are no counts*/
		     {
		        /* Add spectrum to histogram */
		        if (pDet->obSpectraFlag[i])
			{
			   OBHist(pDet, i, 0, addCounts, gtList, &total1);
			} 
		        if (pDet->obSpectraFlag[i+SPISTRUCT_NDETS])
		        {
			   OBHist(pDet, i, 1, addCounts, gtList, &total2);
		        }
		     }
		     status = ISDC_OK; 
		  }
	       } /* End single ob spectrum processing */ 
	    } /* End ob spectra loop */
	 }
      } /* End det i = 0-18 loop */
      sprintf(msg, "%ld OB counts added to dets 0-18",total1);
      DispMsg(Log_1, msg, status, program, pOutput->chatter);
      sprintf(msg, "%ld OB counts added to dets 85-103",total2);
      DispMsg(Log_1, msg, status, program, pOutput->chatter);
      
      DALfreeDataBuffer(pha, ISDC_OK);
      DALfreeDataBuffer(addCounts, ISDC_OK);
   }
   if (pSpectra)
   {
      for (i=0; i< nSpectra; ++i)
      {
         DALfreeDataBuffer(pSpectra[i], ISDC_OK);
      }
      DALfreeDataBuffer(pSpectra, ISDC_OK);
   }
   DALobjectClose(calib, DAL_SAVE, ISDC_OK);
   return status;
}  
