/* 
-------- Function: -----------------------------------------------------------------------

     CopyPointingExt


-------- Description: --------------------------------------------------------------------

     This subroutine writes the POINTING extension for a PHA file of type II. 

-------- Usage: --------------------------------------------------------------------------

   Input Parameters:
       INPUTDS      dal_element   : Pointer to the input data structure.
       PARENT	    dal_element	  :Output group to which this extension
      					   is attached.
       PHEADER      headerParsPtr
       CHATTER      int           : Chattiness flag for o/p (5 quite, 10 normal,
                                    >20 silly)

             headerParsPtr pars
	     ------------------
       TSKSTR       char          : Name+version of the task
       TELESCOPE    char          : String listing telescope/mission
       INSTRUME     char          : String listing instrument/detector
       FILTER       char          : String listing instrument filter in use


-------- Include files -------------------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: ---------------------------------------------------

   Craig Gordon (3.0.8; May 03)
   Sandhia Bansal  (1.0.0; Aug 00)


--------------------------------------------------------------------------------------- */

#include "spihist.h"



void CopyPointingExt(dal_element *inputDS, dal_element *parent,
			headerParsPtr pHeader, memoryPtr *pBase, int chatter)
{
   int                status=ISDC_OK;
   
   char               *program="CopyPointingExt";
   char               comment[FLEN_COMMENT];
   char               message[DAL_BIG_STRING];
   char		      *templ="SPI.-OBS.-PNT";
   dal_element *inPointing=NULL;
   dal_element *doomed=NULL;
   dal_element *pTable=NULL;
   
   /* No test is performed here for existence of input pointing extension.
      It was done much earlier in the program, in ReadDither. */
   DALobjectFindElement(inputDS, templ, &inPointing, status);
   
   status = DALobjectFindElement(parent, templ, &doomed, status); 
   if (status == DAL_ELEMENT_NOT_FOUND)
   {
      status = ISDC_OK;
   }
   else if (status == ISDC_OK && doomed)
   {
      // Remove existing pointing table.
      DALtableDelete(doomed, status);
   }
   else
   {
      strcpy(message,"Error initializing CopyPointingExt");
      Cleanup(status, message, *pBase, program);
   }
   status = DALtableCopy(inPointing, parent, NULL, ACCESS_UNKNOWN, &pTable, status);
   if (status != ISDC_OK)
   {
      strcpy(message,"Error while copying pointing extension to output.");
      Cleanup(status, message, *pBase, program);
   }
   /* write the keywords */

   /* creator */
   strcpy(comment, "Program creating PHA file");
   PutKeyword(pTable, "CREATOR", pHeader->tskstr, CHAR, comment, program);

   strcpy(comment, "Version of format (OGIP memo OGIP-92-007a)");
   PutKeyword(pTable, "HDUVERS", (void *) PHAVERSN, CHAR, comment, program);

   strcpy(comment, "Telescope or Mission name");
   PutKeyword(pTable, "TELESCOP", (void *) pHeader->telescope, CHAR, comment, program);

   strcpy(comment, "Instrument name");
   PutKeyword(pTable, "INSTRUME", (void *) pHeader->instrume, CHAR, comment, program);

   strcpy(message, " ... Written the extension header keywords");
   DispMsg(Log_1, message, status, program, chatter);

   CommonStampObject(pTable, "", status);
   
   strcpy(message, " ... Successfully written the POINTING extension");
   DispMsg(Log_1, message, status, program, chatter);
}
