/* 
-------- Function: -----------------------------------------------------------------------

     CreateDataIIExt

-------- Description: --------------------------------------------------------------------

     This subroutine writes the SPECTRUM extension for a PHA file of type II. 


-------- Functions called: ---------------------------------------------------------------

   Spihist Functions:
      PutHduclas2           -- Writes HDUCLAS2 keyword to SPECTRUM extension

   SPI  Functions:
      Allocate              -- Allocates specified number of bytes and keeps track of all
                               allocated memory during a run
      IdentifyProgram       -- Displays the name and version number of the current program
      Cleanup               -- Deallocates all memory allocated during the current run in
                               the event of an error and exits the program
      DispMsg               -- Writes messages to the screen and a log file
      PutKeyword            -- Writes the specified keyword to the table header
      FreeNode              -- Frees some of the allocated memory when not needed

   DAL Functions:
      DALtableAddCol        -- Adds a column to the current table
      DALtableAddRows       -- Adds specified number of rows to the current col
      DALtableGetColStruct  -- Returns column specific information (column name, number of
                               rows, bin size, data type and whether or not the column 
			       bins are of variable size)
      DALtableGetNumCols    -- Returns the number of columns in current table
      DALtablePutColBins    -- Writes the specified data to column
      DALtablePutColStrings -- Writes the input string array to the specified column


-------- Usage: --------------------------------------------------------------------------

   Input Parameters:
       PTABLE       dal_element    : Pointer to the Data extension
       PBASE        memoryPtr      : Pointer to the list that keeps track of allocated
                                     memory
       PLAST        memoryPtr      : Pointer to the current node in the list pointed to by
                                     PBASE
       POINTINGNUM  int            : Index to the pointing 
       DETNAM       char           : String listing specific detector name
       GTLIST       gtIntPtr       : Structs containing Good Time + Livetime info
       PGTROWS	    gtRowsPtr
       DETSUMFLAG   int		   : +1 Indicates all detectors within a
       				     pointing will be summed.
       PNTSUMFLAG   int		   : +1 Indicates each detector is summed
       				     across pointings.
				     A -1 for both indicates both cases
				     are output, along with the normal 
				     individual detector spectra.
       TOTPOINTINGS int		   : The total number of pointings that will
       				     be processed during spihist run.
       LASTFLAG     int		   : Indicates this will be the last call
       				     to this function.

                outputParsPtr pars
		------------------
       DTYPE        int            : Flag to denote counts (1) or count rates (2) or
                                     count rates per energy (3)
       CHATTER      int            : Chattiness flag for o/p (5 quite, 10 normal,
                                     >20 silly)

		binningPtr pars
		---------------
       ENERGY       float          : Array containing the bin bounds 

                detinfoPtr pars
		---------------
       DETCHANS     long           : Total number of possible detector channels 
       MAXDET       int            : Total number of det for the instrument
       DET          int            : Det[i]=1 if ith detector is requested to be 
                                     processed; Valid values are 0 and 1.
       HIST         long           : Hist[i]!=0 if a histogram was accumulated for ith
                                     detector

                headerParsPtr pars
		------------------
       CHANTYP      char           : Type of detector channels (PHA, PI etc)
       QERROR       int            : 1 if stat errors were passed down
       QSYS         int            : 1 if systematic errors were passed down
       QQUAL        int            : 1 if quality array were passed down
       QGROUP       int            : 1 if grouping array were passed down
       TSKSTR       char           : Name+version of the task
       TELESCOPE    char           : String listing telescope/mission
       INSTRUME     char           : String listing instrument/detector
       FILTER       char           : String listing instrument filter in use
       AREASCAL     float          : Area scaling factor
       BACKSCAL     float          : Background scaling factor
       CORRSCAL     float          : Correction scaling factor
       CORRFIL      char           : Associated correction filename
       BACKFIL      char           : Associated background filename
       RESPFIL      char           : Detector redistribution matrix file
       ANCRFIL      char           : Ancillary response file
       XFLT0001     char           : XSPEC filter description for each XSPEC 


-------- Include files -------------------------------------------------------------------

   "spihist.h"


-------- Origin: -------------------------------------------------------------------------

   This function used CreateDataIIExt.c as a basis.


-------- Authors/Modification History: ---------------------------------------------------

                (3.1.1; Aug 03)   TLMINn and TLMAXn keywords: n
                                   is no longer hardcoded, but found
                                   by searching for "CHANNEL" column.
		(3.0.8; May 03) - Exposure keyword not needed (fr new
		                  pha2 spectrum template).
                (3.0.4; Feb 03) - Modifications for GTI,livetime, and
				  pointingID handling, using new 
				  gtInt and gtRows structs.
                (3.0.2; Feb 03) - bug fix to dtype 3 sum output
   Craig Gordon (3.0.0; Dec 02) - removed multipointing option
   Craig Gordon (2.2.0; Sep 02) - added various spectrum summations capability
   Sandhia Bansal  (1.0.0; Aug 00)


--------------------------------------------------------------------------------------- */

#include "spihist.h"


#define NK_HISTORY     1  
              /* number of history records to be created in the ebounds ext */
#define NK_COMM        1       
              /* number of comment records to be created in the ebounds ext */




void CreateDataIIExt(dal_element *pTable, memoryPtr *pBase, memoryPtr *pLast, 
		     int pointingNum, char *detnam, gtIntPtr **gtList,
		     gtRowsPtr pGtrows, outputParsPtr pOutput, binningPtr pBin, 
		     detinfoPtr pDet, headerParsPtr pHeader, int detSumFlag, 
		     int pntSumFlag, int totPointings, int lastFlag)
		     
{
   double             dkeyval=0.0;
   static double      *sumsExposure=NULL;
   static double      *exposure=NULL;
   const double       VERY_SMALL = 1.0e-30;

   
   static float       *fdata=NULL;
   static float       *statErr=NULL;
   static float       *sumsErr=NULL;
   static float       *sysErr=NULL;
   float              temp=0.0;
   static float       *fsumsBuffer=NULL;

   static long        *data=NULL;
   static long        *sumsBuffer=NULL;
   long               numRows=0;
   long               numEntries=0;
   long               total=0;
   long               nRows=0;
   long               lkeyval=0L;
   long		      extRows=0;
   long		      incr=0;

   static int         startBin=1;
   static int         firstFlag=1;
   int                status=ISDC_OK;
   int                tfields=5;   /* table will have at least 4 columns */
   int                i=0, j=0, idet=0;
   int                numCols=0;
   int                chanCol=0;
   int                varLength=0;
   int                nch = 0;
   int                irow=0;
   int		      nDetsSelected=0;
   int                tlmax=TLMIN+pDet->detchans-1;
   int                pointing=0;
   int		      pntDets=0;
   int		      onlySums=0;
   int		      *sumsChannels=NULL;
   		
   static short       *specNum=NULL;    /* static, so we don't have to keep allocating it
					   for every call */
   static int	      *sumsSpecNum=NULL;   
   static short       *channels=NULL;
   static short       *grouping=NULL;
   static short       *quality=NULL;
   
   static char        **rowid=NULL;
   static char	      **sumsid=NULL;
   static char        **respfile=NULL;
   static char        **ancrfile=NULL;
   char               colName[DAL_MAX_NAME_SIZE];
   char               *program="CreateDataIIExt";
   char               *version="1.0.0";
   char 	      phaVersion[FLEN_VALUE];
   char		      ogipClass[FLEN_VALUE];
   char               keyname[FLEN_KEYWORD];
   char               tcomm[FLEN_VALUE];
   char               tunit[FLEN_VALUE];
   char               comment[FLEN_COMMENT]="\0";
   char               **history = NULL;
   char               **commkeywd = NULL;
   char               hduclas3[FLEN_VALUE];
   char               message[DAL_MAX_STRING];
   char               *extname="SPECTRUM";
   char               tlminKeyword[10];
   char               tlmaxKeyword[10];

   dal_dataType       type;


   if (detSumFlag > 0 || pntSumFlag > 0)
   {
      onlySums = 1;
   }
   else
   {
      onlySums = 0;
   }
   pntDets = pDet->maxdet;
   
   if (firstFlag)
   {
      IdentifyProgram(program, version, pOutput->chatter);
      
      pHeader->qerror = 1;
      
      if (NK_HISTORY > 0)
      {
	 Allocate(program, "-history", NK_HISTORY, FLEN_COMMENT, (void **) &history, pBase,
		  pLast);

	 strcat(strcat(strcat(strcat(strcpy(history[0], extname), 
				     " extension written by "),
			      program), 
		       " Version "), 
		version);
	 for (i=1; i<NK_HISTORY; i++)
	    strcpy(history[i], "\0");

	 strcpy(comment, "\0");

	 for (i=0; i<NK_HISTORY; i++)
	    PutKeyword(pTable, "HISTORY", history[i], CHAR, comment, program);
	 FreeNode(pBase, (void **) &history, pLast);
      }

      if (NK_COMM > 0)
      {
	 Allocate(program, "-commkeywd", NK_COMM, FLEN_COMMENT, (void **)&commkeywd, pBase,
		  pLast);

	 strcpy(commkeywd[0], "This extension contains the spectra");
	 for (i=1; i<NK_COMM; i++)
	    strcpy(commkeywd[i], "\0");

	 strcpy(comment, "\0");

	 for (i=0; i<NK_COMM; i++)
	    PutKeyword(pTable, "COMMENT", commkeywd[i], CHAR, comment, program);
	 FreeNode(pBase, (void **) &commkeywd, pLast);
      }
	
      /* Add exposure column */
      status = DALtableAddCol(pTable, "EXPOSURE", DAL_DOUBLE, 1, status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while adding EXPOSURE column to the table");
	 Cleanup(status, message, *pBase, program);         
      }
      /* and delete the exposure keyword from the header */
      DALattributeDelete(pTable, "EXPOSURE", status);
      	 
      /* Add channel column with DETCHANS bins in SHORT format if it is a PHA II file */
      if (strcmp(pOutput->format, "PHA") == 0)
      {
	 status = DALtableAddCol(pTable, "CHANNEL", INT_I, (long) pDet->detchans, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error while adding CHANNEL column to the table");
	    Cleanup(status, message, *pBase, program);
	 }
      }

      if ((strcmp(pHeader->chantype, "PHA") == 0) || 
	  (strcmp(pHeader->chantype, "pha") == 0))
	 strcpy(tcomm, "Pulse Height Analyser (PHA) Channel ");
      else if ((strcmp(pHeader->chantype, "PI") == 0) || 
	       (strcmp(pHeader->chantype, "pi") == 0))
	 strcpy(tcomm, "Pulse Invarient (PI) Channel ");
      else
	 strcpy(tcomm, "Detector channel (type unknown)");
      
      PutKeyword(pTable, "TCOMM2", tcomm, CHAR, comment, program);

      /* Add COUNTS/RATE column with DETCHANS bins in LONG/FLOAT format 
	 Determine whether to write data as COUNTS or COUNTS/SEC or OUNTS/SEC/KEV */
      strcpy(hduclas3, "RATE");
      if (pOutput->dtype == 1)
      {
	 strcpy(colName, "COUNTS");
	 type = INT_J;
	 strcpy(tunit, "counts");
	 strcpy(tcomm, "PHA data stored as Counts");
	 strcpy(hduclas3, "COUNT");
      }
   
      else if (pOutput->dtype == 2)
      {
	 strcpy(colName, "RATE");
	 type = REAL_E;
	 strcpy(tunit, "counts/sec");
	 strcpy(tcomm, "PHA data stored as Counts/s");
      }

      else if (pOutput->dtype == 3)
      {
	 strcpy(colName, "RATE");
	 type = REAL_E;
	 strcpy(tunit, "counts/sec/keV");
	 strcpy(tcomm, "PHA data stored as Counts/s/keV");
      }

      /* Write hduclas3 keyword */
      PutKeyword(pTable, "HDUCLAS3", hduclas3, CHAR, tcomm, program);
 
      status = DALtableAddCol(pTable, colName, type, (long) pDet->detchans, status); 

      if (status != ISDC_OK)
      {
	 strcat(strcat(strcpy(message, "Error while adding "), colName),
		" to the table");
	 Cleanup(status, message, *pBase, program);
      }

      /* Write TUNIT3 and TCOMM3 keywords */
      PutKeyword(pTable, "TCOMM3", tcomm, CHAR, comment, program);

      /* Add STAT_ERR column with DETCHANS bins in FLOAT format 
	 Determine whether to write data as COUNTS or COUNTS/SEC or COUNTS/SEC/KEV */
      if (pHeader->qerror != 0)
      {
	 strcpy(colName, "STAT_ERR");
	 type = REAL_E;
	 if (pOutput->dtype == 1)
	 {
	    strcpy(tunit, "counts");
	    strcpy(tcomm, "Statistical error on COUNTS");
	 }
	 
	 else if (pOutput->dtype == 2)
	 {
	    strcpy(tunit, "counts/s");
	    strcpy(tcomm, "Statistical error on RATE");
	 }

	 else if (pOutput->dtype == 3)
	 {
	    strcpy(tunit, "counts/s/keV");
	    strcpy(tcomm, "Statistical error on RATE/keV");
	 }
 
	 status = DALtableAddCol(pTable, colName, type, (long) pDet->detchans, status);

	 if (status != ISDC_OK)
	 {
	    strcat(strcat(strcpy(message, "Error while adding "), colName),
		   " to the table");
	    Cleanup(status, message, *pBase, program);
	 }

	 sprintf(keyname, "TUNIT%d", tfields);
	 PutKeyword(pTable, keyname, tunit, CHAR, comment, program);
	 sprintf(keyname, "TCOMM%d", tfields);
	 PutKeyword(pTable, keyname, tcomm, CHAR, comment, program);
	 tfields++;
      }

      /* Add SYS_ERR column with DETCHANS bins in FLOAT format 
	 Determine the comment based on the units of data */
      if (pHeader->qsys != 0)
      {
	 strcpy(colName, "SYS_ERR");
	 type = REAL_E;
	 strcpy(tunit, " ");
	 if (pOutput->dtype == 1)
	 {
	    strcpy(tcomm, "Fractional systematic errors on COUNTS");
	 }

	 else if (pOutput->dtype == 2)
	 {
	    strcpy(tcomm, "Fractional systematic errors on RATE");
	 }

	 else if (pOutput->dtype == 3)
	 {
	    strcpy(tcomm, "Fractional systematic errors on RATE/keV");
	 }

	 status = DALtableAddCol(pTable, colName, type, (long) pDet->detchans, status); 

	 if (status != ISDC_OK)
	 {
	    strcat(strcat(strcpy(message, "Error while adding "), colName),
		   " to the table");
	    Cleanup(status, message, *pBase, program);
	 }
	 

	 Allocate(program, "-sysErr", 0, pDet->maxdet*pDet->detchans*sizeof(float), 
		  (void **) &sysErr, pBase, pLast);

	 sprintf(keyname, "TUNIT%d", tfields);
	 PutKeyword(pTable, keyname, tunit, CHAR, comment, program);
	 sprintf(keyname, "TCOMM%d", tfields);
	 PutKeyword(pTable, keyname, tcomm, CHAR, comment, program);
	 tfields++;
      }

      else
      {
	 lkeyval = 0L;
	 strcpy(comment, "No systematic error specified");
	 PutKeyword(pTable, "SYS_ERR", &lkeyval, INT_J, comment, program);
	 strcpy(comment, "\0");
      }

      /* Add QUALITY column with DETCHANS bins in SHORT format */
      if (pHeader->qqual != 0)
      {
	 strcpy(tunit, " ");
	 strcpy(tcomm, "Quality flag of this channel (0=good)");

	 status = DALtableAddCol(pTable, "QUALITY", INT_I, (long) pDet->detchans, status);

	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error while adding QUALITY column to the table");
	    Cleanup(status, message, *pBase, program);
	 }

	 Allocate(program, "-quality", 0, pDet->maxdet*pDet->detchans*sizeof(short), 
		  (void **) &quality, pBase, pLast);

	 sprintf(keyname, "TUNIT%d", tfields);
	 PutKeyword(pTable, keyname, tunit, CHAR, comment, program);
	 sprintf(keyname, "TCOMM%d", tfields);
	 PutKeyword(pTable, keyname, tcomm, CHAR, comment, program);
	 tfields++;
      }
      
      else
      {
	 lkeyval = 0L;
	 strcpy(comment, "no data quality information specified");
	 PutKeyword(pTable, "QUALITY", &lkeyval, INT_J, comment, program);
	 strcpy(comment, "\0");
      }

      if (pHeader->qgroup != 0)
      {
	 strcpy(tunit, " ");
	 strcpy(tcomm, "Grouping flag of this channel (0=undefined)");

	 status = DALtableAddCol(pTable, "GROUPING", INT_I, (long) pDet->detchans, 
				 status); 

	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error while adding GROUPING column to the table");
	    Cleanup(status, message, *pBase, program);
	 }
      

	    
	 Allocate(program, "-grouping", 0, 
	       pntDets*pDet->detchans*sizeof(short), (void **) &grouping,
	       pBase, pLast);
         
	 sprintf(keyname, "TUNIT%d", tfields);
	 PutKeyword(pTable, keyname, tunit, CHAR, comment, program);
	 sprintf(keyname, "TCOMM%d", tfields);
	 PutKeyword(pTable, keyname, tcomm, CHAR, comment, program);
	 tfields++;
      }
  
      else
      {
	 lkeyval = 0L;
	 strcpy(comment, "Grouping flag of this channel (0=undefined)");
	 PutKeyword(pTable, "GROUPING", &lkeyval, INT_J, comment, program);
	 strcpy(comment, "\0");
      }

      /* Add keywords to the table */
      strcpy(comment, "Version of format (OGIP memo OGIP-92-007a)");
      /* Fix added for gcc compilation */
      strcpy(phaVersion, PHAVERSN);
      PutKeyword(pTable, "HDUVERS", (void *)phaVersion, CHAR, comment, program);

      /* Write the HDUCLAS2 keyword to this extension */
      PutHduclas2(pTable, pHeader);

      chanCol = ColNameToIndex(pTable, "CHANNEL");
      sprintf(tlminKeyword, "TLMIN%d", chanCol);
      sprintf(tlmaxKeyword, "TLMAX%d", chanCol);
      lkeyval = (long) TLMIN;
      strcpy(comment, "Lowest legal channel number");
      PutKeyword(pTable, tlminKeyword, &lkeyval, INT_J, comment, program);

      lkeyval = (long) tlmax;
      strcpy(comment, "Highest legal channel number");
      PutKeyword(pTable, tlmaxKeyword, &lkeyval, INT_J, comment, program);

      strcpy(comment, "Program creating PHA file");
      /* creator */
      PutKeyword(pTable, "CREATOR", pHeader->tskstr, CHAR, comment, program);

      strcpy(comment, "Telescope or Mission name");
      PutKeyword(pTable, "TELESCOP", pHeader->telescope, CHAR, comment, program);

      strcpy(comment, "Instrument name");
      PutKeyword(pTable, "INSTRUME", pHeader->instrume, CHAR, comment, program);

      strcpy(comment, "Specific detector name in use");
      PutKeyword(pTable, "DETNAM", detnam, CHAR, comment, program);

      strcpy(comment, "Filter in use");
      PutKeyword(pTable, "FILTER", pHeader->filter, CHAR, comment, program);

      dkeyval = (double) pHeader->areascal;
      strcpy(comment, "Area scaling factor");
      PutKeyword(pTable, "AREASCAL", &dkeyval, REAL_D, comment, program);

      if (strcmp(pHeader->backfil, "none") == 0)
	 strcpy(pHeader->backfil, "NONE");
      strcpy(comment, "Associated background filename");
      PutKeyword(pTable, "BACKFILE", pHeader->backfil, CHAR, comment, program);

      dkeyval = (double) pHeader->backscal;
      strcpy(comment, "Background file scaling factor");
      PutKeyword(pTable, "BACKSCAL", &dkeyval, REAL_D, comment, program);

      if (strcmp(pHeader->corrfil, "none") == 0)
	 strcpy(pHeader->corrfil, "NONE");
      strcpy(comment, "Associated correction filename");
      PutKeyword(pTable, "CORRFILE", pHeader->corrfil, CHAR, comment, program);

      dkeyval = (double) pHeader->corrscal;
      strcpy(comment, "Correction file scaling factor");
      PutKeyword(pTable, "CORRSCAL", &dkeyval, REAL_D, comment, program);

      if ((strcmp(pHeader->respfil, "") == 0) ||
	  (strcmp(pHeader->respfil, "none") == 0) ||
	  (strcmp(pHeader->respfil, "NONE") == 0))
      {
	 strcpy(pHeader->respfil, "NONE");
	 strcpy(comment, "Associated redistrib matrix filename");
	 PutKeyword(pTable, "RESPFILE", pHeader->respfil, CHAR, comment, program);
      }
      
      else
      {
	 /* insert a new column RESPFILE in the table */
	 status = DALtableAddCol(pTable, "RESPFILE", CHAR, DAL_BIG_STRING, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error while adding RESPFILE column to the table");
	    Cleanup(status, message, *pBase, program);
	 }
      }

      if ((strcmp(pHeader->ancrfil, "") == 0) || 
	  (strcmp(pHeader->ancrfil, "none") == 0) ||
	  (strcmp(pHeader->ancrfil, "NONE") == 0))
      {
	 strcpy(pHeader->ancrfil, "NONE");
	 strcpy(comment, "Associated ancillary response filename");
	 PutKeyword(pTable, "ANCRFILE", pHeader->ancrfil, CHAR, comment, program);
      }

      else
      {
	 /* insert a new column ANCRFILE in the table */
	 status = DALtableAddCol(pTable, "ANCRFILE", CHAR, DAL_BIG_STRING, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error while adding ANCRFILE column to the table");
	    Cleanup(status, message, *pBase, program);
	 }
      }

      strcpy(comment, "OGIP classification of FITS format");
      /* Fix added for gcc compilation */
      strcpy(ogipClass, "1992a");
      PutKeyword(pTable, "PHAVERSN", (void *)ogipClass, CHAR, comment, program);

      lkeyval = (long) pDet->detchans;
      strcpy(comment, "Total number of detector channels");
      PutKeyword(pTable, "EBIN_NUM", &lkeyval, INT_J, comment, program);

      strcpy(comment, "channel type (PHA, PI etc)");
      PutKeyword(pTable, "CHANTYPE", pHeader->chantype, CHAR, comment, program);

      if (pHeader->qerror == 0)
	 lkeyval = 1L;   /* for dtype=1 (COUNTS per channel) and qerror=0
			    (no statistical errors), poissonian=TRUE */
      else
	 lkeyval = 0L;

      strcpy(comment, "Poissonian errors to be assumed");
      PutKeyword(pTable, "POISSERR", &lkeyval, BOOL, comment, program);

      if (pHeader->qerror == 0)
      {
	 lkeyval = 0L;
	 strcpy(comment, "No statistical error specified");
	 PutKeyword(pTable, "STAT_ERR", &lkeyval, INT_J, comment, program);
      }

      strcpy(comment, "XSPEC selection filter description");
      PutKeyword(pTable, "XFLT0001", pHeader->xflt0001, CHAR, comment, program);

      strcpy(message, " ... Written the extension header keywords");
      DispMsg(Log_1, message, status, program, pOutput->chatter);

      /* Allocate memory for data buffers */
      if (!onlySums)
      {
	 Allocate(program, "-statErr", 0, 
		  pntDets*pDet->detchans*sizeof(float), (void **) &statErr, 
		  pBase, pLast);
	 if (pOutput->dtype == 1)
	 {
	    Allocate(program, "-data", 0, pntDets*pDet->detchans*sizeof(long),
		     (void **) &data, pBase, pLast);
	 }

	 else
	 {
	    Allocate(program, "-fdata", 0, 
		     pntDets*pDet->detchans*sizeof(float), (void **) &fdata, 
		     pBase, pLast);
	 }

	 Allocate(program, "-specNum", 0, pntDets*sizeof(short), 
		  (void **) &specNum, pBase, pLast);

	 Allocate(program, "-channels", 0, 
		  pntDets*pDet->detchans*sizeof(short), (void **) &channels,
		  pBase, pLast);

	 Allocate(program, "-rowid", pntDets, ROWIDSZ, (void **) &rowid, pBase,
		  pLast);

	 if (strcmp(pHeader->respfil, "NONE") != 0)
	 {
	    Allocate(program, "-respfile", pntDets, DAL_BIG_STRING,
		     (void **) &respfile, pBase, pLast);
	 }

	 if (strcmp(pHeader->ancrfil, "NONE") != 0)
	 {
	    Allocate(program, "-ancrfile", pntDets, DAL_BIG_STRING,
		     (void **) &ancrfile, pBase, pLast);
	 }

	 Allocate(program, "-exposure", 0, pntDets*sizeof(double),
   		   (void **) &exposure, pBase, pLast);

	 for (i=0; i<pntDets; ++i)
	 {
            exposure[i] = .0;
	 }
      } 
   } /* end if firstFlag */

   if (!onlySums)
   {
      /* Determine the number of rows to be added at the end of the table */
      nRows = 0;
      irow  = 0;
      pointing = pointingNum + 1;

      nDetsSelected=0;
      for (idet=0; idet<pDet->maxdet; idet++)
      {
	 if (pDet->det[idet] == 1)
	 {
	    ++nDetsSelected;
	    specNum[irow] = (short) irow;

	    sprintf(rowid[irow], "POINTG_%d_DET_%d", pointing, idet);
	    exposure[irow] = pGtrows->livetime[pointingNum][idet];
	    
	    if (respfile != NULL)
	       strcat(strcat(strcat(strcpy(respfile[irow], pHeader->respfil), 
				    "_"), 
			     rowid[irow]),
		      ".rmf");

	    if (ancrfile != NULL)
	       strcat(strcat(strcat(strcpy(ancrfile[irow], pHeader->ancrfil), 
				    "_"), 
			     rowid[irow]),
		      ".arf");

	    for (i=TLMIN; i<=tlmax; i++)
	    {
	       channels[nch] = (short) i;
	       if (pOutput->dtype == 1)
	       {
		  data[nch]    = pDet->hist[idet][i];
		  statErr[nch] = sqrt((float)(pDet->hist[idet][i]));
	       }

	       else if (pOutput->dtype == 2)
	       {
		  if (exposure[irow] > VERY_SMALL)
		  {
		     fdata[nch]   = pDet->hist[idet][i] / exposure[irow];
		     statErr[nch] = sqrt((float)(pDet->hist[idet][i]))
		     			 / exposure[irow];
		  }

		  else
		     fdata[nch] = statErr[nch] = .0;
	       }

	       else if (pOutput->dtype == 3)
	       {
		  if (exposure[irow] > VERY_SMALL)
		  {
		     temp = exposure[irow] * (pBin->energy[i+1]-pBin->energy[i]);

		     fdata[nch]   = pDet->hist[idet][i] / temp;
		     statErr[nch] = sqrt((float)(pDet->hist[idet][i])) / temp;
		  }

		  else
		     fdata[nch] = statErr[nch] = .0;
	       }

	       if (grouping)
		  grouping[nch] = 1;

	       nch++;
	    }
	    ++irow;
	 }
      }

      nRows = (long) irow;

      if (nRows > 0)
      {
	 /* We must add the desired rows. 
	    Here we add nRows rows to the end of the table */
	 status = DALtableAddRows(pTable, -1, nRows, status);

	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error while adding rows to the table.");
	    Cleanup(status, message, *pBase, program);
	 }

	 if (numCols == 0)
	 {
	    /* Get the number of columns in the table */
	    status = DALtableGetNumCols(pTable, &numCols, status);

	    if (status != ISDC_OK)
	    {
	       strcpy(message, "Error while getting number of columns");
	       Cleanup(status, message, *pBase, program);
	    }
	 }

	 /* for each table column, get its structure-related info and fill it
	    with data */

	 for (i=0; i<numCols; i++)
	 {
	    /* determine the structure of the column */
	    strcpy(colName, "\0");
	    status = DALtableGetColStruct(pTable, i+1, &numRows, colName, &type, &numEntries,
					  &varLength, status);
	    if (status != ISDC_OK)
	    {
	       strcpy(message, "Error while reading column structure");
	       Cleanup(status, message, *pBase, program);
	    }

	    total = nch;

	    /* based upon the column data type, create a buffer of data and
	       write it to the column.  Note the pattern for writing data:
	       (1) determine how large the data buffer must be,
	       (2) allocate the correct size data buffer,
	       (3) write the data, and
	       (4) deallocate the allocated data buffer */

	    switch(type)
	    {
	       case INT_I:

		  if (strcmp(colName, "SPEC_NUM") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, nRows,
						 specNum, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing SPEC_NUM");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else if (strcmp(colName, "CHANNEL") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, total,
						 channels, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing CHANNEL");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else if (strcmp(colName, "GROUPING") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, total,
						 grouping, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing GROUPING");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else if (strcmp(colName, "QUALITY") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, total,
						 quality, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing QUALITY");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else
		  {
		     status = DAL_TABLE_COL_NOT_FOUND;
		     sprintf(message, "Invalid column name: %s \n", colName);
		     Cleanup(status, message, *pBase, program);
		  }

		  break;

	       case INT_J:

		  /* the column contains INT_J data. */

		  if (strcmp(colName, "COUNTS") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, total,
						 data, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing COUNTS");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  break;

	       case REAL_E:
		  /* the column contains REAL_E data. */

		  if (strcmp(colName, "RATE") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, total,
						 fdata, status);	
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing RATE");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else if (strcmp(colName, "STAT_ERR") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, total,
						 statErr, status);	
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing STAT_ERR");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else if (strcmp(colName, "SYS_ERR") == 0)
		  {
		     status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 0, total,
						 sysErr, status);	
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing SYS_ERR");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  break;

	       case REAL_D:
		  if (strcmp(colName, "EXPOSURE")==0)
		  {
	             status = DALtablePutColBins(pTable, NULL, i+1, type, startBin, 
		  		   0, nRows, exposure, status);
		  }
		  break;

	       case CHAR:

		  /* the column contains CHAR data. */
		  if (strcmp(colName, "ROWID") == 0)
		  {
		     status = DALtablePutColStrings(pTable, NULL, i+1, startBin, 0, nRows,
						    rowid, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing ROWID");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else if (strcmp(colName, "RESPFILE") == 0)
		  {
		     status = DALtablePutColStrings(pTable, NULL, i+1, startBin, 0, nRows, 
						    respfile, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing RESPFILE");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  else if (strcmp(colName, "ANCRFILE") == 0)
		  {
		     status = DALtablePutColStrings(pTable, NULL, i+1, startBin, 0, nRows,
						    ancrfile, status);
		     if (status != ISDC_OK)
		     {
			strcpy(message, "Error while writing ANCRFILE");
			Cleanup(status, message, *pBase, program);
		     }
		  }

		  break;

	       default:
		  status = DAL_INVALID_DATA_TYPE;
		  strcpy(message, "Invalid data type");
		  Cleanup(status, message, *pBase, program);

		  break;
	    }
	 }

	 startBin += nRows;
      } /* end if nRows >0 */
   } /* end if only sums */
   
   /*  This code has been added in to perform and write summations (over
       detectors and/or pointings) to the output.  */
   if (detSumFlag || pntSumFlag)
   {
   /* Initialization common to all summation selections. */
      if (detSumFlag<0 || pntSumFlag<0)
      {
         extRows = totPointings + pDet->maxdet + 1;
      }
      else if (!detSumFlag)
      {
         /* If this block of code is reached, it means the number of 
	 detectors the user has requested hasn't been calculated previously,
	 so it is done now. */
	 for (idet=0; idet<pDet->maxdet; ++idet)
	 {
	    if (pDet->det[idet] == 1)
	    {
	       ++nDetsSelected;
	    }
	 }
	 extRows = nDetsSelected;
      }
      else if (!pntSumFlag)
      {
         extRows = totPointings;
      }
      else
      {
         extRows = 1;
      }
      if (firstFlag)
      {
	 Allocate(program, "-sumsBuffer", 0, extRows*pDet->detchans
		*sizeof(long), (void **) &sumsBuffer, pBase, pLast);
	 for (i=0; i<extRows*pDet->detchans; ++i) 
	 { 
		sumsBuffer[i] = 0;
         }
	 if (pOutput->dtype != 1)
	 {
            Allocate(program, "-fsumsBuffer", 0, extRows*pDet->detchans
	    		*sizeof(float), (void **) &fsumsBuffer, pBase, pLast);
	    for (i=0; i<extRows*pDet->detchans; ++i)
	    {
	       fsumsBuffer[i] = 0.0;
	    }
	 }
	 Allocate(program, "-sumsExposure", 0, extRows*sizeof(double), 
	 		(void **)&sumsExposure, pBase, pLast);
	 for (i=0; i<extRows; ++i)
	 {
	    sumsExposure[i] = 0.0;
	 }	
         Allocate(program, "-sumsSpecNum",0, extRows*sizeof(int),
	 		(void **)&sumsSpecNum, pBase, pLast);
	 Allocate(program, "-sumsid", extRows, ROWIDSZ, (void **) &sumsid, 
	 		pBase, pLast);
	 Allocate(program, "-sumsErr",0, extRows*pDet->detchans*sizeof(float),
	 		(void **)&sumsErr, pBase, pLast);
      }
      irow = 0;

      if (detSumFlag<0 || (detSumFlag && !pntSumFlag))
      {
	 /* Calculate sum over detectors in each pointing */
	 incr = (irow+pointingNum)*pDet->detchans;
	 for (idet=0; idet<pDet->maxdet; ++idet)
	 {
	    if (pDet->det[idet] == 1)
	    {
	       sumsExposure[irow+pointingNum] += 
	       		pGtrows->livetime[pointingNum][idet];
	    }
	 }
	 for (i=0; i<pDet->detchans; ++i)
	 {
	    for (idet=0; idet<pDet->maxdet; ++idet)
	    {
	       if (pDet->det[idet]==1)
	       {
		  sumsBuffer[incr+i] += pDet->hist[idet][i];
	       }
	       sumsErr[incr+i] = sqrt((float)sumsBuffer[incr+i]);
	    }   
	 }
	 if (pOutput->dtype != 1)
	 {
	    if (sumsExposure[irow+pointingNum] > VERY_SMALL)
	    {
	       for (i=0; i<pDet->detchans; ++i)
	       {
		    fsumsBuffer[incr+i] = 
		       sumsBuffer[incr+i]/sumsExposure[irow+pointingNum];
		    sumsErr[incr+i] /= sumsExposure[irow+pointingNum];
		    if (pOutput->dtype == 3)
		    {
		       fsumsBuffer[incr+i] /=
			  (pBin->energy[i+1]-pBin->energy[i]);
		       sumsErr[incr+i] /= (pBin->energy[i+1]-pBin->energy[i]);
		    }
	       }
	    }
	    /* else do nothing, fsumsBuffer[i] will be 0.0 */
	 }	    
	 sprintf(sumsid[pointingNum], "POINTG_%d_SUM", pointingNum+1);
	 irow = totPointings;
      } /* end of case detSumFlag=1 pntSumFlag=0 */ 

      if (detSumFlag<0 || (!detSumFlag && pntSumFlag))
      {
	 for (idet=0; idet<pDet->maxdet; ++idet)
	 {
	    incr = irow*pDet->detchans;
	    if (pDet->det[idet] == 1)
	    {
	       sumsExposure[irow] += pGtrows->livetime[pointingNum][idet];
	       for (i=0; i<pDet->detchans; ++i)
	       {
		  sumsBuffer[incr+i] += pDet->hist[idet][i];
		  if (lastFlag)
		  {
	             sumsErr[incr+i] = sqrt((float)sumsBuffer[incr+i]);
		  }
	       }
	       if (pOutput->dtype != 1)
	       {
	          if (lastFlag)
		  {
	             if (sumsExposure[irow] > VERY_SMALL)
		     {
	        	for (i=0; i<pDet->detchans; ++i)
			{
			     fsumsBuffer[incr+i] = 
		     		sumsBuffer[incr+i]/sumsExposure[irow];
			     sumsErr[incr+i] /= sumsExposure[irow];
			     if (pOutput->dtype == 3)
			     {
		        	fsumsBuffer[incr+i] /=
				   (pBin->energy[i+1]-pBin->energy[i]);
				sumsErr[incr+i] /= 
				   (pBin->energy[i+1]-pBin->energy[i]);
			     }
			}
		     }
		  } 
	       } /* end of if dtype != 1 */
	       if (lastFlag)
	       {
		  sprintf(sumsid[irow], "DET_%d_SUM",idet);
	       }
	       ++irow;
	    }
	 }  /* end of detectors loop */
      }  /* end of case detSumFlag=0 pntSumFlag=1 */

      if (detSumFlag<0 || (detSumFlag && pntSumFlag))
      {
	 incr = irow*pDet->detchans;
	 for (idet=0; idet<pDet->maxdet; ++idet)
	 {
	    if (pDet->det[idet]==1)
	    {
	       sumsExposure[irow] += pGtrows->livetime[pointingNum][idet];
	       for (i=0; i<pDet->detchans; ++i)
	       {
		  sumsBuffer[incr+i] += pDet->hist[idet][i];
	       }
	    }
	 } /* end detector loop */
         if (lastFlag)
	 {
	    for (i=0; i<pDet->detchans; ++i)
	    {
	       sumsErr[incr+i] = sqrt((float)sumsBuffer[incr+i]);
	    }
	    if (pOutput->dtype != 1)
	    {
	       if (sumsExposure[irow] > VERY_SMALL)
	       {
	          for (i=0; i<pDet->detchans; ++i)
		  {
		     fsumsBuffer[incr+i] = sumsBuffer[incr+i]/sumsExposure[irow];
		     sumsErr[incr+i] /= sumsExposure[irow];
		     if (pOutput->dtype==3)
		     {
		        fsumsBuffer[incr+i] /= (pBin->energy[i+1]-pBin->energy[i]);
			sumsErr[incr+i] /= (pBin->energy[i+1]-pBin->energy[i]);
		     }
		  }
	       }
	    }
	    strcpy(sumsid[irow], "SUM_ALL");
	 }
	 ++irow;
      }  /* end of case detSumFlag=1 pntSumFlag=1 */
      
      if (lastFlag && extRows > 0)
      {
         status = DALtableAddRows(pTable, -1, extRows, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message,"Error while adding summation rows to table.");
	    Cleanup(status, message, *pBase, program);
	 }
	 for (i=0; i<extRows; ++i)
	 {
	    sumsSpecNum[i] = i;
	 }
	 status = DALtablePutColBins(pTable, "SPEC_NUM", 0, DAL_INT, startBin,
	 		0, extRows, sumsSpecNum, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error writing summed spectrum to SPEC_NUM");
	    Cleanup(status, message, *pBase, program);
	 }
	 status = DALtablePutColStrings(pTable, "ROWID", 0, startBin, 0,
	 		extRows, sumsid, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error writing summed spectrum to ROWID");
	    Cleanup(status, message, *pBase, program);
	 }
	 status = DALtablePutColBins(pTable, "EXPOSURE", 0, DAL_DOUBLE,
	 		startBin, 0, extRows, sumsExposure, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error writing summed spectrum to EXPOSURE");
	    Cleanup(status, message, *pBase, program);
	 }
	 Allocate(program, "-sumsChannels", 0, pDet->detchans*sizeof(int),
	 	(void **)&sumsChannels, pBase, pLast);
	 for (i=TLMIN; i<=tlmax; ++i)
	 {
	    sumsChannels[i] = i;
	 }
	 for (i=0; i<extRows; ++i)
	 {
	    status = DALtablePutColBins(pTable, "CHANNEL", 0, DAL_INT, 
	    	startBin+i, startBin+i, pDet->detchans, sumsChannels, status);
	 }
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error writing summed spectrum to CHANNEL");
	    Cleanup(status, message, *pBase, program);
	 }
	 if (pOutput->dtype == 1)
	 {
	    status = DALtablePutColBins(pTable, "COUNTS", 0, DAL_LONG, startBin,
	 		0, extRows*pDet->detchans, sumsBuffer, status);
	    if (status != ISDC_OK)
	    {
	       strcpy(message, "Error writing summed spectrum to COUNTS");
	       Cleanup(status, message, *pBase, program);
	    }
	 }
	 else 
	 {
	    status = DALtablePutColBins(pTable, "RATE", 0, DAL_FLOAT, startBin,
	 		0, extRows*pDet->detchans, fsumsBuffer, status);
	    if (status != ISDC_OK)
	    {
	       strcpy(message, "Error writing summed spectrum to RATE");
	       Cleanup(status, message, *pBase, program);
	    }
	 }
	 status = DALtablePutColBins(pTable, "STAT_ERR", 0, DAL_FLOAT,
	    	     startBin, 0, extRows*pDet->detchans, sumsErr, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error writing summed spectrum to STAT_ERR");
	    Cleanup(status, message, *pBase, program);
	 }
      }
   }  /* end if summations */
   firstFlag = 0;
}
