/* 
-------- Function: -----------------------------------------------------------------------

     CreatePHA_II


-------- Description: --------------------------------------------------------------------

     This program creates a new FITS file and writes a new SPECTRUM extension in type II 
     PHA format.  It also creates an EBOUNDS and POINTING extensions in the output file.  



-------- Usage: --------------------------------------------------------------------------

   Input Parameters:
       INPUTDS       dal_element     : Pointer to input data structure.
       ACCESS        dal_accessType  : ACCESS_UNKNOWN=-1, DAL_HTTP=1, DAL_FTP=2,
                                       DAL_SHMEM=3, DAL_MEM=4, DAL_DISK=5, DAL_ROOT=6
       BASEPTR       memoryPtr       : Pointer to the list that keeps track of allocated
                                       memory
       LASTPTR       memoryPtr       : Pointer to the current node in the list pointed to
                                       by BASEPTR
       POINTINGNUM   int             : Index to pointing
       DCOUNT        int             : # of rows in the DITHER extension in the input file
       DITHERLIST    ditherPtr       : Each node in this list corresponds to a single 
                                       pointing and contains the associated x,y,z-ra/dec 
				       values
       GTLIST        gtIntPtr      : Structs containing Good Time + Livetime info
       PGTROWS	     gtRowsPtr
       
       DETSUMFLAG   int		   : +1 Indicates all detectors within a
       				     pointing will be summed.
       PNTSUMFLAG   int		   : +1 Indicates each detector is summed
       				     across pointings.
				     A -1 for both indicates both cases
				     are output, along with the normal 
				     individual detector spectra.
       TOTPOINTINGS int		   : The total number of pointings that will
       				     be processed during spihist run.

                outputParsPtr pars
		------------------
       FNAME         string          : Name of the output FITS file
       CHATTER       int             : Chattiness flag for o/p (5 quite, 10 normal,
                                       >20 silly)
                detinfoPtr pars
		---------------
       DETCHANS      long            : Total number of possible detector channels

                headerParsPtr pars
		------------------
       INSTRUME      string          : String listing instrument/detector


   Output Parameters:
                headerParsPtr pars
		------------------
       EXTNAME       string          : Name of the EBOUNDS extension


-------- Include files -------------------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: ---------------------------------------------------

                 (3.0.8; May 03) ;Using new copyEbounds and Pointing functions.
		 (3.0.0; Dec 02) ;Removed multipointing option
   Craig Gordon  (2.2.0; Sep 02) ;pass extra parameters to createdataIIext.c
   Sandhia Bansal  (1.0.0; Aug 00)


--------------------------------------------------------------------------------------- */

#include "spihist.h"


void CreatePHA_II(dal_element *inputDS, dal_accessType access, memoryPtr *pBase, memoryPtr *pLast, 
		  int pointingNum, int dcount, ditherPtr ditherList, 
		  gtIntPtr **gtList, gtRowsPtr pGtrows, outputParsPtr pOutput, 
		  binningPtr pBin, detinfoPtr pDet, headerParsPtr pHeader, 
		  int detSumFlag, int pntSumFlag, int totPointings)
{
  
   int                    status=ISDC_OK;
   int                    numChildren=0;
   int                    i=0;
   int 			  lastFlag=0;

   static char            detnam[FLEN_VALUE];
   char                   *program="CreatePHA_II";
   char                   message[DAL_MAX_STRING];
   char                   *tmplate="SPI.-PHA2-GRP.tpl";

   static dal_element     *object=NULL;
   static dal_element     *child[MAX_ELEMENTS];

   static dal_baseType    baseType;



   lastFlag = (dcount < 0);
   if (!object)
   {
      strcpy(detnam, pHeader->instrume);

      /* use clobber flag to delete the pre-existing files */
      strcat(pOutput->fname, ".fits");
      status = CheckFile(pOutput->fname, pOutput->clobber);
      if (status != ISDC_OK)
      {
	 strcpy(message, "failed to create new output file");
	 Cleanup(status, message, *pBase, program);
      }

      status = DALobjectCreate(pOutput->fname, access, tmplate, &object, status);

      if (status != ISDC_OK)
      {
	 strcpy(message, "failed to create new object");
	 Cleanup(status, message, *pBase, program);
      }
      
      strcpy(message, "Program that created this FITS file");
      PutKeyword(object, "CREATOR", (void *)pHeader->tskstr, DAL_CHAR, message, program);

      /* get the number of child elements of the top level element and print out 
	 information about them */
      status = DALelementGetNumChildren(object, &numChildren, status);

      for (i=0; i<numChildren && status == ISDC_OK; i++)
      {
	 /* retrieve a pointer to the top level elements ith child element */
	 status = DALelementGetChild(object, i+1, child+i, &access, &baseType, status);
      }

      if (status != ISDC_OK)
      {
	 strcpy(message, "failed to get object information");
	 Cleanup(status, message, *pBase, program);
      }
      
      /* Won't be needing the RESPFILE column */
      DALtableDelCol(child[DATA_CHILD], "RESPFILE", 0, status);
      status = ISDC_OK;
   }

   CreateDataIIExt(child[DATA_CHILD], pBase, pLast, pointingNum, detnam, gtList,
		   pGtrows, pOutput, pBin, pDet, pHeader, detSumFlag, 
		   pntSumFlag,totPointings, lastFlag);

   if (lastFlag)
   {
      strcpy(message, " ... Successfully written the SPECTRUM extension");
      DispMsg(Log_1, message, status, program, pOutput->chatter);
      CommonStampObject(child[DATA_CHILD],"",status);

      /* Create the Ebounds extension */
      strcpy(pHeader->extname, "SPI.-PHA2-SPE");
      CopyEboundsExt(inputDS, object, pHeader, pBase, pOutput->chatter);

      /* create the pointing extension */
      CopyPointingExt(inputDS,object, pHeader, pBase, pOutput->chatter); 

      CommonStampObject(object,"",status);
      status = DALobjectClose(object, DAL_SAVE, ISDC_OK);
      if (status != ISDC_OK)
      {
         strcpy(message, "");
	 Cleanup(status, message, *pBase, program);
      }
   }
}
