/* 
-------- Function: -----------------------------------------------------------------------

     DataExtension


-------- Description: --------------------------------------------------------------------

     This function writes the SPECTRUM extension for a file in ISDC format. 


-------- Functions called: ---------------------------------------------------------------

   Spihist Functions:
      SpecKeywords            -- Puts keywords in DATA extension

   SPI  Functions:
      Allocate                -- Allocates specified number of bytes and keeps track of 
                                 all allocated memory during a run
      IdentifyProgram         -- Displays the name and version number of the current
                                 program
      Cleanup                 -- Deallocates all memory allocated during the current run
                                 in the event of an error and exits the program
      DispMsg                 -- Writes messages to the screen and a log file
      PutKeyword              -- Writes specified keyword to the table header
      FreeNode                -- Frees some of the allocated memory when not needed

   DAL Functions:
      DALobjectFindElement    -- Finds element of a DAL object with specified name and
                                 returns a pointer to the element
      DALtableAddCol          -- Adds a column to the current table
      DALtableAddRows         -- Adds specified number of rows to current col
      DALtablePutColBins      -- Writes the specified data to column


-------- Usage: --------------------------------------------------------------------------

   void DataExtension(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast,
                      int chatter, detinfoPtr pDet, headerParsPtr pHeader)

   Input Parameters:
       INPUTDS      dal_element  : Pointer to the Observation group DOL
       PBASE        memoryPtr    : Pointer to the list that keeps track of allocated
                                   memory
       PLAST        memoryPtr    : Pointer to the current node in the list pointed to by
                                   PBASE
       CHATTER      int          : Chattiness flag for o/p (5 quite, 10 normal,
                                   >20 silly)

                detinfoPtr pars
		---------------
       DETCHANS     long         : Total number of possible detector channels 
       MAXDET       int          : Total number of det for the instrument
       DET          int          : Det[i]=1 if ith detector is requested to be processed;
                                   Valid values are 0 and 1.
       HIST         long         : Hist[i]!=0 if a histogram was accumulated for ith det

                headerParsPtr pars
		------------------
       TSKSTR       char         : Name+version of the task
       CHANTYP      char         : Type of detector channels (PHA, PI etc)


-------- Include files -------------------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: ---------------------------------------------------

		 (3.0.0; Dec 02)  removed multipointing option
   Craig Gordon  (2.2.0; Aug 02)  change COUNTS output from long to 
   					unsigned long
   Craig Gordon  (2.1.8; May 02)   ;Now calls CommonStampObject
   Sandhia Bansal  (1.0.0; Aug 00)


--------------------------------------------------------------------------------------- */

#include "spihist.h"


#define NK_HISTORY     1  
              /* number of history records to be created in the ebounds ext */
#define NK_COMM        1       
              /* number of comment records to be created in the ebounds ext */




void DataExtension(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
		   int chatter, detinfoPtr pDet, headerParsPtr pHeader)
{
   static float        *statErr=NULL;

   static long         *data=NULL;
   long                total=0;
   long                nRows=0;
   long                lkeyval=0L;

   static int          startBin=1;
   int                 status=ISDC_OK;
   int                 i=0;
   int                 nch = 0;
   int                 idet=0;
   int                 tlmax=TLMIN+pDet->detchans-1;

   char                *program="DataExtension";
   char                *version="1.0.0";
   char                comment[FLEN_COMMENT]="\0";
   char                **history = NULL;
   char                **commkeywd = NULL;
   char                message[DAL_BIG_STRING];
   char                *extname="SPECTRUM";

   static dal_element  *dataPtr=NULL;




   if (!data)
   {
      IdentifyProgram(program, version, chatter);
      
      /* Find data and pointing extensions */
      status = DALobjectFindElement(inputDS, "SPI.-OBS.-DSP", &dataPtr, status);

      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while finding SPI.-OBS.-DSP extension");
	 Cleanup(status, message, *pBase, program);
      }

      if (NK_HISTORY > 0)
      {
	 Allocate(program, "-history", NK_HISTORY, FLEN_COMMENT, (void **) &history, pBase,
		  pLast);

	 strcat(strcat(strcat(strcat(strcpy(history[0], extname), 
				     " extension written by "),
			      program), 
		       " Version "), 
		version);
	 for (i=1; i<NK_HISTORY; i++)
	    strcpy(history[i], "\0");

	 strcpy(comment, "\0");
	 
	 for (i=0; i<NK_HISTORY; i++)
	    PutKeyword(dataPtr, "HISTORY", history[i], CHAR, comment, program);
	 FreeNode(pBase, (void **) &history, pLast);
      }

      if (NK_COMM > 0)
      {
	 Allocate(program, "-commkeywd", NK_COMM, FLEN_COMMENT, (void **)&commkeywd, pBase,
		  pLast);

	 strcpy(commkeywd[0], "This extension contains the spectra");
	 for (i=1; i<NK_COMM; i++)
	    strcpy(commkeywd[i], "\0");

	 strcpy(comment, "\0");
      
	 for (i=0; i<NK_COMM; i++)
	    PutKeyword(dataPtr, "COMMENT", commkeywd[i], CHAR, comment, program);
	 FreeNode(pBase, (void **) &commkeywd, pLast);
      }
	 
      /* Add COUNTS and STAT_ERR columns */
      status = DALtableAddCol(dataPtr, "COUNTS", DAL_ULONG, (long) pDet->detchans, status); 

      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while adding COUNTS to the table");
	 Cleanup(status, message, *pBase, program);
      }

      status = DALtableAddCol(dataPtr, "STAT_ERR", REAL_E, (long) pDet->detchans, status);

      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while adding STAT_ERR to the table");
	 Cleanup(status, message, *pBase, program);
      }

      /* Copy keywords from inputDS to the EVENTS extension */
      /*SpecKeywords(inputDS, dataPtr, *pBase);*/

      strcpy(comment, "Program creating PHA file");
      PutKeyword(dataPtr, "CREATOR", pHeader->tskstr, CHAR, comment, program);

      lkeyval = (long) pDet->detchans;
      strcpy(comment, "Total number of detector channels");
      PutKeyword(dataPtr, "EBIN_NUM", &lkeyval, INT_J, comment, program);

      strcpy(comment, "channel type (PHA, PI etc)");
      PutKeyword(dataPtr, "CHANTYPE", pHeader->chantype, CHAR, comment, program);

      strcpy(message, " ... Written the extension header keywords");
      DispMsg(Log_1, message, status, program, chatter);
      
      CommonStampObject(dataPtr, NULL, status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error during CommonStampObject");
	 Cleanup(status, message, *pBase, program);
      }

      /* Allocate memory for data buffers */
      Allocate(program, "-statErr", 0, 
	       pDet->maxdet*pDet->detchans*sizeof(float), (void **) &statErr, 
	       pBase, pLast);
      Allocate(program, "-data", 0, pDet->maxdet*pDet->detchans*sizeof(long), 
	       (void **) &data, pBase, pLast);
   }

   /* Determine the number of rows to be added at the end of the table */
   nRows = 0;
   nch   = 0;

   for (idet=0; idet<pDet->maxdet; idet++)
   {
      if (pDet->det[idet] == 1)
      {
	 nRows++;

	 for (i=TLMIN; i<=tlmax; i++)
	 {
	    data[nch]      = pDet->hist[idet][i];
	    statErr[nch++] = sqrt((float)(pDet->hist[idet][i]));
	 }
      }
   }

   if (nRows > 0)
   {
      /* We must add the desired rows. 
	 Here we add nRows rows to the end of the table */
      status = DALtableAddRows(dataPtr, -1, nRows, status);

      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while adding rows to the table.");
	 Cleanup(status, message, *pBase, program);
      }

      total = nch;
      status = DALtablePutColBins(dataPtr, "COUNTS", 0, DAL_ULONG, startBin, 0, total, data, 
				  status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while writing COUNTS");
	 Cleanup(status, message, *pBase, program);
      }

      status = DALtablePutColBins(dataPtr, "STAT_ERR", 0, REAL_E, startBin, 0, total, 
				  statErr, status);	
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while writing STAT_ERR");
	 Cleanup(status, message, *pBase, program);
      }

      startBin += nRows;
   }
}
