/* 
-------- Function: -------------------------------------------------------------

     GetGTForSCW


-------- Description: ----------------------------------------------------------
     
     This subroutine copies into a local array the values of the good time 
     intervals for the particular science window of interest, and then searches
     that array to find the gaps where none of the detectors have a good time.
     This is done by performing a sort on the OBSTART values given in the
     gti file.  
     
     The good time intervals (in  OBTime format)of the OVERALL collection of
     detectors are then placed in the dynamically allocated obgtStart and Stop
     arrays, for the purpose of  events selection. The number of intervals is
     returned. 
   
--------------------------------------------------------------------------------
   
   Input Parameters:
       GTLIST        gtIntPtr     : Pointer to all of the good time values
       PGTROWS       gtRowsPtr    : More good time information
       ISCW          int          : Scw index number
       PBASE         memoryPtr    : Pointer to the list that keeps track of
                                    allocated memory
       PLAST         memoryPtr    : Pointer to the current node in the list
                                    pointed to by the PBASE
       PDET	     detinfoPtr   : Contains OB spectra information
   Output Parameters:
       OBGTSTART       OBTime     : Arrays containing the gti start and
       OBGTSTOP        OBTime         stop times, in OBT format

   Return Value:
      The number of good time intervals (equals the number of valid elements)
      in OBGTSTART and STOP
      
-------- Authors/Modification History: -----------------------------------------

 		 (3.0.4; Feb 03) - Using new gti structs to reduce mem alloc.
		 		   Also sort using OB format, not Julian date.
   		 (3.0.0; Dec 02) - Add check for OB spectra
   Craig Gordon  (2.1.9; Jul 02) - Removed the constraint that all detectors
   				   in a SCW should have the same GTIs.  				   
   Craig Gordon  (2.1.6; Mar 02)
   Craig Gordon  (2.1.5; Feb 02)


----------------------------------------------------------------------------- */

#include "spihist.h"

/* Helper function to increase the size of the dynamically allocated
   obgtStart and Stop arrays */
static void Resize(memoryPtr *pBase, memoryPtr *pLast, OBTime **obgtArray, 
			int *cap);

int GetGTForSCW (memoryPtr *pBase, memoryPtr *pLast, gtIntPtr **gtList, 
		gtRowsPtr pGtrows, int iScw, detinfoPtr pDet, 
		OBTime **obgtStart, OBTime **obgtStop)
{

   int		i,j,idx, sz, cap, maxdet, numIntv=0;
   extern const int   gl_boundaries[2][6];
   extern const int   gl_numbnds;
   extern const short gl_oper;
   const int    CAP = 20;
   OBTime	maxObStop;
   gtIntPtr	gtNodePtr=NULL;
   gtIntPtr     *intervals=0;
   char		*program="GetGTForSCW";
   
   maxdet = pDet->maxdet;
   /* Use malloc/free instead of Allocate for this array of pointers to 
      structures.  This will allow faster cleanup, and anyway the intervals 
      array is local to this function only. */
   intervals = (gtIntPtr *)malloc((pGtrows->total+1)*sizeof(gtIntPtr));
   
   Allocate(program, "-obgtStart", 0, CAP*sizeof(OBTime),
		(void **) obgtStart, pBase, pLast);
   Allocate(program, "-obgtStop", 0, CAP*sizeof(OBTime),
		(void **) obgtStop, pBase, pLast);

   
   /* Intervals array is 1-based as required by hsort.  The zero element 
      is not used. */
   intervals[0] = 0;
   sz = 0;   
   for (i=0; i<maxdet; i++)
   {
      if (IsSingleOrPsd(i, &idx))
      {
         if (idx < gl_boundaries[0][0]) /* dets 0-18 */
	 {
	    if (pDet->spiMode[idx] != gl_oper)
	       continue;
	 }
	 else  /* dets 85-103 */
	 {
	    if (pDet->spiMode[idx] != gl_oper || 
	    		pDet->obSpectraFlag[idx] != 0)
	       continue;
	 }
      }
      for (j=0; j<pGtrows->nInts[iScw][i]; j++)
      {
         gtNodePtr = &gtList[iScw][i][j];
	 intervals[++sz] = (gtIntPtr)malloc(sizeof(GTINT));
	 intervals[sz]->obStart = gtNodePtr->obStart;
	 intervals[sz]->obStop = gtNodePtr->obStop;
      }
   }
   for (i=sz+1; i<=pGtrows->total; i++)
   {
      intervals[i] = 0;
   }
   
   if (sz > 0)
   {
      Hsort(intervals, sz);
      
      i=1;
      numIntv=0;
      cap = CAP;
      while (i<=sz)
      {
         ++numIntv;
	 if (numIntv > cap)
	 {
	    Resize(pBase, pLast, obgtStart, &cap);
	    cap /=2;  /*  since each call to Resize doubles cap */
	    Resize(pBase, pLast, obgtStop, &cap);
	 } 
	 
         (*obgtStart)[numIntv-1] = intervals[i]->obStart;
	 maxObStop = intervals[i]->obStop;
	 while (++i<=sz && intervals[i]->obStart <= maxObStop)
	 {
	    if (intervals[i]->obStop > maxObStop)
	    {
	       maxObStop = intervals[i]->obStop;
	    }
	 }
	 (*obgtStop)[numIntv-1] = maxObStop;	 
      }
      for (i=numIntv; i<cap; i++)
      {
         (*obgtStart)[i] = 0;
	 (*obgtStop)[i] = 0;
      }
   }
      
   for (i=1; i<=sz; i++)
   {
      free(intervals[i]);
   }
   free(intervals);

   return numIntv;		
}


void Resize(memoryPtr *pBase, memoryPtr *pLast, OBTime **obgtArray, 
			int *cap)
{
   OBTime *newArray=0;
   int i, oldSz;
   char *program = "Resize";
   
   oldSz = (*cap);
   (*cap) *= 2;
   Allocate(program, "-newObArray", 0, (*cap)*sizeof(OBTime),
		(void **) &newArray, pBase, pLast);  
   for (i=0; i<oldSz; i++)
   {
      newArray[i] = (*obgtArray)[i];      
   }   
   FreeNode(pBase, (void **)obgtArray, pLast);
   *obgtArray = newArray; 
   
}
