/*----------- Function -------------------------------------------------

   GetObSpectra

------------- Description ----------------------------------------------

   Open the appropriate On-board spectra file (Oper or Emer mode) and read
   the information into an array of obSpectrum structures.
   
------------------------------------------------------------------------
   
   Input Parameters:
   	SWG		dal_element*		:Input scw w/ ob spectra
	PDET		detinfoPtr		:Detector information
   
   Output Parameters:
	PSPECTRA	obSpectrumPtr*		:Pointer to array of OB
					 	 spectrum structures 
	NSPECTRA	int			:Size of OB spectra array
	
   Return Value:
   	ISDC_OK if no error
	
------------- Authors/Modification History: ----------------------------

   Craig Gordon   (3.0.0;  Dec 02)

------------------------------------------------------------------------
*/   

#include "spihist.h"

int GetObSpectra(dal_element *swg, detinfoPtr pDet, obSpectrumPtr **pSpectra, 
			long *nSpectra, outputParsPtr pOutput)
{
   int  status = ISDC_OK;
   long i, j, numRows=0, newStart=0;
   unsigned char *buffer1=NULL;
   unsigned short *buffer2=NULL;
   OBTime *buffer3=NULL;
   char rawName[DAL_MAX_STRING];
   char prpName[DAL_MAX_STRING];
   char msg[DAL_MAX_STRING];
   char *program="GetObSpectra";
   dal_element *obRaw=NULL, *obPrp=NULL;
   dal_dataType type;
   /* Assumption is that obOperFlag and obEmerFlag cannot both be set to 1.
      This is checked for early on in spihist.c.  Also, if neither is 1, 
      this function should not be reached. */
   if (pDet->obOperFlag)
   {
      strcpy(rawName, "SPI.-ACOF-RAW");
      strcpy(prpName, "SPI.-ACOF-PRP");
   }
   else
   {
      strcpy(rawName, "SPI.-ACON-RAW");
      strcpy(prpName, "SPI.-ACON-PRP");
   }
   
   do
   {
      status = DALobjectFindElement(swg, rawName, &obRaw, status);
      status = DALobjectFindElement(swg, prpName, &obPrp, status);
      if (status != ISDC_OK)
      {
	/* strcpy(msg, "Unable to find raw and/or prp OB spectra data structures.");
	 DispMsg(Error_2, msg, status, program, pOutput->chatter);
        */
	status = 1;
	 continue;
      }
      status = DALtableGetNumRows(obRaw, &numRows, status);
      if (status != ISDC_OK || !numRows)
      {
         strcpy(msg, "Unable to read rows in OB spectra table.");
	 DispMsg(Error_2, msg, status, program, pOutput->chatter);
	 status = (status==ISDC_OK) ? -1 : status;
	 continue;	 
      }
      
      /* In case this has been called already for a previous swg, set newStart 
         to where the new rows will be appended.  In that case, *pSpectra != 0  
	 and DALallocateDataBuffer will call realloc on pSpectra */
      newStart = *nSpectra;
      *nSpectra += numRows;
      
      DALallocateDataBuffer((void **)pSpectra, *nSpectra*sizeof(obSpectrumPtr), status);
      for (i=newStart; i<*nSpectra; ++i)
      {
         (*pSpectra)[i] = 0;
         DALallocateDataBuffer((void **)(*pSpectra+i), sizeof(OBSPECTRUM), status);
      }
      DALallocateDataBuffer((void **)&buffer1, numRows*sizeof(unsigned char), status);
      DALallocateDataBuffer((void **)&buffer2, 
      		numRows*SPIHIST_NCHANS*sizeof(unsigned short), status);
      DALallocateDataBuffer((void **)&buffer3, numRows*sizeof(OBTime), status);
      
      type = DAL_BYTE;
      status = DALtableGetCol(obRaw, "DET_ID", 0, &type, 0, buffer1, status);
      for (i=0; i<numRows; ++i)
      {
         (*pSpectra)[newStart+i]->det_id = buffer1[i];
      }
      status = DALtableGetCol(obRaw, "E_RANGE", 0, &type, 0, buffer1, status);
      for (i=0; i<numRows; ++i)
      {
         (*pSpectra)[newStart+i]->e_range = buffer1[i];
      }
      type = DAL_USHORT;
      status = DALtableGetCol(obRaw, "COUNTS", 0, &type, 0, buffer2, status);
      for (i=0; i<numRows; ++i)
      {
         for (j=0; j<SPIHIST_NCHANS; ++j)
	 {
            (*pSpectra)[newStart+i]->counts[j] = buffer2[i*SPIHIST_NCHANS+j];
	 }
      }
      status = DAL3GENtableGetOBT(obPrp, "OBT_START", 0, 0, buffer3, status);
      for (i=0; i<numRows; ++i)
      {
         (*pSpectra)[newStart+i]->obt_start = buffer3[i];
      }
      status = DAL3GENtableGetOBT(obPrp, "OBT_END", 0, 0, buffer3, status);
      for (i=0; i<numRows; ++i)
      {
         (*pSpectra)[newStart+i]->obt_end = buffer3[i];
      }
      if (status != ISDC_OK)
      {
         strcpy(msg, "Error reading in OB spectra data");
	 DispMsg(Error_2, msg, status, program, pOutput->chatter);
	 continue;
      }
            
   } while (0);
   
   if (buffer1)
   {
      DALfreeDataBuffer(buffer1, ISDC_OK);
   }
   if (buffer2)
   {
      DALfreeDataBuffer(buffer2, ISDC_OK);
   }
   if (buffer3)
   {
      DALfreeDataBuffer(buffer3, ISDC_OK);
   }
   return status;
}
