/* 
-------- Function: -----------------------------------------------------------------------

    GetValidGT


-------- Description: --------------------------------------------------------------------

     The program searches through the arrays containing GOOD TIMES to extract valid times.
     It outputs an array of lists indexed at pointing number and detector id.  Each node
     of the lists contains start and stop GOOD TIMES and livetime associated
     with a pointing and detector.  It also returns a list of detector ids read from the 
     first pointing - for ISDC output format only.

     ASSUMPTION:
     -----------
     Detector list for each pointing in the GOOD TIMES table is identical.


-------- Functions called: ---------------------------------------------------------------

   SPIHIST Functions:
      Livetime               -- Reads LIVETIME from SPI.-OBS.-DTI extension

   SPI Functions:
      Allocate               -- Allocates specified number of bytes and keeps track of all
                                allocated memory during a run
      Cleanup                -- Deallocates all memory allocated during the current run in
                                the event of an error
      FreeNode               -- Frees some of the allocated memory when not needed


   DAL3 Functions:
      DAL3AUXconvertOBT2IJD  -- Converts times from OBTime to IJD format


-------- Usage: --------------------------------------------------------------------------

   Input Parameters:
       INPUTDS       dal_element      : Pointer to the Observation group DOL
       PBASE         memoryPtr        : Pointer to the list that keeps track of allocated
                                        memory
       PLAST         memoryPtr        : Pointer to the current node in the list pointed to
                                        by PBASE
       OBSTART       OBTime           : Array containing start times
       OBSTOP        OBTime           : Array containing stop times
       DETID         long             : NGT detector ids read from GTI table
       POINTINGID    char **          : IDs from each from of GTI table
       MAXNGTI	     long	      : max number of GTIs

   Output Parameters:
       GTLIST        array of gtPtr   : Pointer to array of GTI structs
       PGTROWS       gtRowsPtr        : Livetime and nInt info for GT table rows.
                                        
             goodTimesPtr pars (NULL for PHA, non-NULL for ISDC output format)
	     -----------------------------------------------------------------
       DETID         int              : Array of detector ids for the first pointing


-------- Include files -------------------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: ---------------------------------------------------
   
                 (3.1.2; Jan 04) ;New criteria for determining end of pointing.
                                  Previously it was determined by a change of
                                  pointing_id string in gti file.  For new split
                                  pointings, this no longer works.  Now looks for
                                  a negative change in detector number from the
                                  row above.
                 (3.0.4; Feb 03) Using new gtint, gtrow structs to reduce
		                 memory allocation.
   		 (3.0.0; Dec 02) ;New gti scheme, dets 85-141 no longer have
		                   same gti's as 0-18
   Craig Gordon  (2.1.6; Mar 02)
   Craig Gordon  (2.1.5; Feb 02)
   Craig Gordon  (2.1.3; Oct 01)
   Craig Gordon  (2.1.2; Sep 01)
   Sandhia Bansal  (1.0.0; Aug 00)


--------------------------------------------------------------------------------------- */

#include "spihist.h"


void GetValidGT(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
		OBTime *obStart, OBTime *obStop, long *detid, 
		goodTimesPtr pGtimes, char **pointingId, 
		long maxnGTI, gtIntPtr **gtList, gtRowsPtr pGtrows)
{
   int     ipointing=0, idet=0, igtRow=0, igtInt=0, i=0, j=0;
   int     status=0;
   long    prevDetId;
   double  *livetime=0;
   char    *program="GetValidGT";
   char    message[DAL_MAX_STRING];
   
   gtIntPtr   currNode=NULL;

   Livetime(inputDS, pBase, pLast, &livetime);
   
   strcpy(pGtrows->pointingId[0], pointingId[0]);
   prevDetId = detid[0] - 1;
   for (igtRow=0; igtRow<pGtrows->nRows; igtRow++)
   {
      if (prevDetId >= detid[igtRow])
      {
	 ipointing++;
	 strcpy(pGtrows->pointingId[ipointing], pointingId[igtRow]);
	 idet=0;
      }
      if ((ipointing==0) && (pGtimes))
	 pGtimes->detid[idet++] = detid[igtRow];
	 
      /* Determine number of valid GTIs in each row of the GTI table */	 
      igtInt = 0;
      for (i=0; i<maxnGTI; i++)
      {
         if ((obStart[j+igtInt] == DAL3_NO_OBTIME) ||
	     (obStop[j+igtInt] == DAL3_NO_OBTIME))
	 {
	    break;
	 }
	 ++igtInt;
      }
      pGtrows->nInts[ipointing][detid[igtRow]] = igtInt;
      pGtrows->total += igtInt;
      if (igtInt)
      {
         Allocate(program, "-gtList[i][j]", 0, igtInt*sizeof(GTINT),
	 	(void **) &gtList[ipointing][detid[igtRow]], pBase, pLast);
	 for (i=0; i<igtInt; ++i)
	 {
	    currNode = &gtList[ipointing][detid[igtRow]][i];
	    currNode->obStart = obStart[j+i];
	    currNode->obStop = obStop[j+i];
	 }
	 pGtrows->livetime[ipointing][detid[igtRow]] = livetime[igtRow];
      }
      j += maxnGTI;
      prevDetId = detid[igtRow];
   }
   FreeNode(pBase, (void **) &livetime, pLast);

}


