/* 
-------- Function: -----------------------------------------------------------------------

     Livetime


-------- Description: --------------------------------------------------------------------

     This subroutine reads the LIVETIME column from the SPI.-OBS.-DTI table.
   

-------- Functions called: ---------------------------------------------------------------

   SPI Functions:
      Allocate                 -- Allocates specified number of bytes and keeps track of 
                                  all allocated memory during a run
      Cleanup                  -- Deallocates all memory allocated during the current run
                                  in the event of an error and exits the program

   DAL Functions:
      DALobjectFindElement     -- Finds element of a DAL object with specified name and
                                  returns a pointer to the element
      DALtableComputeBuffSize  -- Computes size of a data transfer buffer in bytes needed
                                  to hold the data of table column starting and ending at
				  specified bins
      DALtableGetCol           -- Retrive all data values from the table column


-------- Usage: --------------------------------------------------------------------------

   void Livetime(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
                 double **livetime)

   Input Parameters:
       INPUTDS       dal_element  : Pointer to the Observation group DOL
       PBASE         memoryPtr    : Pointer to the list that keeps track of allocated
                                    memory
       PLAST         memoryPtr    : Pointer to the current node in the list pointed to by
                                    PBASE


   Output Parameters:
       LIVETIME      double       : Array containing livetimes


-------- Include files -------------------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: ---------------------------------------------------

   Sandhia Bansal  (1.0.0; Aug 00)


--------------------------------------------------------------------------------------- */

#include "spihist.h"


void Livetime(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, double **livetime)
{
   long          buffSize=0L;
   long          ndet=0L;

   int           status=0;

   char          extname[DAL_MAX_ATT_STR_VALUE_SIZE];
   char          message[DAL_MAX_STRING];
   char          *program="Livetime";

   dal_element   *tablePtr=NULL;

   dal_dataType  type=DAL_DT_UNKNOWN;


   /* Locate the DEADTIME extension */
   strcpy(extname, "SPI.-OBS.-DTI");
   tablePtr = NULL;
   status = DALobjectFindElement(inputDS, extname, &tablePtr, status); 

   if (status != ISDC_OK)
   {
      strcat(strcat(strcpy(message, "Error while finding "),
		    extname), 
	     " extension");
      Cleanup(status, message, *pBase, program);
   }

   /* Read LIVETIME column */
   status = DALtableComputeBuffSize(tablePtr, "LIVETIME", 0, 1, 0, &buffSize, status);

   if (status != ISDC_OK)
   {
      strcpy(message, "Error while computing buffer size for LIVETIME");
      Cleanup(status, message, *pBase, program);
   }

   Allocate(program, "-LIVETIME", 0, (int)buffSize, (void **) livetime, pBase, pLast);

   type = DAL_DT_UNKNOWN;
   status = DALtableGetCol(tablePtr, "LIVETIME", 0, &type, &ndet, *livetime, status);

   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading livetime");
      Cleanup(status, message, *pBase, program);
   }
}
