/*----------- Function -------------------------------------------------

   ParseNumString
   
------------- Description ----------------------------------------------

   This function parses a user-input string of integers for selection 
   purposes (ie detector numbers, or pointing row numbers).  The format
   allows for the use of hyphens to indicate consecutive rows.  For 
   example, "-3, 5, 8-10, 15-"  will select the first relevant number
   (indicated by a -1 in the rangeLow array) through to the 3rd, 5, 8, 9,
   10, as well as 15 through to the last (a -1 in the rangeHigh array).  
   A bare hyphen, "-", can be specified to select an entire range.  All
   numbers specified are assumed to be in sorted order.
   
------------------------------------------------------------------------
   
   Input Parameters:
      INPUT		char*	:Input string to be parsed.
   Output Parameters:
      RANGEHIGH,	int*	:Integer arrays that will specify the
      RANGELOW                   upper and lower bounds of each
				 consecutive sequence.   
      NUMRANGES		int	:The length of the range arrays
      
   Return Value:
      ISDC_OK if executed properly
      
------------- Authors/Modification History: ----------------------------

   Craig Gordon   (2.2.0; Aug 02)
   
------------------------------------------------------------------------
*/   
#include "spihist.h"

int ParseNumString(char *input, int *rangeLow, int *rangeHigh, int *numRanges)
{
   int i, iStart, len, lastNonBlank, numBlanks, hyphenCount;
   int tokLength, status=ISDC_OK;
   char prevChar, currChar, *token=0, *hyphPtr=0, comma[]=",";
   char *program="ParseNumString";
   
   /* Get the string length, without any trailing blanks. */
   len = strlen(input);
   for (i=len-1; i>=0; --i)
   {
      if (!isspace(input[i]))   break;
   }
   len = i+1;
   if (len == 0)  return -1;
   /* Find first non-blank */
   for (i=0; i<len; ++i)
   {
      if (!isspace(input[i]))  break;
   }
   iStart=i;
   
   /* Now remove any blanks from the interior of string, but return an
      error for the case of  <digit> <sp> <digit>.  */
   i = lastNonBlank = iStart;
   numBlanks=0;
   while (i<len)
   {
      if (isspace(input[i]))
      {
         /* Proceed to next non-blank - there must be one or this block
	    couldn't have been reached. */
	 do
	 {
            ++numBlanks;
	    ++i;
	 }
	 while (isspace(input[i])); 
	 if (isdigit(input[lastNonBlank]) && isdigit(input[i]))
	 {
	    return -1;
	 } 
      }
      lastNonBlank = i;
      if (numBlanks)
      {
	 input[i-numBlanks] = input[i];
      }
      ++i;
   }
   len -= numBlanks;
   input[len] = 0;
   
   
   /* 2nd pass through the string - determines the number of ranges, and
      does further error detection. */
   prevChar = hyphenCount = 0;
   *numRanges = 1;
   for (i=iStart; i<len; ++i) 
   {
      currChar = input[i];
      if (currChar == '-')
      {
        /* There must not be two or more hyphens between any pair of commas. */
	if (++hyphenCount > 1)
	{
	   return -1;
	}
	if (prevChar == ',')
	{
	  /* Do not allow "....,-nn,..." */
	   return -1;
	}
	if (i!=(len-1) && !isdigit(input[i+1]))
	{
	   /* Do not allow "...,nn-,..."*/
	   return -1;
	}
      }
      else if (currChar == ',')
      {
         if (prevChar == ',')
	 {
	    return -1;
	 }
	 if (i == iStart || i == (len-1))
	 {
	    return -1;
	 }
	 ++(*numRanges);
	 hyphenCount = 0;
      }
      else if (!isdigit(currChar))
      {
         return -1;
      }
      prevChar = currChar;
   }
   
   if (*numRanges >= MAXRANGE)
   {
      return -1;
   }
   /* Final pass through string - fill in the range arrays. */
   token = strtok(&input[iStart],comma); 	   	
   for (i=0; i<*numRanges; i++)
   {
      if (!token)
      {
         return -1;
      }
      if (hyphPtr = strchr(token,'-'))
      {
         tokLength = strlen(token);
         if (tokLength==1)  /* "-" */
	 {
	    rangeLow[i] = rangeHigh[i] = -1;
	 }
	 else if (hyphPtr == token)  /* "-nn,.." */
	 {
	    rangeLow[i] = -1;
	    rangeHigh[i] = atoi(token+1);
	 }
	 else if (hyphPtr == &token[tokLength-1])  /* "...,nn-" */
	 {
	    *hyphPtr = 0;
	    rangeLow[i] = atoi(token);
	    rangeHigh[i] = -1;
	 }
	 else                      /* "...,nn-nn,..." */
	 {
	    *hyphPtr = 0;
	    rangeLow[i] = atoi(token);
	    rangeHigh[i] = atoi(++hyphPtr);
	 }
      }
      else 
      {
         rangeLow[i] = rangeHigh[i] = atoi(token);
      }
      token = strtok(NULL,comma);
   }

   return ISDC_OK;
}
