/* 
-------- Function: -------------------------------------------------------------

     ProcessData


-------- Description: ----------------------------------------------------------

     This subroutine looks at each data row and puts the events in appropriate
     detectors to create time histories.


-------- Functions called: -----------------------------------------------------

   SPIHIST Functions:
      TimeInGT          -- Checks whether specified time is a GOOD TIME
      TimeHist          -- Creates a time history

   SPI Functions:
      PseudoDetLookup   -- Returns the detector number that created ith event 
                           along with the corresponding energy


-------- Usage: ----------------------------------------------------------------

   Input Parameters:
       POINTINGNUM   int          : Index to the pointing in the SPI.-OBS.-PNT
                                    extension
               binningPtr pars
	       ---------------
       NBINS         int          : Number of bins in the binning file
       ENERGY        float        : Array contining bin boundaries read from the
                                    binning file

               detinfoPtr pars
	       ---------------
       HIST          long         : Hist[i]!=0 if a histogram was accumulated 
                                    for ith detector
   Output Parameters:
               detinfoPtr pars
	       ---------------
       DET0          short        : Id of the selected detector
       DET1          short        : Id of the selected detector (PSD only)
       PI0           float        : Energy of the selected detector



       EVENTLIST     list         : List of all events within the current 
                                    pointing

               eventsInfoPtr pars
	       ------------------
       NEVENT        long         : Number of events in the EVENTS extension
       TIME          double       : Array containing the times read from the 
                                    EVENTS extension
       PI            float        : Array containing the energies read from the 
                                    EVENTS extension
       DETE          short        : Array containing the detector ids read from
                                    the EVENTS extension
       MULT          short        : Array containing the multiplicities read 
                                    from the EVENTS extension
       DELT          long         : Array containing the DELT column from the 
                                    EVENTS extension

               detinfoPtr pars
	       ---------------
       DETOFFSET     int          : 1 if the detector ids are 0 based, 
                                    0 otherwise
       DETECTORS     int          : Array containing user-assigned detector# for
                                    multi-event processing - used when user 
				    specifies a multi-detector file
       NDET          int          : Number of valid input detector ranges
       DETRANGE1     int          : Array of the lower bounds on detectors
       DETRANGE2     int          : Array of the upper bounds on detectors
       DET0          short        : Id of the selected detector
       PI0           float        : Energy of the selected detector


               binningPtr pars
	       ---------------		
       ENERGY        float        : Array contining bin boundaries read from the
                                    binning file


   Output Parameters:
       DET0          short        : Id of the selected detector
       PI0           float        : Energy of the selected detector

               detinfoPtr pars
	       ---------------
       HIST          long         : Hist[i]!=0 if a histogram was accumulated 
                                    for ith detector


-------- Include files ---------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: -----------------------------------------
   
                 (3.0.4; Feb 03)  New interface, using gtInt and gtRows structs
                 (3.0.0; Dec 02)  Many modifications to take into account
		                  new gti scheme and reading of OB spectra.
				  Also, removed multipointing option.
   Craig Gordon  (2.2.1; Nov 02)  The TimeInGT function now takes the event
   					time in OBT format rather than IJD.
   Craig Gordon  (2.1.3; Oct 01)
   Craig Gordon  (2.1.1; Aug 01)
   Sandhia Bansal  (1.0.0; Aug 00)

----------------------------------------------------------------------------- */

#include "spihist.h"


void ProcessData(int pointingNum, binningPtr pBin, detinfoPtr pDet, 
			eventsInfoPtr pEvents, gtIntPtr **gtList, 
			gtRowsPtr pGtrows, int *evCount)
{

   int irow = 0; 
   int addFlag = 0;
   int idx = 0;
   extern const short gl_oper;
   extern const int gl_boundaries[2][6];
   
   while (irow < pEvents->nEVENT)
   {
      addFlag = 0;
      /* Determine detector number from the pseudo detector look up table */ 
      PseudoDetLookup(irow, pEvents, pDet);
      /* If a detector number is found in the table, create time history
	 after making sure that this event was created within a good time
	 interval */
      /*printf("det0:%d pi0: %f\n", pDet->det0, pDet->pi0);*/
      if (pDet->det0 >= 0)
      {
	 /* For PSDs, PseudoDetLookup returns pseudodetectors
	 corresponding to either 10XX or 11XX.  Either of these 
	 should also be placed in pseudodetectors 0-18. */
	 if (pDet->multiplicity == 0)
	 {
	    if (TimeInGT(pointingNum, pDet->det0, pEvents->obTime[irow],
		  gtList, pGtrows))
	    {
	       if (Timehist(pBin, pDet))
	       {
	          ++(*evCount);
		  addFlag = 1;
	       }
	    }
	    /* Now select the appropriate 0-18 detector and attempt to 
	       bin again. */	    	    
	    pDet->det0 = pEvents->dete[irow];
	    if (pDet->spiMode[pDet->det0] == gl_oper &&
	          TimeInGT(pointingNum, pDet->det0, pEvents->obTime[irow],
		  gtList, pGtrows))   
	    {
	       if (Timehist(pBin, pDet))
	       {
	          /* Do not count the same event twice */
		  if (!addFlag)
		  {
		     ++(*evCount);
		  }
	       }
	    }
	 } /* end mult = 0 */
	 
	 /* For pure singles 85-103, also attempt to add to 0-18 */
	 else if (pDet->multiplicity == 1)
	 {
	    idx = pDet->det0 - gl_boundaries[0][2] + gl_boundaries[0][0];
	    if (pDet->spiMode[idx] == gl_oper && 
	          !pDet->obSpectraFlag[idx] &&
	          TimeInGT(pointingNum, pDet->det0, pEvents->obTime[irow],
		  gtList, pGtrows))
	    {
	       if (Timehist(pBin, pDet))
	       {
	          ++(*evCount);
		  addFlag = 1;
	       }
	    }
	    pDet->det0 = pEvents->dete[irow];
	    if (pDet->spiMode[pDet->det0] == gl_oper && 
	          !pDet->obSpectraFlag[pDet->det0] &&
	          TimeInGT(pointingNum, pDet->det0, pEvents->obTime[irow],
		  gtList, pGtrows))
	    {
	       if (Timehist(pBin, pDet))
	       {
	          /* Do not count the same event twice */
		  if (!addFlag)
		  {
		     ++(*evCount);
		  }
	       }
	    }	       
	 } /* end mult 1 */
	 
	 else
	 {
	    if ((TimeInGT(pointingNum, pDet->det0, pEvents->obTime[irow], 
			  gtList, pGtrows)) && (pDet->pi0 > 0))
	    {	  
	       if (Timehist(pBin, pDet))
	       {
		  ++(*evCount);
	       }
	    }
	 }
      } /* end if pDet->det0 >= 0 */

      irow++;
   }
   
}






