/* 
-------- Function: ------------------------------------------------------------

     ReadBinFile


-------- Description: ---------------------------------------------------------

     This subroutine reads the input binning file.  


-------- Functions called: -----------------------------------------------------

   SPI Functions:
      Allocate                 -- Allocates specified number of bytes and keeps
                                  track of all allocated memory during a run
      Cleanup                  -- Deallocates all memory allocated during the 
                                  current run in the event of an error
      FreeNode                 -- Frees some of the allocated memory when not 
                                  needed

   DAL Functions:
      DALobjectFindElement     -- Finds element of a DAL object with specified
                                  name and returns a pointer to the element
      DALtableGetNumRows       -- Returns the number of rows in the element
      DALtableGetCol           -- Retrive all data values from the table column


-------- Usage: ---------------------------------------------------------------

   int ReadBinFile(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
                   char *chantype, binningPtr pBin)

   Input Parameters:
       INPUTDS       dal_element  : Pointer to the Observation group DOL
       PBASE         memoryPtr    : Pointer to the list that keeps track of
                                    allocated memory
       PLAST         memoryPtr    : Pointer to the current node in the list
                                    pointed to by the PBASE


   Output Parameters:
       CHANTYPE      char         : PHA: If table contains PHA_MIN/PHA_MAX cols
                                    PI : If table contains E_MIN/E_MAX cols

	      binningPtrInfo pars
	      -------------------
       NENERGY       int          : Number of bins
       ENERGY        float        : Array contining bin boundaries


-------- Include files --------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: ----------------------------------------

   Sandhia Bansal  (1.0.0; Aug 00)


---------------------------------------------------------------------------- */

#include "spihist.h"

int ReadBinFile(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
		char *chantype, binningPtr pBin)
{
   float         *temp=NULL;

   long          numRows=0L;

   int           status=ISDC_OK;

   char          *program="ReadBinFile";
   char          message[DAL_MAX_STRING];
   char          extname[DAL_MAX_STRING];

   dal_element   *tablePtr=NULL;
   dal_dataType  type=DAL_DT_UNKNOWN;



   strcpy(extname, "SPI.-EBDS-SET");
   status = DALobjectFindElement(inputDS, extname, &tablePtr, status); 

   if (status != ISDC_OK)
   {
      strcat(strcat(strcpy(message, "Error while finding "),
		    extname), 
	     " extension");
      Cleanup(status, message, *pBase, program);
   }

   status = DALtableGetNumRows(tablePtr, &numRows, status);
   if (status != ISDC_OK)
   {
      strcat(strcat(strcpy(message, "Error while reading number of rows in "),
		    extname), 
	     " extension");
      Cleanup(status, message, *pBase, program);
   }

   pBin->nEnergy = (int) numRows + 1;

   Allocate(program, "-ENERGY_MIN", 0, pBin->nEnergy*sizeof(float), 
	    (void **)&pBin->energy, pBase, pLast);
   Allocate(program, "-TEMP", 0, numRows*sizeof(float), (void **)&temp, pBase, 
	    pLast);
   status = DALtableGetCol(tablePtr, "E_MIN", 0, &type, &numRows, 
			   (void *)pBin->energy, status);
   status = DALtableGetCol(tablePtr, "E_MAX", 0, &type, &numRows, 
			   (void *)temp, status);
   pBin->energy[numRows] = temp[numRows-1];
   FreeNode(pBase, (void **)&temp, pLast);

   if (status == ISDC_OK)
   {
      strcpy(chantype, "PI");
   }

   else
   {
      status = ISDC_OK;   /* Try reading PHA_MIN */
      status = DALtableGetCol(tablePtr, "PHA_MIN", 0, &type, &numRows, 
			      (void *)pBin->energy, status);

      if (status != ISDC_OK)
      {
	 strcpy(message, "error while reading bin boundaries");
	 Cleanup(status, message, *pBase, program);
      }

      strcpy(chantype, "PHA");
   }

   return(status);
}


/* ------------------------------------------------------------------------- */
