/* 
-------- Function: -------------------------------------------------------------

     ReadDither


-------- Description: ----------------------------------------------------------

     This program opens up the DITHER extension and if successful, reads 
     the time, and x,y,z ra and dec values from the table.


-------- Functions called: -----------------------------------------------------

   SPI Functions:
      Allocate                 -- Allocates specified number of bytes and keeps
                                  track of all allocated memory during a run
      Cleanup                  -- Deallocates all memory allocated during the 
                                  current run in the event of an error
      FreeNode                 -- Frees some of the allocated memory when not 
                                  needed

   DAL Functions:
      DALobjectFindElement     -- Finds element of a DAL object with specified
                                  name and returns a pointer to the element
      DALtableComputeBuffSize  -- Computes size of a data transfer buffer in 
                                  bytes needed to hold the data of table column
				  starting and ending at specified bins
      DALtableGetCol           -- Retrive all data values from the table column


   DAL3 Functions:
      DAL3GENtableGetOBT       -- Reads OBtimes from specified table
      DAL3AUXconvertOBT2IJD    -- Converts times from OBTime to IJD format


--------- Usage: ---------------------------------------------------------------

   void ReadDither(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast,
                   pointingInfoPtr pDither, outputParsPtr pOutput)

   Input Parameters:
       INPUTDS              dal_element  : Pointer to the Observation group DOL
       PBASE                memoryPtr    : Pointer to the list that keeps track
                                           of allocated memory
       PLAST                memoryPtr    : Pointer to the current node in the 
                                           list pointed to by the PBASE


   Output Parameters:
              pointingInfoPtr pars
	      --------------------
       NDT                  long         : Number of pointings
       OBSTART              OBTIME       : Array of pointing start times
       OBSTOP               OBTIME       : Array of pointing stop times
       TIMES                double       : Array containing the pointing start/
                                           stop times
       XRA,XDEC, ZRA,ZDEC   double       : x,z-ra/dec in degrees


-------- Include files ---------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: -----------------------------------------

   V. Beckmann   (3.0.9; Jun 03) OBSERVER, OBJECT, OBS_ID are no valid
                                 keywords anymore and have been removed
		 (2.2.1; Nov 02) OBTstart and OBTstop are now read directly
		 		 from the pointing file.  Previously they
				 were converted from the Tstart and Tstop
				 values with DAL3AUXconvertOBT2IJD.
   Craig Gordon  (2.2.0; Sep 02) if PHA mode, do not try to write to
   			          dsp extension
   Craig Gordon	 (2.1.9: Jul 02) -read DATAMODE from GTI rather than
		                  pointing extension.
   Craig Gordon  (2.1.4; Oct 01)  -read DET_NUM from GTI extension
   Sandhia Bansal  (1.0.0; Aug 00)


----------------------------------------------------------------------------- */

#include "spihist.h"



void ReadDither(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
		pointingInfoPtr pDither, outputParsPtr pOutput)
{
   double            *stopTimes=NULL;

   long               buffSize=0;
   long               temp=0L;

   int                i=0, j=0;
   int                lastBin=0L;
   int                status=ISDC_OK;

   char               *program="ReadDither";
   char               message[DAL_MAX_STRING];
   char               extname[DAL_MAX_ATT_STR_VALUE_SIZE];

   dal_element        *pointPtr=NULL;
   dal_element        *dtiPtr=NULL;
   dal_element	      *gtiPtr=NULL;

   dal_dataType       type=DAL_DT_UNKNOWN;



   dal_element        *dspPtr=NULL;

   
   /* set ISDCLEVL of the observation group to BIN_I */
   status = DALattributePutChar(inputDS, "ISDCLEVL", "BIN_I", NULL, NULL, 
				status);

   status = DALobjectFindElement(inputDS, "SPI.-OBS.-PNT", &pointPtr, status); 
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while finding SPI.-OBS.-PNT extension");
      Cleanup(status, message, *pBase, program);
   }
   
   /* Now open GTI */
   status = DALobjectFindElement(inputDS,"SPI.-OBS.-GTI",&gtiPtr,status);
   if (status != ISDC_OK) 
   {
      strcpy(message, "Error while finding SPI.-OBS.-GTI extension");
      Cleanup(status, message, *pBase, program);
   }

   status = DALobjectFindElement(inputDS, "SPI.-OBS.-DTI", &dtiPtr, status); 
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while finding SPI.-OBS.-DTI extension");
      Cleanup(status, message, *pBase, program);
   }

   if (strcmp(pOutput->format, "ISDC") == 0)
   { 
      /* Copy keywords from the PNT in the output DSP */

      /* Open DSP and PNT first */
      status = DALobjectFindElement(inputDS,"SPI.-OBS.-DSP",&dspPtr,status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while finding SPI.-OBS.-DSP extension");
	 Cleanup(status, message, *pBase, program);
      }

      /* Copy number of pointPtr., number of ISOC windows */
      status = DAL3GENattributeCopy(pointPtr, dspPtr, 
				    "PT_NUM,ISOC_NUM", status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying PT_NUM, ISOC_NUM to DSP");
	 Cleanup(status, message, *pBase, program);
      }

      /* Copy IDs */
      status = DAL3GENattributeCopy(pointPtr, dspPtr,
				    "OGID", 
				    status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying OGID to DSP");
	 Cleanup(status, message, *pBase, program);
      }

      /* Copy sky position and extension of observation */
      status = DAL3GENattributeCopy(pointPtr, dspPtr,
				    "RA_MIN,DEC_MIN,RA_MEAN,DEC_MEAN,RA_MAX,"\
				    "DEC_MAX", status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying RA_MIN,DEC_MIN,RA_MEAN,DEC_MEAN,"\
		"RA_MAX,DEC_MAX to DSP");
	 Cleanup(status, message, *pBase, program);
      }

      status = DAL3GENattributeCopy(pointPtr, dspPtr,
				    "RA_MI_S,DEC_MI_S,RA_ME_S,DEC_ME_S,RA_MA_S,"\
				    "DEC_MA_S", status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying RA_MI_S,DEC_MI_S,RA_ME_S,"\
		"DEC_ME_S,RA_MA_S to DSP");
	 Cleanup(status, message, *pBase, program);
      }

      status = DAL3GENattributeCopy(pointPtr, dspPtr,
				    "B2_MIN,L2_MIN,B2_MEAN,L2_MEAN,B2_MAX,L2_MAX",
				    status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying B2_MIN,L2_MIN,B2_MEAN,"\
		"L2_MEAN,B2_MAX,L2_MAX to DSP");
	 Cleanup(status, message, *pBase, program);
      }

      /* Copy date and duration of observation */
      status = DAL3GENattributeCopy(pointPtr, dspPtr,
				    "DATE-OBS,DATE-END,TSTART,TSTOP,TELAPSE",
				    status);
      /*"DATE-OBS,DATE-END,TSTART,TSTOP,TELAPSE,"\
	"ONTIME",status);*/
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying DATE-OBS,DATE-END,TSTART,"\
		"TSTOP,TELAPSE,ONTIME to DSP");
	 Cleanup(status, message, *pBase, program);
      }
      /* Copy the number of detectors */
      status = DAL3GENattributeCopy(gtiPtr, dspPtr,"DET_NUM,DATAMODE", status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying DET_NUM to DSP");
	 Cleanup(status, message, *pBase, program);
      }
      status = DAL3GENattributeCopy(dtiPtr, dspPtr, "DEADC,ONTIME", status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error while copying DEADC and ONTIME to DSP");
	 Cleanup(status, message, *pBase, program);
      }

   }

   /* Locate the POINTING extension */
   /*strcpy(extname, "SPI.-OBS.-PNT");
   status = DALobjectFindElement(inputDS, extname, &pointPtr, status); 

   if (status != ISDC_OK)
   {
      strcat(strcat(strcpy(message, "Error while finding "),
		    extname), 
	     " extension");
      Cleanup(status, message, *pBase, program);
      }*/

   /* Determine size of OBT_START column */
   status = DALtableComputeBuffSize(pointPtr, "OBT_START", 0, 1, 0, &buffSize, 
				    status);

   if (status != ISDC_OK)
   {
      strcpy(message, "Error while computing buffer size for OBT_START");
      Cleanup(status, message, *pBase, program);
   }

   /* Allocate memory to hold OBT_START and OBT_END values, and read columns */
   Allocate(program, "-obStart", 0, (int)buffSize, (void **) &pDither->obStart, 
	    pBase, pLast);

   pDither->nDT = 0;
   status = DAL3GENtableGetOBT(pointPtr, "OBT_START", 0, &pDither->nDT, 
			       pDither->obStart, status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading OBT_START");
      Cleanup(status, message, *pBase, program);
   }

   Allocate(program, "-obStop", 0, (int)buffSize, (void **) &pDither->obStop,
	    pBase, pLast);

   temp = 0;
   status = DAL3GENtableGetOBT(pointPtr, "OBT_END", 0, &temp, pDither->obStop, 
			       status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading OBT_END");
      Cleanup(status, message, *pBase, program);
   }

   status = DALtableComputeBuffSize(pointPtr, "TSTART", 0, 1, 0, &buffSize, 
				    status);
   if (pDither->nDT != buffSize/sizeof(double))
   {
      strcpy(message, "Buffer size error reading TSTART and TSTOP");
      Cleanup(status, message, *pBase,program);
   }
   Allocate(program, "-times", 0, (int)buffSize*2,  (void **) &pDither->times, 
   		pBase, pLast);
   status = DALtableGetCol(pointPtr, "TSTART", 0, NULL, NULL, (void *)
   		pDither->times, status);
   Allocate(program, "-stopTimes", 0, (int)buffSize, (void **) &stopTimes, 
   		pBase, pLast);
   status = DALtableGetCol(pointPtr, "TSTOP", 0, NULL, NULL, (void *)
   		stopTimes, status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error reading TSTART and TSTOP columns");
      Cleanup(status, message, *pBase, program);
   }   
   j=0;
   lastBin = (int) 2 * pDither->nDT;
   for (i=(int)pDither->nDT; i<lastBin; i++)
      pDither->times[i] = stopTimes[j++];
   FreeNode(pBase, (void **)&stopTimes, pLast);

   /* Allocate memory to hold RA_SPIX values, and read column */
   Allocate(program, "-RA_SPIX", 0, (int)buffSize, (void **) &pDither->xRa, 
	    pBase, pLast);

   temp = 0;
   status = DALtableGetCol(pointPtr, "RA_SPIX", 0, &type, &temp, pDither->xRa, 
			   status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading RA_SPIX");
      Cleanup(status, message, *pBase, program);
   }

   /* Allocate memory to hold DEC_SPIX values, and read column */
   Allocate(program, "-DEC_SPIX", 0, (int)buffSize, (void **) &pDither->xDec,
	    pBase, pLast);

   temp = 0;
   status = DALtableGetCol(pointPtr, "DEC_SPIX", 0, &type, &temp, pDither->xDec,
			   status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading DEC_SPIX");
      Cleanup(status, message, *pBase, program);
   }

   /* Allocate memory to hold RA_SPIZ values, and read column */
   Allocate(program, "-RA_SPIZ", 0, (int)buffSize, (void **) &pDither->zRa,
	    pBase, pLast);

   temp = 0;
   status = DALtableGetCol(pointPtr, "RA_SPIZ", 0, &type, &temp, 
			   pDither->zRa, status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading RA_SPIZ");
      Cleanup(status, message, *pBase, program);
   }

   /* Allocate memory to hold DEC_SPIZ values, and read column */
   Allocate(program, "-DEC_SPIZ", 0, (int)buffSize, (void **) &pDither->zDec,
	    pBase, pLast);

   temp = 0;
   status = DALtableGetCol(pointPtr, "DEC_SPIZ", 0, &type, &temp, pDither->zDec,
			   status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading DEC_SPIZ");
      Cleanup(status, message, *pBase, program);
   }
}
