/* 
-------- Function: -------------------------------------------------------------

     ReadEvents


-------- Description: ----------------------------------------------------------

     This subroutine reads the events data from the input SPI FITS file.


-------- Usage: ----------------------------------------------------------------


   Input Parameters:
       INPUTDS         dal_element   : Pointer to the Observation group DOL
       PBASE           memoryPtr     : Pointer to the list that keeps track of
                                       allocated memory
       PLAST           memoryPtr     : Pointer to the last node in the list
                                       pointed to by the PBASE
       EVENTTYPE       int           : 0:SINGLE, 1:PSD, 2:DOUBLE, 3:TRIPLE, 
                                       4:QUADRUPLE, 5:QUINTUPLE, 6:HI_MULT
       CHANTYP         char          : Type of detector channels (PHA, PI etc)
       PSDFLAG         char          : Raw/Cor
       MULTIPLICITY    short         : Multiplicity associated with EVENTTYPE
       NTIME           int           : Number of elements in OBSTART/OBSTOP 
                                       arrays
       OBSTART/OBSTOP  OBTIME        : Events will be accumulated within
                                       [OBSTART[i],OBSTOP[i]], i=1...NTIME
       SPIMODE	       SPI_mode	     : Enumerated constant for chosing mode
       					of data to select (ie. OPERATIONAL,
					EMERGENCY..)


   Output Parameters:
              eventsInfoPtr pars
	      ------------------
       NEVENT        long            : Number of events in the EVENTS extension
       TIME          double          : Array containing the times read from the
                                       EVENTS extension
       PSDFLAG       DAL3_Byte       : 0: (Det numbers are 1000-1018)
                                       1: (Det numbers are 1100-1118)
                                       3: (Det numbers are 1300-1318)
       DETE          short           : Array containing the detector ids read 
                                       from the EVENTS extension
       PI            float           : Array containing the energies read from
                                       the EVENTS extension


-------- Include files ---------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: -----------------------------------------

                 (3.0.4; Feb 03)   Use modified events selection functions
		                     to increase program speed.
		 (2.2.1; Nov 02)   Removed call to DAL3AUXconvertOBT2IJD,
		 		     events data is no longer converted
				     to IJD format.
		 (2.2.0; Aug 02)   Added additional status error checking
   Craig Gordon  (2.1.8; May 02)     Add capability to select various spiMode
   				     events.   
   Craig Gordon  (2.1.5; Dec 01)     change COR param to ALL_PAR in DAL3SPI
   				     selectEvents call.
   Craig Gordon  (2.1.4; Oct 01)     nEvents and time selected written to 
                                     message log rather than standard output.
   Craig Gordon  (2.1.3; Oct 01)     The call to read the PSDRAW flags is now
                                     made with DAL rather than DAL3 functions.
				     This takes care of the previous 'column not
				     found error'.
   Craig Gordon  (2.1.1; Aug 01)
   Sandhia Bansal  (1.0.1; May 01)   Use TCOR_ANY instead of TCOR_ACCURATE in 
                                     the call to DAL3AUXconvertOBT2IJD.  This
				     function fills the output array with NaN
				     values if TCOR_ACCURATE is passed to it.
				     Eventually, this will probably change?

   Sandhia Bansal  (1.0.0; Aug 00)


----------------------------------------------------------------------------- */

#include "spihist.h"



void ReadEvents(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
		int eventType, char *chantype, char *psdType, 
		short multiplicity, int ntime, OBTime *obStart, OBTime *obStop,
		eventsInfoPtr pEvents, SPI_mode spiMode)
{
   float               *pi=NULL;

   static long         *ph=NULL;

   int                 status=ISDC_OK;
   int                 i=0;
   int                 numGoodEvents=0;
   int                 nbin=multiplicity;

   unsigned short      *pha=NULL;

   char                *program="ReadEvents";
   char                message[DAL_MAX_STRING];

   dal_dataType        type=DAL_DT_UNKNOWN;

   dal_element *tmpdata;
   long numvalues;

   
   int    j=0, k=0, l=0;

   /* Select event data */
   status = SelectEvents(inputDS, spiMode, (SPI_type)eventType, ntime, 
   		obStart, obStop, psdType, chantype, status);
	 
   if (status != ISDC_OK)
   {
      if (status != DAL3SPI_NO_SPI_EVENTS)
      {
         strcpy(message, "Error while selecting events");
         Cleanup(status, message, *pBase, program);
      }  
      pEvents->nEVENT=0;    
      status = ISDC_OK;
      return;
   }

   /* Get number of selected events */
   status=SpihGetNumEvents(&pEvents->nEVENT, status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading number of selected events");
      Cleanup(status, message, *pBase, program);
   }

   /* If event data exists, read OBTime, DETE, and PI/PHA 
      Also read PSD-FLAG if PSD events are to be read */
   if (pEvents->nEVENT > 0)
   {
      /* make calls to SpihGetEvents to read various columns */
      /* OBTime */
      type = DAL_DT_UNKNOWN;
      Allocate(program, "-obTime", 0, pEvents->nEVENT*sizeof(OBTime), 
	       (void **) &pEvents->obTime, pBase, pLast);
      status = SpihGetEvents(OB_TIME, &type, pEvents->obTime, status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error reading events data: OB_TIME column");
	 Cleanup(status, message, *pBase, program);
      }

      if (multiplicity == 0)   /* PSD data */
      {
	 /* PSD Flags */
	 type = DAL_BYTE;

	 Allocate(program, "-psdFlag", 0, pEvents->nEVENT*sizeof(DAL_BYTE), 
		  (void **) &pEvents->psdFlag, pBase, pLast);

	 if (strcmp(psdType, "RAW") == 0)
	 {    
              /* Access the data which contains the PSD_RAWFLAG */
	    status = SpihGetEvents(PSD_RAWFLAG, &type, pEvents->psdFlag, 
				      status); 

       /*     status=DALobjectFindElement(inputDS,"SPI.-OPSD-RAW",&tmpdata,status);
        */    if(status != ISDC_OK) {
	         sprintf(message, "Cannot find SPI.-OPSD-RAW. DALobjectFind returns status: %d", status);
	         RILlogMessage(NULL, Error_0, message);
              }
              /* Read the column which contains the PSD_RAWFLAG */

  /*           status = DALtableGetColBins(tmpdata, "PSD_RAWFLAG", 0, &type, 1,
	              pEvents->nEVENT, &numvalues, pEvents->psdFlag, status); 
*/
	 }
	 else   /* corrected flags */
	    status = SpihGetEvents(PSD_CORFLAG, &type, pEvents->psdFlag, 
				      status);
	 nbin = 1;
	 
      }

      /* DETE */
      type = DAL_BYTE;
      Allocate(program, "-dete", 0, pEvents->nEVENT*nbin*sizeof(DAL_BYTE), 
	       (void **) &pEvents->dete, pBase, pLast);
      status = SpihGetEvents(DETE, &type, pEvents->dete, status);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Error reading events data: DETE column");
	 Cleanup(status, message, *pBase, program);
      }

      /* PHA/ENERGY */
      type = DAL_FLOAT;
      Allocate(program, "-pi", 0, pEvents->nEVENT*nbin*sizeof(DAL_FLOAT), 
	       (void **) &pEvents->pi, pBase, pLast);

      if (strcmp(chantype, "PHA") == 0)
      {
	 type = DAL_USHORT;
	 Allocate(program, "-pha", 0, pEvents->nEVENT*nbin*sizeof(DAL_USHORT), 
		  (void **) &pha, pBase, pLast);
	 status = SpihGetEvents(PHA, &type, pha, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error reading events data: PHA column");
	    Cleanup(status, message, *pBase, program);
	 }

	 for (i=0; i<pEvents->nEVENT; i++)
	    pEvents->pi[i] = pha[i];

	 FreeNode(pBase, (void **) &pha, pLast);
      }

      else
      {
	 status = SpihGetEvents(ENERGY, &type, pEvents->pi, status);
	 if (status != ISDC_OK)
	 {
	    strcpy(message, "Error reading events data: ENERGY column");
	    Cleanup(status, message, *pBase, program);
	 }
      }
	 
     sprintf(message,"nEvents selected: %d \n", pEvents->nEVENT);
     RILlogMessage(NULL, Log_1, message);
   }

   /* Closing the selected events table */
   status = SpihCloseEvents(ISDC_OK);
}
      


