/* 
-------- Function: -------------------------------------------------------------

     ReadGT


-------- Description: ----------------------------------------------------------

     This subroutine reads the GOOD TIMES extension from the input SPI FITS 
     file.
   
     ASSUMPTION:
     ----------
     The number of pointings and pointing ids in the POINTING extension of the 
     input file match with those in the GTI table.


-------- Functions called: -----------------------------------------------------

   Spihist Functions:
      GetValidGT               -- Creates list of valid good times

   SPI Functions:
      Allocate                 -- Allocates specified number of bytes and keeps
                                  track of all allocated memory during a run
      Cleanup                  -- Deallocates all memory allocated during the 
                                  current run in the event of an error
      FreeNode                 -- Frees some of the allocated memory when not 
                                  needed

   DAL Functions:
      DALobjectFindElement     -- Finds element of a DAL object with specified
                                  name and returns a pointer to the element
      DALtableComputeBuffSize  -- Computes size of a data transfer buffer in 
                                  bytes needed to hold the data of table column
				  starting and ending at specified bins
      DALtableGetCol           -- Retrive all data values from the table column
      DALtableGetColStrings    -- Retriev string data values from table column


   DAL3 Functions:
      DAL3GENtableGetOBT       -- Reads OBtimes from specified table


-------- Usage: ----------------------------------------------------------------

   Input Parameters:
       INPUTDS       dal_element  : Pointer to the Observation group DOL
       PBASE         memoryPtr    : Pointer to the list that keeps track of
                                    allocated memory
       PLAST         memoryPtr    : Pointer to the current node in the list
                                    pointed to by the PBASE
       NDT           long         : Number of rows in the POINTING extension
       MAXDET        int          : Total number of detectors for the instrument


   Output Parameters:
       MAXNGTI       long*	  :number of gti's as read from MAXGTINM keyword
       PGTROWS	     gtRowsPtr    :Livetime and nInts info for each row in GTI table
       STATUS        int          : 0=OK

	      goodTimesPtr pars
	      -----------------
       NGT           long         : Number of rows in the GOOD TIME extension
       DETID         long         : Array containing detector ids read from each
                                    row of the GOOD TIME extension


-------- Include files ---------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: -----------------------------------------
                
		(3.0.4; Feb 03) Using new gtint structs more efficient memory allocation
		(3.0.2; Feb 03) explicitly initialize gtList livetimes (to be safe)
                (2.1.9; Jul 02) 
   Craig Gordon (2.1.6; Mar 02)
   Craig Gordon (2.1.5; Feb 02)
   Sandhia Bansal  (1.0.0; Aug 00)


----------------------------------------------------------------------------- */

#include "spihist.h"


gtIntPtr **ReadGT(dal_element *inputDS, memoryPtr *pBase, memoryPtr *pLast, 
	       long nDT, int maxdet, goodTimesPtr pGtimes, long *maxnGTI, 
	       gtRowsPtr pGtrows, int *status)
{
   long          buffSize=0L;
   long          ndet=0L;
   long          nOBT=0L;
   long          temp=0L;
   long          *detid=NULL;
   long          nGT=0;

   int           idet=0;
   int           i=0, j=0, k=0, igt=0, sz=0;

   char          *program="ReadGT";
   char          message[DAL_MAX_STRING];
   char          extname[DAL_MAX_ATT_STR_VALUE_SIZE];
   char          name[DAL_MED_STRING];
   char          qFound=0;
   char          **pointingId=NULL;

   gtIntPtr         **gtList=NULL;

   dal_element   *tablePtr=NULL;

   dal_dataType  type=DAL_DT_UNKNOWN;

   OBTime        *obStart=NULL, *obStop=NULL;


   /* Locate the GOOD TIME extension */
   strcpy(extname, "SPI.-OBS.-GTI");
   *status = DALobjectFindElement(inputDS, extname, &tablePtr, *status); 

   if (*status != ISDC_OK)
   {
      strcat(strcat(strcpy(message, "Error while finding "),
		    extname), 
	     " extension");
      Cleanup(*status, message, *pBase, program);
   }
   
   /* Get the number of GTIs keyword */
   *status = DALattributeGetInt(tablePtr, "MAXGTINM", maxnGTI, NULL, NULL,
   		*status);
   if (*status != ISDC_OK)
   {
      strcpy(message, "Error while reading MAXGTINM keyword");
      Cleanup(*status, message, *pBase, program);
   }

   /* Read DETID column */
   *status = DALtableComputeBuffSize(tablePtr, "DET_ID", 0, 1, 0, &buffSize, 
				     *status);

   if (*status != ISDC_OK)
   {
      strcpy(message, "Error while computing buffer size for DET_ID");
      Cleanup(*status, message, *pBase, program);
   }

   Allocate(program, "-DETID", 0, (int)buffSize, (void **) &detid, pBase, pLast);

   *status = DALtableGetCol(tablePtr, "DET_ID", 0, &type, &nGT, detid, *status);

   if (*status != ISDC_OK)
   {
      strcpy(message, "Error while reading detid");
      Cleanup(*status, message, *pBase, program);
   }
   pGtrows->nRows = nGT;
   /* Memory for pGtrows->livetime is allocated inside Livetime function*/
   /* Read POINTING column */
   Allocate(program, "-PTID_SPI", (int)nGT, PTIDSZ, (void **) &pointingId,
   		 pBase, pLast);

   *status = DALtableGetColStrings(tablePtr, "PTID_SPI", 0, 1L, 0L, &temp, 
		   pointingId, *status);

   if (*status != ISDC_OK)
   {
      strcpy(message, "Error while reading pointing Id");
      Cleanup(*status, message, *pBase, program);
   }

   /* Read OBT_START column */
   *status = DALtableComputeBuffSize(tablePtr, "OBT_START", 0, 1, 0, &buffSize, 
				     *status);

   if (*status != ISDC_OK)
   {
      strcpy(message, "Error while computing buffer size for OBT_START");
      Cleanup(*status, message, *pBase, program);
   }

   obStart = NULL;
   Allocate(program, "-obStart", 0, (int)buffSize, (void **) &obStart, pBase, 
	    pLast);

   nOBT = 0;
   *status = DAL3GENtableGetOBT(tablePtr, "OBT_START", 0, &nOBT, obStart, 
				*status);
   if (*status != ISDC_OK)
   {
      strcpy(message, "Error while reading OBT_START");
      Cleanup(*status, message, *pBase, program);
   }

   /* Read OBT_STOP column */
   obStop = NULL;
   Allocate(program, "-obStop", 0, (int)buffSize, (void **) &obStop, pBase, 
	    pLast);

   *status = DAL3GENtableGetOBT(tablePtr, "OBT_END", 0, &nOBT, obStop, *status);
   if (*status != ISDC_OK)
   {
      strcpy(message, "Error while reading OBT_END");
      Cleanup(*status, message, *pBase, program);
   }

   if (nGT*(*maxnGTI) != nOBT)
   {
      strcpy(message, "READGT:: Error in number of GTs");
      Cleanup(*status, message, *pBase, program);
   }

   
   Allocate(program, "-gtList", 0, (int)nDT*sizeof(GTINT **), (void **) &gtList,
	    pBase, pLast);
   Allocate(program, "-ngtInts", 0, (int)nDT*sizeof(int *), 
   		(void **)&pGtrows->nInts, pBase, pLast);
   Allocate(program, "-gtrLvt", 0, (int)nDT*sizeof(double *),
   		(void **)&pGtrows->livetime, pBase, pLast);
   Allocate(program, "-gtrPtid", (int)nDT, PTIDSZ, (void **)&pGtrows->pointingId,
   		pBase, pLast);
   for (i=0; i<nDT; i++)
   {
      gtList[i] = NULL;
      pGtrows->nInts[i] = NULL;
      pGtrows->livetime[i] = NULL;
      Allocate(program, "-gtList[i]", 0, maxdet*sizeof(GTINT *), 
	       (void **) &gtList[i], pBase, pLast);
      Allocate(program, "-ngtInts[i]", 0, maxdet*sizeof(int), 
   		(void **)&pGtrows->nInts[i], pBase, pLast);
      Allocate(program, "-gtrLvt[i]", 0, maxdet*sizeof(double),
      		(void **)&pGtrows->livetime[i],pBase, pLast);
      for (j=0; j<maxdet; j++)
      {
	 gtList[i][j] = NULL;
	 pGtrows->nInts[i][j] = 0;
	 pGtrows->livetime[i][j] = .0;
      }

   }

   if (pGtimes)
   {
      pGtimes->nGT = nGT;

      /* Allocate space for detector ids (per pointing) to be returned to the
	 calling program */
      Allocate(program, "-detid", 0, maxdet*sizeof(long), 
	       (void **) &pGtimes->detid, pBase, pLast);
      for (i=0; i<maxdet; i++)
	 pGtimes->detid[i] = -1;
   }

   /* Determine number of valid Good Time intervals for requested detectors */
   GetValidGT(inputDS, pBase, pLast, obStart, obStop, detid, pGtimes, 
   		pointingId, *maxnGTI, gtList, pGtrows);

   FreeNode(pBase, (void **) &obStart, pLast);
   FreeNode(pBase, (void **) &obStop, pLast);
   FreeNode(pBase, (void **) &detid, pLast);
   FreeNode(pBase, (void **) &pointingId, pLast);   

   return(gtList);
}


/* ------------------------------------------------------------------------- */
