/* 
-------- Function: -------------------------------------------------------------

     ReadPseudoDetTable


-------- Description: ----------------------------------------------------------

     This program reads the pseudo detector look up information.


-------- Functions called: -----------------------------------------------------

   SPIHIST Functions:
      Multiplicity   :  Returns the multiplicity of a given detector

   SPI Functions:
      Allocate       :  Allocates specified number of bytes and keeps track of 
                        all allocated memory during a run
      Cleanup        :  Deallocates all memory allocated during the current run
                        in the event of an error
      DetInsert      :  Inserts a new node(PSD flag, detector ids and associated
                        pseudo detector number) in the list


--------- Usage: ---------------------------------------------------------------

   void ReadPseudoDetTable(memoryPtr *pBase, memoryPtr *pLast, detinfoPtr pDet)

   Input Parameters:
       PBASE         memoryPtr       : Pointer to the list that keeps track of
                                       allocated memory
       PLAST         memoryPtr       : Pointer to the current node in the list
                                       pointed to by the PBASE

		 detinfoPtr pars
		 ---------------
       MULTIDETFILE  char            : Name of the multi-detector file if 
       				       external file is used


   Output Parameters:

		 detinfoPtr pars
		 ---------------
       NDETROWS      int             : Number of rows in the DITHER extension
       DETCODE       string          : String consisting of real detector ids
       DETLIST       pseudoDetPtr    : Pointer to the first node in the list
                                       containing PSD flag, detector ids and 
				       associated pseudo detector number


-------- Include files ---------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: -----------------------------------------

		 (3.0.0; Dec 02) ;Moved standardTable definition to global
		 		  variable in spihist.C
   Craig Gordon  (2.1.2; Sept 01)
   Sandhia Bansal  (1.0.0; Aug 00)

----------------------------------------------------------------------------- */

#include "spihist.h"

extern const struct multiDetEntry 
{
   int id;
   char *detcode;
} gl_standardTable[SPIHIST_NSTND];

void ReadPseudoDetTable(memoryPtr *pBase, memoryPtr *pLast, detinfoPtr pDet)
{
   int     status=ISDC_OK;
   int     i=0;
   int     len=0;
   int     mult=0;

   char    *program="ReadPseudoDetTable";
   char    str[DAL_MAX_STRING];
   char    message[DAL_MAX_STRING];
   char    tmp1;
	   

/*  Use standard multidet table */
   pDet->nDetRows = SPIHIST_NSTND;
   
   Allocate(program, "-detcode", pDet->nDetRows, DAL_MAX_STRING, 
	    (void **) &pDet->detcode, pBase, pLast);
   for (i=0; i<pDet->nDetRows; i++)
      strcpy(pDet->detcode[i], gl_standardTable[i].detcode);

   Allocate(program, "-pseudoDetList", 0, NEVENTTYPE*sizeof(pseudoDetPtr), 
	    (void **) &pDet->pseudoDetList, pBase, pLast);
   for (i=0; i<NEVENTTYPE; i++)
      pDet->pseudoDetList[i] = NULL;
   for (i=0; i<pDet->nDetRows; i++)
   {
      mult = Multiplicity(pDet->detcode[i]);

      if (mult >= 0)
      {
	 DetInsert(pBase, pLast, mult, i, pDet->detcode[i], 
		   &pDet->pseudoDetList[mult]);
      }

      else
      {
	 sprintf(message, "Invalid detector code: %s", pDet->detcode[i]);
	 Cleanup(SPI_INVALID_PSEUDODET, message, *pBase, program);
      }   

   }

}
