/*----------- Function -------------------------------------------------

   Rebin
   
------------- Description ----------------------------------------------

   Function to take a spectrum originally binned by single pha channel
   and convert to binning as specified by pha values of a converted
   ebounds array.
   
------------------------------------------------------------------------
   
   Input Parameters:
	COUNTSIN	short *		:Original counts spectrum
	ERANGE		unsigned char   : 0 if spectrum is for low e range
					  1 for high, in which case channels
					  should be shifted by 16384.
	BOUNDS		unsigned short* :Specifies bins of output spectrum.
	NCHANS		int		:Size of output spectrum, should
					  correspond to number of channels
					  in spihist output, and is 1 less
					  than the size of the bounds array.
					     
   Output Parameters:
   	COUNTSOUT	long *		:Re-binned output spectrum
   Return Value:
      ISDC_OK if no error
      -1 if there are counts outside of bounds.
      1 if spectrum contains no counts
      
------------- Authors/Modification History: ----------------------------

   Craig Gordon   (3.0.0;  Dec 02)

------------------------------------------------------------------------
*/   
#include "spihist.h"

int Rebin(short *countsIn, unsigned char eRange, unsigned short *bounds, 
		int nChans, long *countsOut)
{
   int j, status = ISDC_OK;
   int iBounds= 0;
   int firstCounts = -1;
   int anyCounts = 0;
   unsigned short i;
   unsigned short pha;
   unsigned short low, high;
   const unsigned short iMax = (unsigned short)SPIHIST_NCHANS;
   
   /* Initialize the countsOut array */
   for (j=0; j<nChans; ++j)
   {
      countsOut[j] = 0;
   }
      
   low = bounds[0];
   high = bounds[1];
   i=0;   
   /* Initialize - find first firstCounts */
   while (i < iMax)
   {
      pha = eRange ? i+SPIHIST_NCHANS : i;
      if (countsIn[i] > 0)
      {
         anyCounts = 1;
         if (pha < low)
	 {
	    /* These counts will not be binned anywhere.  
	       If all counts turn out to be below first ebin, firstCounts
	       will never be properly set and function will return
	       further below. */
	    status = -1;
	    ++i;
	 }
	 else if (pha >= high)
	 { /* Get the next ebounds bin */
	    if (iBounds < nChans-1)
	    {
	       ++iBounds;
	       low = bounds[iBounds];
	       high = bounds[iBounds+1];	       
	    }
	    else
	    {
	     /* We've reached the end - all ebins are below the first counts */
	        status = -1;
		break;
	    }
	 }
	 else
	 {
	    /* The starting bin has been found */
	    firstCounts = i;
	    break;
	 }
      }
      else
      {
         ++i;
      }
   }
   
   if (!anyCounts)
   {
      return 1;
   }   
   if (firstCounts < 0)
   {
      return status;
   }
   
   /* We have a first case where bounds[iBounds] <= pha < bounds[iBounds+1],
      and countsIn[i] > 0, now begin actual rebinning */
   while (i < iMax)
   {
      if (pha < high)
      {
         countsOut[iBounds] += countsIn[i];
	 ++i;
	 ++pha;
      }
      else
      {
         /* Proceed to next bin and try again.*/
         ++iBounds;
	 if (iBounds >= nChans)
	 {
	    break;
	 }
	 high = bounds[iBounds+1];
      }
   }
   
   /* Test for any remaining counts that may still exist beyond highest ebin */
   while (i < iMax)
   {
      if (countsIn[i])
      {
         status = -1;
	 break;
      }
      ++i;
   }
      
   return status;
}
