/*
------------- Description ----------------------------------------------

This code is taken from and a slightly modified version of:
       DAL3SPIselectEvents   v 3.7.1  - Stephane Paltani
and the routines found in:
       dal3gen_events.c  v 4.6.5 - Stephane Paltani
Modifications for spihist v3.0.4 - Craig Gordon Feb 03
   
   This is a specialized version for use only in Spihist.  To avoid confusion, 
not to mention possible linkage errors, the DAL3SPI and DAL3GEN function
prefixes are replaced by "Spih". 
   The primary specialization has to do with limiting both the allowed
event types (currently triples are the highest allowed multiplicity) and
the number of events data columns that are read into memory.  In the
functions SpihIsSelectColumn and SpihCreateSelTable, only the columns that
are actually used in Spihist are selected.  This can significantly increase
program speed.  
   Also in SpihSelectEvents, row selection is turned off by hardcoding 
the noSelectFlag to 1.  This is necessary to prevent the program from 
slowing down when there are large numbers (>10^2) of GTIs on which to
perform the selection.

------------------------------------------------------------------------
   
   
*/
#include "spihist.h"

static int SpihSelectEvents(dal_element **ListDS,int NumChild,
	OBTime *StartSWG,OBTime *EndSWG,int NumSWG,Event_type EvType,
	int NumIntervals,OBTime *gtiStart,OBTime *gtiStop, 
	dal_element *InputDS, char *psdType, char *chanType, int status);


int SelectEvents(dal_element *InputDS, SPI_mode SpiMode, SPI_type EvType,
		int NumIntervals, OBTime *gtiStart, OBTime *gtiStop, 
		char *psdType, char *chanType, int status)
/*  Wrapper function, modified from DAL3SPIselectEvents */
{
  int numChild;
  const int IERR = -1;
  char nameExt[DAL_MED_STRING];
  dal_element **listDS=NULL;
  Event_type locType = SPI__SINGLE; 
  int i,numSWG;
  OBTime *obtStart=NULL,*obtEnd=NULL;
  
  if (status != ISDC_OK)  return status;
  
  do {
    /* Select the mode of the events                                         */    
    switch (SpiMode) {
    case OPERATIONAL : strcpy(nameExt,"SPI.-O");        break;
    case CALIBRATION : strcpy(nameExt,"SPI.-C");        break;
    case EMERGENCY   : strcpy(nameExt,"SPI.-E");        break;
    case DIAGNOSTIC  : strcpy(nameExt,"SPI.-D");        break;
    default          : status=IERR;     break;
    }
    if (status!=ISDC_OK) continue;
    /* Select the type of events                                             */    
    switch (EvType) {
    case SINGLE        : strcat(nameExt+6,"SGL-"); locType=SPI__SINGLE      ; break;
    case PSD           : strcat(nameExt+6,"PSD-"); locType=SPI__PSD         ; break;
    case DOUBLE        : strcat(nameExt+6,"ME2-"); locType=SPI__DOUBLE      ; break;
    case TRIPLE        : strcat(nameExt+6,"ME3-"); locType=SPI__TRIPLE      ; break;
    default            : status=IERR;                    break;
    }
    if (status!=ISDC_OK) continue;
    strcpy(nameExt+10,"*");
    status=DAL3GENbuildList(InputDS,nameExt,&listDS,&numChild,status);
    if (status!=ISDC_OK) continue;
    if (numChild==0) 
    {
      status=DAL3SPI_NO_SPI_EVENTS;
      continue;
    }
    /* Find the OBT boundaries of the Science Windows                        */
    numSWG=0;
    if (NumIntervals>0) 
    {
      status=DAL3GENgetNumSWG(InputDS,&numSWG,status);
      if (status!=ISDC_OK && status!=DAL3GEN_NOT_AN_ISDC_GROUP) continue;
      if (status==DAL3GEN_NOT_AN_ISDC_GROUP) 
      {
	status=ISDC_OK;
      } 
      else 
      {
	obtStart=(long long *)malloc(numSWG*sizeof(OBTime));
	obtEnd=(long long *)malloc(numSWG*sizeof(OBTime));
	status=DAL3GENgetSWGbounds(InputDS,1,0,&numSWG,obtStart,obtEnd,status);
	for (i=0;i<numSWG;i++) 
	{
	  obtStart[i]-=DAL3GEN_SWG_BOUNDARY;
	  obtEnd  [i]+=DAL3GEN_SWG_BOUNDARY;
	}
      }
    }  
    status=SpihSelectEvents(listDS,numChild,obtStart,obtEnd,numSWG,locType,
			       NumIntervals,gtiStart,gtiStop, InputDS,
			       psdType, chanType, status);

  } while (0);
  /* Delete the list                                                         */ 
  free(listDS);
  /* Delete the arrays                                                       */ 
  free(obtStart);
  free(obtEnd);
    
  return status;
}

/*****************************************************************************/
/*                                                                           */
/*                       INTEGRAL SCIENCE DATA CENTRE                        */
/*                                                                           */
/*                          DAL3  General  LIBRARY                           */
/*                           EVENT MANIPULATIONS                             */
/*                                                                           */
/*                                 C CODE                                    */
/*                                                                           */
/*  Authors: Stphane Paltani       Modified by Craig Gordon                 */
/*  Date:    21 December 2001                   Feb 2003                     */
/*  Version: 3.5.15                             spihist 3.0.4                */
/*                                                                           */
/*****************************************************************************/


/*****************************************************************************/
/*                                                                           */
/*  Static functions and variables                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/*  Array of strings: SpihColName    [column number]                      */
/*                                                                           */
/*  Contains the name of the different columns in the event data             */
/*                                                                           */
/*****************************************************************************/
static const  char        *SpihColName[DAL3GEN_COLUMN_NUMBER]={
   "OB_TIME"        ,      /*   OB_TIME        */
   "DELTA_TIME"     ,      /*   DELTA_TIME     */
   "RISE_TIME"      ,      /*   RISE_TIME      */
   "ISGRI_PHA"      ,      /*   ISGRI_PHA      */
   "PICSIT_PHA"     ,      /*   PICSIT_PHA     */
   "ISGRI_Y"        ,      /*   ISGRI_Y        */
   "PICSIT_Y"       ,      /*   PICSIT_Y       */
   "ISGRI_Z"        ,      /*   ISGRI_Z        */
   "PICSIT_Z"       ,      /*   PICSIT_Z       */
   "SELECT_FLAG"    ,      /*   SELECT_FLAG    */
   "ISGRI_PI"       ,      /*   ISGRI_PI       */
   "ISGRI_ENERGY"   ,      /*   ISGRI_ENERGY   */
   "PICSIT_ENERGY"  ,      /*   PICSIT_ENERGY  */
   "DUMMY_COUNTER"  ,      /*   DUMMY_COUNTER  */
   "TIME_TAG"       ,      /*   TIME_TAG       */
   "DETE"           ,      /*   DETE           */
   "PHA"            ,      /*   PHA            */
   "ENERGY"         ,      /*   ENERGY         */
   "RAWX"           ,      /*   RAWX           */
   "RAWY"           ,      /*   RAWY           */
   "DELTA_POS"      ,      /*   DELTA_POS      */
   "REST_PHA"       ,      /*   REST_PHA       */
   "DETX"           ,      /*   DETX           */
   "DETY"           ,      /*   DETY           */
   "PI"             ,      /*   PI             */
   "REST_PI"        ,      /*   REST_PI        */
   "SPECTRUM"       ,      /*   SPECTRUM       */
   "GREY_FILTER"    ,      /*   GREY_FILTER    */
   "SPEC_CORR"      ,      /*   SPEC_CORR      */
   "DELTA_COUNT"    ,      /*   DELTA_COUNT    */
   "COUNT_RATE"     ,      /*   COUNT_RATE     */
   "STATUS"         ,      /*   STATUS         */
   "RATE_CORR"      ,      /*   RATE_CORR      */
   "PSD_RAWFLAG"    ,      /*   PSD_RAWFLAG    */
   "PSD_DATA"       ,      /*   PSD_DATA       */
   "PSD_ERR"        ,      /*   PSD_ERR        */
   "PSD_TTP1"       ,      /*   PSD_TTP1       */
   "PSD_TTP2"       ,      /*   PSD_TTP2       */
   "PSD_AMP"        ,      /*   PSD_AMP        */
   "PSD_CORFLAG"    ,      /*   PSD_CORFLAG    */
   "MUL_FLAG"       ,      /*   MUL_FLAG       */
   "CAL_FLAG"       ,      /*   CAL_FLAG       */
   "PSD_ADC"        ,      /*   PSD_ADC        */
   "PSD_CURVE"      ,      /*   PSD_CURVE      */
   "PSD_LABEL"      ,      /*   PSD_LABEL      */
   "MULT"                  /*   MULT           */
};

/*****************************************************************************/
/*                                                                           */
/*  2-D Array of int: SpihColMult   [column number] x [mode]              */
/*                                                                           */
/*  Multipicity for a given event property                                   */
/*   0 = property inexistent for given mode                                  */
/*  -i = multiplicity =i, but type is DAL_SHORT (CHEATING!!!)                */
/*                                                                           */
/*****************************************************************************/
static const  int        SpihColMult[DAL3GEN_COLUMN_NUMBER][DAL3GEN_COLUMN_MODES]={
  /*  ISGRI_EVTS          SINGLE                                  FULL
          PICSIT_SGLE         PSD                                     REST
              PICSIT_MULE         DOUBLE                                  SPTI
                  COMPTON_SGLE        TRIPLE                                  TIME
                      COMPTON_MULE        QUADRUPLE                               SPEC
                                              QUINTUPLE                               RATE
                                                  PURE_PSD
					              SEXTUPLE_PSD
                                                          HI_MULT                        
                                                              PSD_CURVES     */

   {  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 },  /*   OB_TIME        */
   {  1,  1,  1,  0,  0,  0,  0,  2,  3,  4,  5,  0,  6, 31,  0, -1,  0, -1, -1,  0,  0 },  /*   DELTA_TIME     */
   {  1,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   RISE_TIME      */
   { -1,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   ISGRI_PHA      */
   {  0, -1, -1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   PICSIT_PHA     */
   {  1,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   ISGRI_Y        */
   {  0,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   PICSIT_Y       */
   {  1,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   ISGRI_Z        */
   {  0,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   PICSIT_Z       */
   {  1,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   SELECT_FLAG    */
   {  1,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   ISGRI_PI       */
   {  1,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   ISGRI_ENERGY   */
   {  0,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   PICSIT_ENERGY  */
   {  0,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   DUMMY_COUNTER  */
   {  0,  0,  0,  1,  1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,  0,  0,  0,  0,  0,  0 },  /*   TIME_TAG       */
   {  0,  0,  0,  0,  0,  1,  1,  2,  3,  4,  5,  1,  6, 31,  1,  0,  0,  0,  0,  0,  0 },  /*   DETE           */
   {  0,  0,  0,  0,  0, -1, -1, -2, -3, -4, -5,  0, -6,-31, -1,  1,  0,  1,  0,  0,  0 },  /*   PHA            */
   {  0,  0,  0,  0,  0,  1,  1,  2,  3,  4,  5,  0,  6, 31,  1,  0,  0,  0,  0,  0,  0 },  /*   ENERGY         */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0 },  /*   RAWX           */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0 },  /*   RAWY           */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0 },  /*   DELTA_POS      */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0 },  /*   REST_PHA       */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0 },  /*   DETX           */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  0,  0,  0 },  /*   DETY           */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  1,  0,  0,  0 },  /*   PI             */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0 },  /*   REST_PI        */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 64,  0 },  /*   SPECTRUM       */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1 },  /*   GREY_FILTER    */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 64,  0 },  /*   SPEC_CORR      */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1 },  /*   DELTA_COUNT    */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1 },  /*   COUNT_RATE     */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  1,  1 },  /*   STATUS         */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1 },  /*   RATE_CORR      */
   {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_RAWFLAG    */
   {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_DATA       */
   {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_ERR        */
   {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_TTP1       */
   {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_TTP2       */
   {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_AMP        */
   {  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_CORFLAG    */
   {  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0 },  /*   MUL_FLAG       */
   {  0,  0,  0,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   CAL_FLAG       */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0 },  /*   PSD_ADC        */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 79,  0,  0,  0,  0,  0,  0 },  /*   PSD_CURVE      */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0 },  /*   PSD_LABEL      */
   {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0 }   /*   MULT           */
};

/*****************************************************************************/
/*                                                                           */
/*  Array of int: SpihColLevl   [column number]                           */
/*                                                                           */
/*  Level at which a property appears                                        */
/*                                                                           */
/*****************************************************************************/
static const int           SpihColLevl[DAL3GEN_COLUMN_NUMBER]={
  2,                       /*   OB_TIME        */
  1,                       /*   DELTA_TIME     */
  1,                       /*   RISE_TIME      */
  1,                       /*   ISGRI_PHA      */
  1,                       /*   PICSIT_PHA     */
  1,                       /*   ISGRI_Y        */
  1,                       /*   PICSIT_Y       */
  1,                       /*   ISGRI_Z        */
  1,                       /*   PICSIT_Z       */
  3,                       /*   SELECT_FLAG    */
  3,                       /*   ISGRI_PI       */
  3,                       /*   ISGRI_ENERGY   */
  3,                       /*   PICSIT_ENERGY  */
  1,                       /*   DUMMY_COUNTER  */
  1,                       /*   TIME_TAG       */
  1,                       /*   DETE           */
  1,                       /*   PHA            */
  3,			   /*   ENERGY         */
  1,			   /*   RAWX           */
  1,			   /*   RAWY           */
  1,			   /*   DELTA_POS      */
  1,			   /*   REST_PHA       */
  3,			   /*   DETX           */
  3,			   /*   DETY           */
  3,			   /*   PI             */
  3,			   /*   REST_PI        */
  1,			   /*   SPECTRUM       */
  1,			   /*   GREY_FILTER    */
  3,			   /*   SPEC_CORR      */
  1,			   /*   DELTA_COUNT    */
  1,			   /*   COUNT_RATE     */
  3,			   /*   STATUS         */
  3,                       /*   RATE_CORR      */
  1,                       /*   PSD_RAWFLAG    */
  1,                       /*   PSD_DATA       */
  2,                       /*   PSD_ERR        */
  2,                       /*   PSD_TTP1       */
  2,                       /*   PSD_TTP2       */
  2,                       /*   PSD_AMP        */
  3,                       /*   PSD_CORFLAG    */
  2,                       /*   MUL_FLAG       */
  1,                       /*   CAL_FLAG       */
  1,                       /*   PSD_ADC        */
  1,                       /*   PSD_CURVE      */
  1,                       /*   PSD_LABEL      */
  1                        /*   MULT           */
};


/*****************************************************************************/
/*                                                                           */
/*  Array of int: SpihColLevm   [column number]                           */
/*                                                                           */
/*  Level at which a property disappears                                     */
/*                                                                           */
/*****************************************************************************/
static const int           SpihColLevm[DAL3GEN_COLUMN_NUMBER]={
  3,                       /*   OB_TIME        */
  1,                       /*   DELTA_TIME     */
  2,                       /*   RISE_TIME      */
  2,                       /*   ISGRI_PHA      */
  2,                       /*   PICSIT_PHA     */
  3,                       /*   ISGRI_Y        */
  3,                       /*   PICSIT_Y       */
  3,                       /*   ISGRI_Z        */
  3,                       /*   PICSIT_Z       */
  3,                       /*   SELECT_FLAG    */
  3,                       /*   ISGRI_PI       */
  3,                       /*   ISGRI_ENERGY   */
  3,                       /*   PICSIT_ENERGY  */
  1,                       /*   DUMMY_COUNTER  */
  1,                       /*   TIME_TAG       */
  3,                       /*   DETE           */
  2,                       /*   PHA            */
  3,			   /*   ENERGY         */
  2,			   /*   RAWX           */
  2,			   /*   RAWY           */
  1,			   /*   DELTA_POS      */
  2,			   /*   REST_PHA       */
  3,			   /*   DETX           */
  3,			   /*   DETY           */
  3,			   /*   PI             */
  3,			   /*   REST_PI        */
  2,			   /*   SPECTRUM       */
  3,			   /*   GREY_FILTER    */
  3,			   /*   SPEC_CORR      */
  2,			   /*   DELTA_COUNT    */
  2,			   /*   COUNT_RATE     */
  3,			   /*   STATUS         */
  3,                       /*   RATE_CORR      */
  2,                       /*   PSD_RAWFLAG    */
  1,                       /*   PSD_DATA       */
  3,                       /*   PSD_ERR        */
  3,                       /*   PSD_TTP1       */
  3,                       /*   PSD_TTP2       */
  3,                       /*   PSD_AMP        */
  3,                       /*   PSD_CORFLAG    */
  3,                       /*   MUL_FLAG       */
  3,                       /*   CAL_FLAG       */
  3,                       /*   PSD_ADC        */
  3,                       /*   PSD_CURVE      */
  3,                       /*   PSD_LABEL      */
  3                        /*   MULT           */
};

/*****************************************************************************/
/*                                                                           */
/*  Array of DAL types: SpihColForm   [column number]                     */
/*                                                                           */
/*  Type of a given event property (unless mult<0 --> DAL_SHORT)             */
/*                                                                           */
/*****************************************************************************/
static const dal_dataType  SpihColForm[DAL3GEN_COLUMN_NUMBER]={
  DAL3_OBT   ,             /*   OB_TIME        */
  DAL_BYTE   ,             /*   DELTA_TIME     */
  DAL_BYTE   ,             /*   RISE_TIME      */
  DAL_BYTE   ,             /*   ISGRI_PHA      */
  DAL_BYTE   ,             /*   PICSIT_PHA     */
  DAL_BYTE   ,             /*   ISGRI_Y        */
  DAL_BYTE   ,             /*   PICSIT_Y       */
  DAL_BYTE   ,             /*   ISGRI_Z        */
  DAL_BYTE   ,             /*   PICSIT_Z       */
  DAL_BYTE   ,             /*   SELECT_FLAG    */
  DAL_USHORT ,             /*   ISGRI_PI       */
  DAL_FLOAT  ,             /*   ISGRI_ENERGY   */
  DAL_FLOAT  ,             /*   PICSIT_ENERGY  */
  DAL_BYTE   ,             /*   DUMMY_COUNTER  */
  DAL_BYTE   ,             /*   TIME_TAG       */
  DAL_BYTE   ,             /*   DETE           */
  DAL_BYTE   ,             /*   PHA            */
  DAL_FLOAT  ,		   /*   ENERGY         */
  DAL_BYTE   ,		   /*   RAWX           */
  DAL_BYTE   ,		   /*   RAWY           */
  DAL_USHORT ,		   /*   DELTA_POS      */
  DAL_BYTE   ,		   /*   REST_PHA       */
  DAL_FLOAT  ,		   /*   DETX           */
  DAL_FLOAT  ,		   /*   DETY           */
  DAL_BYTE   ,		   /*   PI             */
  DAL_BYTE   ,		   /*   REST_PI        */
  DAL_BYTE   ,		   /*   SPECTRUM       */
  DAL_BYTE   ,		   /*   GREY_FILTER    */
  DAL_BYTE   ,		   /*   SPEC_CORR      */
  DAL_SHORT  ,		   /*   DELTA_COUNT    */
  DAL_USHORT ,		   /*   COUNT_RATE     */
  DAL_BYTE   ,		   /*   STATUS         */
  DAL_USHORT , 		   /*   RATE_CORR      */
  DAL_BYTE   ,             /*   PSD_RAWFLAG    */
  DAL_USHORT ,             /*   PSD_DATA       */
  DAL_BYTE   ,             /*   PSD_ERR        */
  DAL_BYTE   ,             /*   PSD_TTP1       */
  DAL_BYTE   ,             /*   PSD_TTP2       */
  DAL_USHORT ,             /*   PSD_AMP        */
  DAL_BYTE   ,             /*   PSD_CORFLAG    */
  DAL_BYTE   ,             /*   MUL_FLAG       */
  DAL_BYTE   ,             /*   CAL_FLAG       */
  DAL_USHORT ,             /*   PSD_ADC        */
  DAL_USHORT ,             /*   PSD_CURVE      */
  DAL_USHORT ,             /*   PSD_LABEL      */
  DAL_BYTE                 /*   MULT           */
};


/*****************************************************************************/
/*                                                                           */
/*  DAL pointer: spihEventDS                                              */
/*                                                                           */
/*  DAL table in MEM containing the selected events                          */
/*                                                                           */
/*****************************************************************************/
static dal_element *spihEventDS = NULL;


/*
  The following is an adaptation of DALobjectSelect. We removed the code
  to process an object.
*/

/*****************************************************************************
 *
 * Select Rows and columns from THE INPUT data element.
 * Its rows are selected using
 * the selection expression given by ROWFILTER; see the DAL user manual
 * for a description of the row filter syntax. The selected rows
 * are written to the output table element ELEMENT.
 *
 * ELEMENT must be of base type DAL_TABLE and have defined all the columns that
 * the calling application wishes to have returned. Thus, the definition of
 * ELEMENT is used as selection critera for the column selection. ELEMENT
 * may have pre-existing rows of data; all rows passing the row selection
 * are appended to the end of ELEMENT.
 *
 * Note the following:
 *
 * 1. The input element must be conformable
 * to the output element ELEMENT, and have a table column defined for each
 * column name referenced in the ROWFILTER expression.
 *
 * 2. Only the table columns defined in ELEMENT, containing data from the
 * table rows which passed the row selection, will be returned to the
 * calling function.
 *
 * 3. Variable length columns may not be used either for row or column
 * selection. Thus, the output ELEMENT table may have no variable length
 * columns defined, and the selected input elements in the subset may not
 * have any of their variable length columns used for row selection.???
 *
 *****************************************************************************/
static int SpihElementSelect(
              dal_element  *inputElement,  /* I ptr to intermediate table */
	      DAL_CHARP     rowFilter,     /* I CFITSIO row filter expression               */
	      dal_element  *element,       /* IO element containing filtered rows           */
	      int           currentStatus)
{
  int  status    = ISDC_OK;
  int  tmpStatus = ISDC_OK;
  int  outCols;
  int  varLength;
  int  i,j;
  int  exact;

  long startRow;
  long numRows;
  long buffSize;
  long numValues;

  char   tmpColName[DAL_MAX_NAME_SIZE];
  char   card[FLEN_CARD];
  char **colName = NULL;

  void *xferBuff = NULL;

  dal_dataType    *type = NULL;


  fitsfile *cfptr = NULL;
  fitsfile *efptr = NULL;


  if(currentStatus != ISDC_OK) return(currentStatus);

  do
    {
      /*
       * initialize variables
       */

      *tmpColName = *card = 0;

      /*
       * determine the starting size of the output element
       */
      
      status = DALtableGetNumRows(element,&startRow,status);
      
      status = DALtableGetNumCols(element,&outCols,status);

      status = DALtableIsConformable(element,inputElement,&exact,status);
      if(status != ISDC_OK) continue;

      /*
       * determine the column names and data types of all the output
       * element columns
       */

      status = DALallocateStringBuffer(&colName,outCols,DAL_MAX_NAME_SIZE,
				       status);

      status = DALallocateDataBuffer((void**)&type,outCols*sizeof(dal_dataType),
				     status);

      for(i = 0, varLength = DAL_NOT_VARLENGTH; 
	  i < outCols && varLength == DAL_NOT_VARLENGTH && status == ISDC_OK; 
	  ++i)
	status = DALtableGetColStruct(element,i+1,NULL,colName[i],type+i,
				      NULL,&varLength,status);

      if(status != ISDC_OK) continue;

      status = DALelementGetFilePtr(element,&efptr,status);


      /* Apply the selection on the input table                              */
      
      status = DALelementGetFilePtr(inputElement,&cfptr,status);
      if(status != ISDC_OK) continue;

      if (rowFilter!=NULL) status = fits_select_rows(cfptr,cfptr,(char*)rowFilter,&status);

      /* determine the number of rows that passed the filtering */

      status = DALtableGetNumRows(inputElement,&numRows,status);

      /* add that number of rows to the output element */

      status = DALtableAddRows(element,-1,numRows,status);

      if(status != ISDC_OK) continue;

      /*
       * if the output element and the current child element have
       * an identical structure then we can to a simple (fast)
       * data transfer between them. Else we must copy data from
       * the temporary table to the output element one column at a
       * time; note that the column selection takes place in this
       * step
       */

      if(exact)
	{
	  /*
	   * the child element (i.e., temporary table) and output
	   * table have the same exact structure so transfer the
	   * data directly
	   */

	  /* allocate the data transfer buffer */

	  status = fits_read_key_lng(cfptr,"NAXIS1",&buffSize,
				     card,&status);
	  buffSize = buffSize*numRows;
	  status = DALallocateDataBuffer(&xferBuff,buffSize,status);

	  /* read all data in the inputElement */

	  status = fits_read_tblbytes(cfptr,1,1,buffSize,
				      (unsigned char*)xferBuff,&status);
	      
	  /* write the tmp table data to the output element */
	      
	  status = fits_write_tblbytes(efptr,startRow+1,1,
				       buffSize,(unsigned char*)xferBuff,
				       &status);	  	  
	    }
      else
	{
	  /*
	   * We must transfer the data from the temporary table to the
	   * output element one column at a time. Note that only the
	   * columns defined in the output element are transfered
	   */

	  for(j = 0; j < outCols && status == ISDC_OK; ++j)
	    {
	  
	      /*
	       * transfer the column data in the standard way
	       */
	      
	      /* compute the xfer buffer and initialize */

	      status = DALtableComputeBuffSize(inputElement,colName[j],0,
					       1,0,&buffSize,status);
	      status = DALallocateDataBuffer(&xferBuff,buffSize,
					     status);

	      /* read the column from the temporary table */

	      status = DALtableGetColBins(inputElement,colName[j],0,type+j,
					  1,0,&numValues,xferBuff,
					  status);

	      /* write the column to the output element */
	      
	      status = DALtablePutColBins(element,colName[j],0,type[j],
					  startRow+1,startRow+numRows,
					  numValues,xferBuff,status);
	    }
	}

      /* increment the output element starting row counter */
	  
      startRow += numRows;

      /* delete the temporary transfer table element */
	  

    }while(0); /* this loop only executes once */

  /*
   * clean up all the buffers and elements allocated in the function
   * if not NULL
   */

  if(type     != NULL) DALfreeDataBuffer(type,tmpStatus);
  if(xferBuff != NULL) DALfreeDataBuffer(xferBuff,tmpStatus);
  if(colName  != NULL) DALfreeStringBuffer(colName,outCols,tmpStatus);

  if(status > 0) status = -1*status + DAL_ERROR_CODE_BASE;

  return(status);
}


static int SpihIsSelectColumn(int EvPar,
				 Event_type   EvType,
				 char 	      *psdType,
				 char         *chanType)
{
   int i, j;
   int nCols;
   int iColsToGet[20];
   
  /* The parameter does not exist for this mode/instrument               */
  if (SpihColMult[EvPar][EvType]==0) return(0);
  
  nCols = 3;
  iColsToGet[0] = 0; iColsToGet[1] = 15; 
  iColsToGet[2] = strcmp(chanType,"PHA") ? 17 : 16;    
  if (EvType == SPI__PSD)
  {
     nCols = 4;
     iColsToGet[3] = strcmp(psdType,"RAW") ? 39 : 33;
  }
  else
  {
     if (EvType != SPI__SINGLE)
     {
        nCols = 5;
	iColsToGet[3] = 1;
	iColsToGet[4] = 40;
     }
  }
  
  for (i=0; i<nCols; ++i)
  {
     j = iColsToGet[i];
     if (j == EvPar)  return (1);
  }
    

  /* The parameter is not requested at all                               */
  return(0);
}



static int SpihCreateSelTable(const char   *nameDS,
				 Event_type    EvType,
				 int           Level,
				 char          *psdType,
				 char          *chanType,
				 int           MinAccept,
				 dal_element **newTable,
				 int           current_status)
{
  int status=ISDC_OK;
  int i, j;
  int iColsToGet[20];
  int nCols=0;
  
  if ( current_status!=ISDC_OK ) return( current_status );

  do {
    status=DALtableCreate(NULL,DAL_MEM,0,nameDS,newTable,status);
    if (status!=ISDC_OK) continue;
    
    nCols = 3;
    iColsToGet[0] = 0; iColsToGet[1] = 15; 
    iColsToGet[2] = strcmp(chanType,"PHA") ? 17 : 16;    
    if (EvType == SPI__PSD)
    {
       nCols = 4;
       iColsToGet[3] = strcmp(psdType,"RAW") ? 39 : 33;
    }
    else
    {
       if (EvType != SPI__SINGLE)
       {
          nCols = 5;
	  iColsToGet[3] = 1;
	  iColsToGet[4] = 40;
       }
    }
    
    for (i=0; i<nCols; ++i)
    {
       j = iColsToGet[i];
	if (SpihColForm[j]==DAL3_OBT) {
	  status=DALtableAddCol(*newTable,SpihColName[j],DAL3_OBT_DALTYPE,DAL3_OBT_BINSIZE,status);
	} else {
	  if (SpihColMult[j][EvType]>0) {
	    status=DALtableAddCol(*newTable,SpihColName[j],SpihColForm[j],SpihColMult[j][EvType],status);
	  } else {
	    status=DALtableAddCol(*newTable,SpihColName[j],DAL_USHORT,-1*SpihColMult[j][EvType],status);
	  }
	}
    }
  } while (0);

  return(status);
}



static int SpihSelectEvents(dal_element **ListDS,int NumChild,
	OBTime *StartSWG,OBTime *EndSWG,int NumSWG,Event_type EvType,
	int NumIntervals,OBTime *gtiStart,OBTime *gtiStop, 
	dal_element *InputDS, char *psdType, char *chanType, int status)
{
  int i,j,k,level,indMinOBT;
  int numTot,numRaw,numPrp,numCor,kill, rowToKill, rowCounter;
  long rowMin,nRows,numVTrows,buffSize,numValues,startRow,numNewRows,rowSel;
  char *locFilter;
  const char *RowFilter=NULL;
  dal_element *evtsVTable,**rawDS=NULL,**prpDS=NULL,**corDS=NULL,*curDS;
  dal_element *tmpSWG, *parent;
  char tmpSWID[DAL_MAX_ATT_STR_VALUE_SIZE];
  char tmpSWIDraw[DAL_MAX_ATT_STR_VALUE_SIZE];
  dal_dataType tmpType;
  void *xferBuff=NULL;
  int sizeFilt;
  int noSelectFlag;
  int l, numParents;
  ISDCLevel ColSel=ALL_PAR;
  
  if ( status!=ISDC_OK ) return(status);

  do {
    /* Create new subsets and determining the level                          */
    level=0;
    /* Subset for RAW events                                                 */
    status=DAL3GENbuildSubList(ListDS,NumChild,"*RAW",&rawDS,&numRaw,status);
    if (status!=ISDC_OK) continue;
    if (numRaw>0) level=1;
    /* Subset for PRP events                                                 */
    status=DAL3GENbuildSubList(ListDS,NumChild,"*PRP",&prpDS,&numPrp,status);
    if (status!=ISDC_OK) continue;
    if (numPrp>0) {
      if (level==0) {
	level=12;
      } else {
	level=2;
      }
    }
    /* Subset for COR events                                                 */
    status=DAL3GENbuildSubList(ListDS,NumChild,"*COR",&corDS,&numCor,status);
    if (status!=ISDC_OK) continue;
    if (numCor>0) {
      if (level==0) {
	level=13;
      } else {
	level=3;
      }
    }
    /* Did we find some event data structures ?                              */
    if (level==0) {
      status=DAL3GEN_NO_EVENTS;
      continue;
    }
    /* Do we have access to the prepared data, if we need them ?             */
    if (numPrp==0 && NumIntervals!=0) {
      status=DAL3GEN_INEXISTENT_PARAMETER;
      continue;
    }

    /* Check that the input group has the correct structure                  */
    numTot=numRaw+numPrp+numCor;
    
    if ( numTot==1 || numTot==numRaw || (numTot==2*numRaw && numCor==0) ||
	 (numTot==3*numRaw && numCor==numPrp) || (NumSWG!=0 && NumSWG!=numPrp) ) {

      /* Now we preselect the SWGs that fall within the good time intervals  */
      /* First, treat the DAL3_NO_OBTIME parameters                          */
      for (i=0; i<NumIntervals; i++) {
	if (gtiStart[i]==DAL3_NO_OBTIME) gtiStart[i]=0;
	if (gtiStop[i]==DAL3_NO_OBTIME) gtiStop[i]=DAL3_MAX_OBTIME;
      }
      /* Check whether some data structures can be removed                   */
      for (j=NumSWG-1; j>=0; j--) {
	kill=1;
	for (i=NumIntervals-1; i>=0 && kill==1; i--) {
	  if (gtiStop[i]>=StartSWG[j] && gtiStart[i]<=EndSWG[j]) kill=0;
	}
	/* SPR 1942, we can no longer assume that if we have 10 science window
	   groups we also have 10 elements in our rawDS etc since now we run 
	   science window clean */

	if (1==kill) {
	  rowToKill = -1;
	  /* now, let's see if it's really the right one */
	  status=DAL3GENfindSWG(InputDS,j+1,&tmpSWG,status);
	  /*	  status=DALattributeGet(tmpSWG,"SWID",DAL_CHAR,
		  tmpSWID,NULL,NULL,status); */
	  /* See if the rawDS[k] has tmpSWG as a parent */
	  /* The previous choice of checking the text SWID's from ALL of 
	     the raw structures and the SWID of the swg was a very very 
	     very very very bad idea since it caused dal to open and therefore
	     uncompress a load of files.  Brain dead doesn't even begin to 
	     cover how bad this was.  Anyway, this is a better way.  JP 
	     says that comparing dal_element * is ok with the dal 2.0+ hash
	     table. SCREW 885 */
	  for (k=0;k<numRaw;k++) {
	    status=DALelementGetNumParents(rawDS[k],&numParents,status);
	    for (l=0;l<numParents;l++) {
	      status=DALelementGetParent(rawDS[k],l+1,&parent,
					 NULL,NULL,status);
	      if (parent == tmpSWG) {
		rowToKill = k;
		break;
	      }
	      if (rowToKill > -1) break;
	    }
	  }
	  if (rowToKill > -1) {
	    status=DAL3GENremoveFromList(rawDS,&numRaw,rowToKill,status);
	    status=DAL3GENremoveFromList(prpDS,&numPrp,rowToKill,status);
	    if (numCor>0) status=DAL3GENremoveFromList(corDS,&numCor,
						       rowToKill,status);
	  }
	}
      }
      /* Creation of the ouput table                                         */
      /* Previous output selection table is deleted if it exists             */
      if ( spihEventDS!=NULL ) {
	status=DALobjectClose(spihEventDS,DAL_DELETE,status);
	if (status!=ISDC_OK) continue;
      }
      /* Creation of the ouput selection table                               */
      status=SpihCreateSelTable("EVENTS",EvType,level,psdType, chanType ,2,&spihEventDS,status);
      /* Add some keywords                                                   */
      status=DALattributePut(spihEventDS,"LEVEL",DAL_INT,&level,NULL,NULL,status);
      status=DALattributePut(spihEventDS,"DATAMODE",DAL_INT,&EvType,NULL,NULL,status);
      if (status!=ISDC_OK) continue;
      /* Run through all the science windows                                 */
      if (level>10) {
	numRaw=1;
	if (level==12) rawDS[0]=prpDS[0];
	if (level==13) rawDS[0]=corDS[0];
      }
      indMinOBT=0;
      
      /* Turn this off and hardcode noSelectFlag to 1 - C.G. */
      /*
      noSelectFlag=0; 
      if (RowFilter==NULL && NumIntervals==0) noSelectFlag=1;  
      */
      noSelectFlag = 1;
      
      /* We create first the "virtual" table                                 */
      if (noSelectFlag==0)
	status=SpihCreateSelTable("EVENT_VIRTUAL_TABLE",EvType,level,psdType, chanType,1,&evtsVTable,status);
      else 
	evtsVTable=spihEventDS;
      if (status!=ISDC_OK) continue;
      startRow=1;

      for (i=0;i<numRaw && status==ISDC_OK;i++) {
	/* Add the rows                                                      */
	status=DALtableGetNumRows(rawDS[i],&numNewRows,status);
	/* If no selection has to be made, one can simplify the treatment    */
	if (noSelectFlag) {
	  status=DALtableAddRows(evtsVTable,-1,numNewRows,status);
	} else {
	  /* We need to correct the number of rows in the "virtual" table    */
	  status=DALtableGetNumRows(evtsVTable,&rowSel,status);
	  if (numNewRows>=rowSel)
	    status=DALtableAddRows(evtsVTable,-1,numNewRows-rowSel,status);
	  else 
	    status=DALtableDelRows(evtsVTable,numNewRows+1,rowSel-numNewRows,status);

	  if (status!=ISDC_OK) break;
	}
	/* Transfer the columns into the table                               */
	for (j=DAL3GEN_FIRST_PAR; j<= DAL3GEN_LAST_PAR && status==ISDC_OK; j++) {
	  if (SpihIsSelectColumn(j,EvType,psdType,chanType)) {
	    if      (SpihColLevl[j]==1 || SpihColLevl[j]==10+level) curDS=rawDS[i];
	    else if (SpihColLevl[j]==2) curDS=prpDS[i];
	    else                           curDS=corDS[i];

	    if ( SpihColMult[j][EvType]>0 )
	      tmpType=SpihColForm[j];
	    else
	      tmpType=DAL_USHORT;

	    if (tmpType==DAL3_OBT) tmpType=DAL3_OBT_DALTYPE;

	    status = DALtableComputeBuffSize(curDS,SpihColName[j],0,1,0,&buffSize,status);

	    if (status==DAL_TABLE_HAS_NO_ROWS) {
	      status=ISDC_OK;
	      /* No need to continue the loop, as all tables should have the */
	      /* same number of rows                                         */
	      break;
	    }
	    
	    status = DALallocateDataBuffer(&xferBuff,buffSize,status);

	    /* read the column from the temporary table                      */
	    status = DALtableGetColBins(curDS,SpihColName[j],0,&tmpType,1,0,&numValues,xferBuff,status);

	    /* write the column to the output element */

	    status = DALtablePutColBins(evtsVTable,SpihColName[j],0,tmpType,startRow,startRow+numNewRows-1,
					numValues,xferBuff,status);
	  }
	}
	/* Perform the selection                                             */
	if (!noSelectFlag) {
	  if (NumIntervals==0) {
	    status=SpihElementSelect(evtsVTable,RowFilter,spihEventDS,status);
	    if (status==DAL_TABLE_HAS_NO_ROWS) status=ISDC_OK;
	  } else {
	    status=DALtableGetNumRows(evtsVTable,&numVTrows,status);
	    /* Build the full selection string, including all OBTs           */
	    sizeFilt=1024+((RowFilter==NULL) ? 0 : strlen(RowFilter)) ;
	    locFilter=(char *) malloc(sizeFilt) ;
	    memset(locFilter,0,sizeFilt);
	    if ( RowFilter==NULL)
	      strcpy(locFilter,"(");
	    else
	      sprintf(locFilter,"(%s) && (",RowFilter);
	    for (j=indMinOBT;j<NumIntervals;j++) {
	      /* Select the rows for the jth GTI                             */
	      status=DAL3GENtableBSearchOBT(prpDS[i],SpihColName[OB_TIME],0,
					    gtiStart[j],gtiStop[j],&rowMin,&nRows,status);
	      if (status!=ISDC_OK) break;
	      /* SP patch for spr 1992 */
	      /* If this interval is beyond the ScW, we can ignore it. There is */
	      /* also no need to loop on the following GTI's                    */
	      if (rowMin>numVTrows) break;
	      /* If this interval is before the Science Window, we can ignore   */
	      /*  this GTI not only for this ScW, but for the other ones too    */
	      if (rowMin==1 && nRows==0) {
	        indMinOBT++;
		continue;
	      }
	      if (nRows>0)
		sprintf(locFilter+strlen(locFilter),"(#row>=%ld && #row<=%ld)||",rowMin,rowMin+nRows-1);
	      /* realloc if we are short on memory...                        */
	      /* SPR 2238, duh, we got this a bit wrong */
	      if (strlen(locFilter)+900 >sizeFilt) {
		sizeFilt+=1024;
		locFilter=(char *) realloc(locFilter,sizeFilt) ;
		memset(locFilter+sizeFilt-1024,0,1024);
	      }
	    }
	    /* Did we find any row to select upon?                           */
	    if (locFilter[strlen(locFilter)-1]!='(') {
	      locFilter[strlen(locFilter)-2]=')';
	      locFilter[strlen(locFilter)-1]=0;
	      status=SpihElementSelect(evtsVTable,locFilter,spihEventDS,status);
	      if (status==DAL_TABLE_HAS_NO_ROWS) status=ISDC_OK;
	      /* SPR 1992, don't do this */
	      /* if (rowMin>numVTrows) {
		 indMinOBT=j;
		 break;
		 } else if (rowMin+nRows-1<=numVTrows) {
		 indMinOBT=j+1;
		 } */
	    }
	    free(locFilter);
	  }
	  /* In case of no selction, just update the number of read rows     */
	} else startRow+=numNewRows;

	if (status!=ISDC_OK) break;
      }
    } else {
      status=DAL3GEN_INVALID_GROUP;
    }

    /* Delete the virtual table                                                */
    if (evtsVTable!=spihEventDS) status=DALobjectClose(evtsVTable,DAL_DELETE,status);

  } while (0);

  /* Delete the sublists                                                     */ 
  free(rawDS);
  free(prpDS);
  free(corDS);

  DALfreeDataBuffer(xferBuff,ISDC_OK);
   
   return status;
}


/*****************************************************************************/
/*                                                                           */
/*  Function: SpihGetNumEvents                                            */
/*                                                                           */
/*  "SpihGetNumEvents" retrieves the number of selected events            */
/*                                                                           */
/*  Parameters:                                                              */
/*    Name          Type          I/O  Description                           */
/*                                                                           */
/*    NumEvents     long         * O   Number of selected events             */
/*                                                                           */
/*  Errors:                                                                  */
/*                                                                           */
/*    No specific error is returned by this function                         */
/*                                                                           */
/*****************************************************************************/
int SpihGetNumEvents(long *NumEvents,
			int   current_status)
{
  int status=ISDC_OK;

  if ( current_status!=ISDC_OK ) return(current_status);

  status=DALtableGetNumRows(spihEventDS,NumEvents,status);

  return(status);
}


/*****************************************************************************/
/*                                                                           */
/*  Function: SpihGetEventsBins                                           */
/*                                                                           */
/*  "SpihGetEventsBins" retrieves the content of the column "colID" for   */
/*  the selected events from rows "startBin" to "endBin"                     */
/*                                                                           */
/*  Parameters:                                                              */
/*    Name          Type          I/O  Description                           */
/*                                                                           */
/*    ColID         Event_params   I   Column identifier                     */
/*    ColType       dal_dataType * I   DAL Type of the "ColBuff" array       */
/*    StartBin      long           I   First row to read in the event table  */
/*    EndBin        long           I   Last row to read in the event table   */
/*    ColBuff       void         * O   Buffer where the event dat is stored  */
/*                                                                           */
/*  Errors:                                                                  */
/*                                                                           */
/*    Specific errors produced by this function are caught by their callers  */
/*                                                                           */
/*****************************************************************************/
int SpihGetEventsBins(Event_params  ColID,
			 dal_dataType *ColType,
			 long          StartBin,
			 long          EndBin,
			 void         *ColBuff,
			 int           current_status)
{
  int status=ISDC_OK;
  int level;
  Event_type evType;
  long numValues;

  if ( current_status!=ISDC_OK ) return(current_status);

  if (ColID<DAL3GEN_FIRST_PAR || ColID> DAL3GEN_LAST_PAR) return(DAL3GEN_INVALID_PARAMETER);

  status=DALattributeGet(spihEventDS,"LEVEL",DAL_INT,&level,NULL,NULL,status);
  status=DALattributeGet(spihEventDS,"DATAMODE",DAL_INT,&evType,NULL,NULL,status);
  
  if (SpihColMult[ColID][evType]==0) {
    status=DAL3GEN_INCOMPATIBLE_PARAMETER;
    return(status);
  }
  if (SpihColLevl[ColID]>level || (level>10 && SpihColLevl[ColID]!=level-10)) {
    status=DAL3GEN_INEXISTENT_PARAMETER;
    return(status);
  }

  if (ColID==OB_TIME) {
    status=DAL3GENtableGetOBTBins(spihEventDS,SpihColName[ColID],0,StartBin,EndBin,&numValues,
				  (OBTime *)ColBuff,status);
  } else {
    status=DALtableGetColBins(spihEventDS,SpihColName[ColID],0,ColType,StartBin,EndBin,&numValues,
			      ColBuff,status);
  }

  return(status);
}


/*****************************************************************************/
/*                                                                           */
/*  Function: SpihGetEvents                                               */
/*                                                                           */
/*  "SpihGetEvents" retrieves the content of the column "colID" for       */
/*  the selected events from rows "startBin" to "endBin"                     */
/*                                                                           */
/*  Parameters:                                                              */
/*    Name          Type          I/O  Description                           */
/*                                                                           */
/*    ColID         Event_params   I   Column identifier                     */
/*    ColType       dal_dataType * I   DAL Type of the "ColBuff" array       */
/*    ColBuff       void         * O   Buffer where the event dat is stored  */
/*                                                                           */
/*  Errors:                                                                  */
/*                                                                           */
/*    Specific errors produced by this function are caught by their callers  */
/*                                                                           */
/*****************************************************************************/
int SpihGetEvents(Event_params  ColID,
		     dal_dataType *ColType,
		     void         *ColBuff,
		     int           current_status)
{
  int status=ISDC_OK;

  status=SpihGetEventsBins(ColID,ColType,1,0,ColBuff,current_status);

  return(status);
}


/*****************************************************************************/
/*                                                                           */
/*  Function: SpihCloseEvents                                             */
/*                                                                           */
/*  "SpihCloseEvents" deletes the selected event table                    */
/*                                                                           */
/*  Parameters:                                                              */
/*    Name          Type          I/O  Description                           */
/*                                                                           */
/*  Errors:                                                                  */
/*                                                                           */
/*    No specific error is returned by this function                         */
/*                                                                           */
/*****************************************************************************/
int SpihCloseEvents(int current_status)
{
  int status=ISDC_OK;

  if ( current_status!=ISDC_OK ) return(current_status);

  if (spihEventDS!=NULL) status=DALobjectClose(spihEventDS,DAL_DELETE,status);
  spihEventDS=NULL;

  return(status);
}




