/* 
-------- Function: -------------------------------------------------------------

     SpihistProcessData


-------- Description: ----------------------------------------------------------

     This program creates time histories from the SPI raw events data.  There 
     are two output file formats available: type II PHA, and ISDC.


-------- Functions called: -----------------------------------------------------

   Spihist Functions:
      CheckParams            -- Validates SPIHIST input parameters
      CreateEventType        -- Returns event type for given multiplicity
      DetRanges              -- Returns detector ranges to be processed (ISDC
                                output option only)
      EchoSpihistInput       -- Echoes the input parametes exclusive to SPIHIST 
      GetGTForSCW	     -- Extracts OBTimes from gtList for call to
      				ReadEvents
      GetRange               -- checks the validity of input string and parses 
                                it to convert it to integer range arrays
      InitGoodTimesStr       -- Initializes GOOD TIME structure
      InitializeHist         -- Initializes histograms for given pointing
      ListDither             -- Creates a list of pointing information (x,y,z-
                                ra and dec values)
      MultiplicityToProcess  -- Returns multiplicities that need to be processed
      ParseNumString	     -- Extracts detector ranges to process based on
      				user input string.
      Processdata            -- Extracts good data based on user criteria and 
                                creates time histories
      ProgVer                -- Returns the task name and version number
      ReadBinFile            -- Reads the binning info from a file
      ReadDither             -- Reads POINTING information
      ReadEvents             -- Reads the EVENTS extension from the SPI FITS 
                                file
      ReadGT                 -- Reads the GOOD TIME interval from the SPI FITS 
                                file
      ReadPseudoDetTable     -- Reads pseudo detector lookup table
      SetDetFlags            -- Set flags for detectors to be processed
      SpihistOutput          -- Writes output to the file

   SPI Functions:
      Allocate               -- Allocates specified number of bytes and keeps 
                                track of all allocated memory during a run
      Cleanup                -- Deallocates all memory allocated during the 
                                current run in the event of an error
      DisplayVersion         -- Displays program name and version
      DispMsg                -- Writes messages to the screen and a log file
      FreeNode               -- Frees some of the allocated memory when not 
                                needed
      InitSpiStr             -- Initializes structures that are common to SPI 
                                programs


   COMMON Functions:
      CommonCloseSWG         -- Closes the observation group

   Macro
      Max                    -- Returns the maximum of the two input values
      Min                    -- Returns the minimum of the two input values


-------- Usage: ----------------------------------------------------------------

   spihist in-og-dol det-spec-dol out-og-dol swg-index-dol gti-dol deadtime-dol
           ebounds-dol clobber detnums  multievts tstart tstop outputFormat 
	   dtype instrume chantype detsummed pntgsummed 

       INFILE        char      : SPI FITS file
       FNAME         char      : Name of the output file to be created
       BINFILE       char      : Name of the file containing binning info
       ELOW          float     : Lower bound on energy range
       EHIGH         float     : Upper bound on energy range
       DETNUMS       char      : Ids of detectors to be processed (allowed 
                                 characters in the string are digits, '-', and 
				 ','
       TSTART        char      : Start time for data accumulations
       TSTOP         char      : Stop time for data accumulation
       FORMAT        char      : Format for the output file
       DTYPE         int       : Output data format (counts=1, rates=2)
       TELESCOPE     char      : String listing telescope/mission
       INSTRUME      char      : String listing instrument/detector
       FILTER        char      : String listing instrument filter in use
       AREASCAL      float     : Area scaling factor
       BACKSCAL      float     : Background scaling factor
       CORRSCAL      float     : Corrections scaling factor
       CORRFIL       char      : Associated corrections file
       BACKFIL       char      : Associated background file
       RESPFIL       char      : Associated redistribution matrix file
       ANCRFIL       char      : Associated ancillary file
       CHANTYP       char      : Type of detector channels (PHA, PI etc)
       HDUCLAS2      char      : String containing HDUCLAS2 value
       QQUAL         int       : Flag as to whether quality array passed down
       QGROUP        int       : Flag as to whether grouping array passed down
       XFLT0001      char      : XSPEC filter description for each XSPEC 
                                 filter keyword
       CLOBBER       int       : 1=Overwrite existing output file, if any
                                 0=Prompt user before overwriting existing 
				 output file
       CHATTER       int       : Chattiness flag for o/p (5 quite,10 normal,
                                 >20 silly)


-------- Include files ---------------------------------------------------------

   "spihist.h"


-------- Authors/Modification History: -----------------------------------------

(For overall history, refer to progver.C)

   Craig Gordon    (3.0.7; Apr 03) Additional mod to undetermined SPImode
                                   case.  Now uses previous pointing mode
				   as a default.
   Volker Beckmann (3.0.6; Apr 03) Mods to handling of undetermined
		                 SPImode cases for dets 0-18,85-103.
   Craig Gordon	   (3.0.2; Feb 03) Initialize nmulti and multiplicity[] at
		                  start of each pointing.  This fixes problem
				  of unnecessary selectEvents calls for
				  multiplicity = 0 events.
                 (3.0.1; Jan 03)
		 (3.0.0; Dec 02) Major overhaul to add capibility to read
		                  On-board spectra.  Also removed obsolete
				  code to process and bin multiple pointings
				  in one pass.
   Craig Gordon  (2.2.1; Nov 02) Removed pEventcriteria->tstart and tstop
   				  parameters, no longer convert event times
				  to IJD format.
   Craig Gordon  (2.2.0; Sep 02) Modified format and placed inside new main 
   				 function to conform with CTS. 
   Craig Gordon  (2.1.9; Jul 02) GetGtForScw now handles arbitrary distribution
   				  of GTIs among detectors in a SCW.
   Craig Gordon (2.1.8; May 02) Added capability to read SPI_modes other than
   		                 just OPERATIONAL.  
   Craig Gordon (2.1.7; Apr 02) Portable to Linux.
   Craig Gordon (2.1.5; Feb 02) Retrieve OBtimes directly from .gti ext.
   Sandhia Bansal  (2.0.1; Nov 00)  Linked with new spiCommon library.
   Sandhia Bansal  (2.0.0; Jun 00)


----------------------------------------------------------------------------- */

#include "spihist.h"


void SpihistProcessData(memoryPtr *pBase, memoryPtr *pLast, dal_element 
		*outGroup, headerParsPtr pHeader, outputParsPtr pOutput,
		eventCriteriaPtr pEventcriteria, eventsInfoPtr pEvents,
		binningPtr pBin, detinfoPtr pDet, int *detSumFlag,  int
		*pntSumFlag, char *coeffDOL)
{
   FILE              *filePtr=NULL;

   DITHERNODE        *ditherList=NULL;


   pointingInfoPtr   pDither=NULL;
   goodTimesPtr      pGtimes=NULL;
   gtIntPtr             **gtList=NULL; 
   gtRowsPtr         pGtrows=NULL;  

   double            startTime=0.0, obsTime=0.0, obmidtime = 0.0;

   extern const int gl_boundaries[2][6];
   extern const short gl_emer;
   extern const short gl_oper;
   extern const short gl_undefined;

   int               status=ISDC_OK, i=0, j=0, k=0;
   int               idet=0, iPointing=0;
   int               detid=0;
   int               dcount=0;
   int               nmulti=0;
   int               multiplicity[7];
   int               eventType=0;
   int		     nGTintv=0;
   int		     detStart=0;
   int		     detStop=0;
   int		     evCount=0;
   int		     readOBFlag=0;
   int		     operFlag=0;
   short	     spiModeInt=0;
   short	     prevSpiMode = gl_oper;
   short	     obSpectrumFlag=0;
   char              message[DAL_MAX_STRING];
   char              program[PROGVER_SIZE];
   char              version[PROGVER_SIZE];
   char		     modeString[15];
   
   dal_accessType    access=DAL_DISK;

   OBTime            obStart=NULL, obStop=NULL;
   OBTime	     *obgtStart=NULL, *obgtStop=NULL;
   
   SPI_mode	     spiMode;

   int ii=0, im=0;

   long maxnGTI=0;  /* added for 2.1.6, nGTIs no longer fixed at 3 */
   
   /* Get the program name and version number */
   ProgVer(program, version);

   if (strcmp(pOutput->format, "ISDC") == 0)
   {
      Allocate(program, "-pGtimes", 0, sizeof(GOODTIMES), (void **) &pGtimes, 
	       pBase, pLast);
      InitGoodTimesStr(pGtimes);
   }

   ReadPseudoDetTable(pBase, pLast, pDet);

   pDet->maxdet = pDet->nDetRows; 

   /* read binning file */
   strcpy(message, "Reading Binning File...");
   DispMsg(Log_1, message, status, program, pOutput->chatter);

   status = ReadBinFile(outGroup, pBase, pLast, pHeader->chantype, pBin);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error while reading binning file.");
      Cleanup(status, message, *pBase, program);
   }

   pDet->detchans = pBin->nEnergy - 1;

   /* read pointing extension */
   strcpy(message, "Reading Pointing Extension... ");
   DispMsg(Log_1, message, status, program, pOutput->chatter);

   Allocate(program, "-pDither", 0, sizeof(POINTINGINFO), (void **) &pDither,
	    pBase, pLast);

   InitPointingStr(pDither);
   ReadDither(outGroup, pBase, pLast, pDither, pOutput);


   /*  initialize det and hist arrays */
   Allocate(program, "-det", 0, pDet->maxdet*sizeof(int), 
	    (void **) &pDet->det, pBase, pLast);

   pDet->hist = NULL;
   Allocate(program, "-hist", 0, pDet->maxdet*sizeof(long *), 
	    (void **) &pDet->hist, pBase, pLast);

   for (idet=0; idet<pDet->maxdet; idet++)
   {
      pDet->hist[idet] = NULL;
      Allocate(program, "-hist[idet]", 0, pBin->nEnergy*sizeof(long), 
	       (void **) &pDet->hist[idet], pBase, pLast);
   }

   /* Read Good Time extension */
   strcpy(message, "Reading Good Time Extension... ");
   DispMsg(Log_1, message, status, program, pOutput->chatter);

   Allocate(program,"-gtrows",0,sizeof(GTROWS),(void **)&pGtrows,pBase,pLast);
   pGtrows->livetime = NULL;
   pGtrows->nInts = NULL;
   pGtrows->pointingId = NULL;
   pGtrows->nRows = 0;
   pGtrows->total = 0;
   gtList = ReadGT(outGroup, pBase, pLast, pDither->nDT, pDet->maxdet,
		   pGtimes, &maxnGTI, pGtrows, &status);
   /*  Any OB spectra files to check?  An empty coeffDOL string
       is used for now to indicate NO. */
   readOBFlag = !(strcmp(coeffDOL," ")==0) ;
   
   /* Get detector ranges to be processed */
   if (strcmp(pOutput->format, "ISDC") == 0) 
      DetRanges(pGtimes, pDet);

   else
      GetRange(pBase, pLast, pDet, detSumFlag, pntSumFlag);

   /* Set flag for detectors to be processed */
   SetDetFlags(pDet);

   Allocate(program, "-obStart", 0, sizeof(OBTime), 
	    (void **) &obStart, pBase, pLast);
   Allocate(program, "-obStop", 0, sizeof(OBTime), 
	    (void **) &obStop, pBase, pLast);

   while ((status == ISDC_OK) && (iPointing < pDither->nDT))
   {
      DispMsg(Log_1, " ", status, program, pOutput->chatter);
      DispMsg(Log_1, " ", status, program, pOutput->chatter);
      sprintf(message, "Pointing (%d): \n", iPointing+1);
      DispMsg(Log_1, message, status, program, pOutput->chatter);

      sprintf(message, "     %f - %f\n", pDither->times[iPointing], 
	      pDither->times[pDither->nDT+iPointing]);
      DispMsg(Log_1, message, status, program, pOutput->chatter);

      InitializeHist(pBin->nEnergy, pDet);
      InitDetSpiModes(pDet, readOBFlag);
      nmulti = 0;
      for (i=0; i<7; ++i)
      {
         multiplicity[i] = -1;
      }
      
      if (readOBFlag)
      {
	 strcpy(message, "Processing detector gti spiMode and OB spectra information ....");
	 DispMsg(Log_1, message, status, program, pOutput->chatter);

	 j=0;
	 for (k=0; k<4; k+=2)
	 {
            detStart = k ? gl_boundaries[0][k] : 0;
	    detStop = k ? gl_boundaries[0][k+1] : gl_boundaries[0][0];
	    for (i=detStart; i<detStop; ++i)
	    {
	       if(gtList[iPointing][i])
	       {
        	  status = SPImode(outGroup, gtList[iPointing][i][0].obStart,
	 			 &spiModeInt, &obSpectrumFlag, status);
		  if (status != ISDC_OK)
		  {
		     strcpy(message, "Error in function SPImode");
		     Cleanup(status, message, *pBase, program);
		  }
		  else 
		  {
		     if (spiModeInt == gl_undefined)
		     {
			status = SPImode(outGroup, gtList[iPointing][i][0].obStop, &spiModeInt, &obSpectrumFlag, status);
			if (status != ISDC_OK)
			{
			    strcpy(message, "Error in function SPImode");
			    Cleanup(status, message, *pBase, program);
			}
			else if (spiModeInt == gl_undefined) 
			{
			   obmidtime = (gtList[iPointing][i][0].obStop + gtList[iPointing][i][0].obStart) / 2.0;
			   status = SPImode(outGroup, gtList[iPointing][i][0].obStop, &spiModeInt, &obSpectrumFlag, status);
			   if (status != ISDC_OK)
			   {
			       strcpy(message, "Error in function SPImode");
			       Cleanup(status, message, *pBase, program);
			   }		      				     
			}
		     }		      
		  }
		  if (spiModeInt == gl_undefined) 
		  {
		     if (prevSpiMode == gl_oper)
		     {
			strcpy(modeString, "OPER");
		     }
		     else
		     {
			strcpy(modeString, "EMER");
		     }
		     sprintf(message, "No valid SPI mode found! Assuming mode to be %s and Spectrum Flag = %d", 
		    			 modeString, obSpectrumFlag);
		     DispMsg(Warning_1, message, status, program, pOutput->chatter);
		     spiModeInt = prevSpiMode; 
		  }
		  pDet->spiMode[j] = spiModeInt;
		  pDet->obSpectraFlag[j] = obSpectrumFlag;
		  if (spiModeInt == gl_emer)
		  {
	        	pDet->obEmerFlag = 1;
		  }
		  else if (spiModeInt == gl_oper)
		  {
		     operFlag = 1;
		     if (obSpectrumFlag)
		     {
		        pDet->obOperFlag = 1;
		     }
		  }
	       }
	       ++j;
	    }
	 }
	 if (pDet->obEmerFlag && operFlag)
	 {
            strcpy(message, "Multiple SPImodes detected for spectra - Unable to process");
            DispMsg(Error_2, message, status, program, pOutput->chatter);	 
	    Cleanup(status, message, *pBase, program);
	 }
	 
	 prevSpiMode = operFlag ? gl_oper : gl_emer;
	 
	 ReportOutputType(pDet, pOutput);
      } /* end if readOBFlag */
      /* Determine multiplicity ranges to be processed */
      status = MultiplicityToProcess(pDet, pGtrows, iPointing, 
      				&nmulti, multiplicity);
      if (status != ISDC_OK)
      {
	 strcpy(message, "Invalid multiplicity");
	 Cleanup(status, message, *pBase, program);
      }   

      obStart = pDither->obStart[iPointing];
      obStop = pDither->obStop[iPointing];

      nGTintv = GetGTForSCW(pBase, pLast, gtList, pGtrows, iPointing, pDet, 
      			&obgtStart, &obgtStop);

      strcpy(message, "Processing Events Data....\n");
      DispMsg(Log_1, message, status, program, pOutput->chatter);
      for (im=0; im<=2; im++)
      {
	 switch (im)
	 {
	    case 1:
	       spiMode = EMERGENCY;
	       strcpy(modeString, "EMERGENCY");
	       break;
	    case 2:
	       spiMode = DIAGNOSTIC;
	       strcpy(modeString, "DIAGNOSTIC");
	       break;
	    default:
	       spiMode = OPERATIONAL;
	       strcpy(modeString, "OPERATIONAL");
	 }
	 sprintf(message, "SPI_mode:  %s",modeString);
	 DispMsg(Log_1, message, status, program, pOutput->chatter);		
	 for (i=0; i<nmulti; i++)
	   /*for (i=ii; i<nmulti; i++)*/
	 {
	    sprintf(message, "Processing detectors of multiplicity %d...", 
		    multiplicity[i]);
	    DispMsg(Log_1, " ", status, program, pOutput->chatter);
	    DispMsg(Log_1, message, status, program, pOutput->chatter);

	    CreateEventType(multiplicity[i], &eventType);
	    /*printf("i:%d mult:%d eventType:%d\n", i, multiplicity[i],
	      eventType);*/

	    pDet->multiplicity = multiplicity[i];

	    strcpy(message, "Reading events...");
	    DispMsg(Log_1, message, status, program, pOutput->chatter);

	    /* read the EVENTS extension */
	    ReadEvents(outGroup, pBase, pLast, eventType, 
		       pHeader->chantype, pEventcriteria->psdType, 
		       pDet->multiplicity, nGTintv, obgtStart, obgtStop, 
		       pEvents, spiMode);
	    /*printf("nevent:%d \n", pEvents->nEVENT);*/
	    if (pEvents->nEVENT > 0)   /* data available to process */
	    {
	       /* Process data */
	       strcpy(message, "Processing Data... ");
	       DispMsg(Log_1, message, status, program, pOutput->chatter);
	       evCount = 0;
	       ProcessData(iPointing, pBin, pDet, pEvents, gtList, 
	       			pGtrows, &evCount);
	       sprintf(message, "nEvents binned: %d \n", evCount);
	       RILlogMessage(NULL, Log_1, message);

	       /* Free memory */
	       FreeNode(pBase, (void **) &pEvents->obTime, pLast);
	       FreeNode(pBase, (void **) &pEvents->dete, pLast);
	       FreeNode(pBase, (void **) &pEvents->pi, pLast);
	       FreeNode(pBase, (void **) &pEvents->psdFlag, pLast);
	    }   /* if there are events to be processed */

	    else
	    {
	       sprintf(message, "No events accumulated...\n");
	       DispMsg(Alert_1, message, status, program, pOutput->chatter);
	    }

	 } /* end multiplicity loop */
      } /* end spiMode loop */
      FreeNode(pBase, (void **) &obgtStart, pLast);
      FreeNode(pBase, (void **) &obgtStop, pLast);

      /* Add dither info to the dither list if output is to be
	 written as distinct pointings in PHAII format */
      if (strcmp(pOutput->format, "PHA") == 0)
      {
	 startTime = pDither->times[iPointing];

	 obsTime = 
	    pDither->times[pDither->nDT+iPointing] - 
	    startTime;

	 CopyRaDec(iPointing, pDither);
	 dcount = ListDither(pBase, pLast, startTime, 
			     obsTime, pDither->raDec, 
			     &ditherList);
      }
      dcount = ((iPointing==pDither->nDT-1) ? -(dcount) : dcount);
      
      if (pDet->obOperFlag || pDet->obEmerFlag)
      {
         status = AddObSpectra(outGroup, coeffDOL, pBin, iPointing, pDet, 
			pOutput, pDither, gtList);
	 if (status != ISDC_OK)
	 {
            Cleanup(status, message, *pBase, program);
	 }
      }
      SpihistOutput(outGroup, access, pBase, pLast, iPointing, dcount,  
		    &ditherList, gtList, pGtrows, pOutput, pBin, pDet, 
		    pHeader, *detSumFlag, *pntSumFlag, pDither->nDT);

      iPointing++;
   } /* end pointing loop */

   FreeNode(pBase, (void **) &obStart, pLast);
   FreeNode(pBase, (void **) &obStop, pLast);

   for (i=0; i<pDither->nDT; i++)
   {
      for (j=0; j<pDet->maxdet; j++)
	 FreeNode(pBase, (void **) &gtList[i][j], pLast);
      FreeNode(pBase, (void **) &gtList[i], pLast);
   }
   FreeNode(pBase, (void **) &gtList, pLast);

   if (pGtimes)
      FreeNode(pBase, (void **) &pGtimes->detid, pLast);

  /* free up all alocated memory and call CommonExit to exit */
   status = CommonCloseSWG(outGroup, ISDC_OK);
   Cleanup(status, " ", *pBase, program);
} 













