/* 
-------- Function: -------------------------------------------------------------

     SpihistGetParams


-------- Description: ----------------------------------------------------------

     This program reads the SPIHIST input parameters from spihist.par file.


-------- Functions called: -----------------------------------------------------

   Common (ISDC) Library Functions:
      CommonPreparePARs  -- Performs start up processing on .par file

   Parameter Interface Library Functions:
      PILGetInt          -- Reads INTEGER input value

   SPI Functions:
      Cleanup            -- Deallocates all memory allocated during the current 
                            run in the event of an error
      GetString          -- Reads STRING input value
      ToUpper            -- Converts input string to upper case


-------- Usage: ----------------------------------------------------------------

   void SpihistGetParams(memoryPtr *pBase, char *detnums, dal_element **outGroup,
	          headerParsPtr pHeader, outputParsPtr pOutput, 
	          eventCriteriaPtr pEventcriteria, int *detSumFlag,
	          int *pntSumFlag)
			
   Output Parameters:
       PBASE         memoryPtr    : Pointer to the list that keeps track of
                                    allocated memory
       DETNUMS       char         : Ids of detectors to be processed (allowed 
                                    characters in the string are digits, '-', 
				    and ','
       OUTGROUP      dal_element  : Pointer to the output file
       DETSUMFLAG   int		   : +1 Indicates all detectors within a
       				     pointing will be summed.
       PNTSUMFLAG   int		   : +1 Indicates each detector is summed
       				     across pointings.
       COEFFDOL	    char*	   : Coeff file for converting on-board spectra
       				     to energy.			 
     
           headerParsPtr pars
	   ------------------
       INSTRUME      char         : String listing instrument/detector

           outputParsPtr Pars
	   ------------------
       FORMAT        char         : Format for the output file
       DTYPE         int          : Output data format (counts=1, rates=2)
       CLOBBER       int          : 1=Overwrite existing output file, if any
                                    0=Prompt user before overwriting existing 
				    output file
       CHATTER       int          : Chattiness flag for o/p (5 quite,10 normal,
                                    >20 silly)
				    
-------- Include files ---------------------------------------------------------

    "spihist.h"


-------- Authors/Modification History: -----------------------------------------

		   (3.0.0; Dec 02)  Added coeffDOL parameter.
		   (2.2.1; Nov 02)  Removed tstart and tstop parameters.
   Craig Gordon    (2.2.0; Sep 02)  Catch CommonPreparePARs error, rename function,
   				    add parameter checking at the end rather than
				    call afterwards from main program.
   Sandhia Bansal  (1.0.1; Dec 00)  Convert format option to uppercase.
   Sandhia Bansal  (1.0.0; Aug 00)


----------------------------------------------------------------------------- */

#include "spihist.h"


int SpihistGetParams(memoryPtr *pBase, char *detnums, dal_element **outGroup,
	       headerParsPtr pHeader, outputParsPtr pOutput, 
	       eventCriteriaPtr pEventcriteria, int *detSumFlag,
	       int *pntSumFlag, char *coeffDOL)
{


   int    status=ISDC_OK;

   char   message[DAL_MAX_STRING];
   char   *program="SpihistGetParams";
   char   *roGroup = "in-og-dol";
   char   *rwGroup = "out-og-dol";
   char   *inDOLs[]={"swg-index-dol", "gti-dol", "deadtime-dol", "ebounds-dol", NULL};
   char   *outDOLs[]={"det-spec-dol", NULL};


   
   GetString("outputformat", "output format", pBase, pOutput->format);
   ToUpper(pOutput->format);

   /* If format==ISDC, we don't need a file name, detnums are read from
      GTI file and dtype is defaulted at 1 (COUNTS - initspistr.c) */
   if (strcmp(pOutput->format, "PHA") == 0)
   {
      GetString("detnums", "detector numbers", pBase, detnums);

      status = PILGetInt("dtype", &pOutput->dtype);
      if (status != 0)
      {
	 strcpy(message, "Error while reading output data format");
	 Cleanup(status, message, *pBase, program);
      }

      GetString("outfile", "output file name", pBase, pOutput->fname);
      
      status = PILGetBool("detsummed", detSumFlag);
      status = PILGetBool("pntgsummed", pntSumFlag);
      if (status != 0)
      {
         strcpy(message, "Error while reading sum flags");
	 Cleanup(status, message, *pBase, program);
      }
   }

   status = CommonPreparePARs(roGroup, inDOLs, rwGroup, outDOLs, 1, outGroup, 
			      &pOutput->clobber, status);
   if (status != ISDC_OK)
   {
      strcpy(message, "Error returned from CommonPreparePARs");
      strcat(message,"\n\tCheck that output spectra file doesn't already exist.");
      Cleanup(status, message, *pBase, program);
   }
   
   GetString("coeffDOL", "coefficients table DOL", pBase, coeffDOL);
   
   GetString("psdtype", "PSD flag (raw/cor)", pBase, pEventcriteria->psdType);
   ToUpper(pEventcriteria->psdType);

   GetString("instrume", "instrument name", pBase, pHeader->instrume);
   ToUpper(pHeader->instrume);

   status = PILGetInt("chatter", &pOutput->chatter);
   if (status != 0)
   {
      strcpy(message, "Error while reading chatter flag");
      Cleanup(status, message, *pBase, program);
   }
   
   /* echo the input if the chatter flag is large enough */
   if (pOutput->chatter >= 20)
      EchoSpihistInput(detnums, pOutput, pEventcriteria, pHeader);

   strcpy(message, "Verifying input parameters...");
   DispMsg(Log_1, message, status, program, pOutput->chatter);

   status = CheckParams(pOutput, pEventcriteria);
   return status;
}
